@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -44,14 +44,14 @@
 static void print_node(Nodes *nodes,index_t item);
 static void print_segment(Segments *segments,index_t item);
 static void print_way(Ways *ways,index_t item);
-static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
+static void print_turn_relation(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
 
 static void print_head_osm(int coordcount,double latmin,double latmax,double lonmin,double lonmax);
 static void print_region_osm(Nodes *nodes,Segments *segments,Ways *ways,Relations *relations,
                              double latmin,double latmax,double lonmin,double lonmax,int option_no_super);
 static void print_node_osm(Nodes *nodes,index_t item);
 static void print_segment_osm(Segments *segments,index_t item,Ways *ways);
-static void print_turnrelation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
+static void print_turn_relation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes);
 static void print_tail_osm(void);
 
 static char *RFC822Date(time_t t);
@@ -138,6 +138,9 @@ int main(int argc,char** argv)
 
  if(option_visualiser)
    {
+    Highway highway;
+    Transport transport;
+
     if(coordcount!=4)
        print_usage(0,NULL,"The --visualiser option must have --latmin, --latmax, --lonmin, --lonmax.\n");
 
@@ -150,6 +153,12 @@ int main(int argc,char** argv)
        OutputSuper(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"oneway"))
        OutputOneway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
+    else if(!strncmp(option_data,"highway",7) && option_data[7]=='-' && (highway=HighwayType(option_data+8))!=Highway_None)
+       OutputHighway(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,highway);
+    else if(!strncmp(option_data,"transport",9) && option_data[9]=='-' && (transport=TransportType(option_data+10))!=Transport_None)
+       OutputTransport(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,transport);
+    else if(!strncmp(option_data,"barrier",7) && option_data[7]=='-' && (transport=TransportType(option_data+8))!=Transport_None)
+       OutputBarrier(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax,transport);
     else if(!strcmp(option_data,"turns"))
        OutputTurnRestrictions(OSMNodes,OSMSegments,OSMWays,OSMRelations,latmin,latmax,lonmin,lonmax);
     else if(!strcmp(option_data,"speed"))
@@ -239,13 +248,12 @@ int main(int argc,char** argv)
     printf("----\n");
     printf("\n");
 
-    printf("sizeof(Way)      =%9lu Bytes\n",(unsigned long)sizeof(Way));
-    printf("Number(compacted)=%9"Pindex_t"\n",OSMWays->file.number);
-    printf("Number(original) =%9"Pindex_t"\n",OSMWays->file.onumber);
+    printf("sizeof(Way)=%9lu Bytes\n",(unsigned long)sizeof(Way));
+    printf("Number     =%9"Pindex_t"\n",OSMWays->file.number);
     printf("\n");
 
     stat(ways_filename,&buf);
-    printf("Total names =%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)sizeof(Ways)-(unsigned long)OSMWays->file.number*(unsigned long)sizeof(Way));
+    printf("Total names=%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)sizeof(Ways)-(unsigned long)OSMWays->file.number*(unsigned long)sizeof(Way));
     printf("\n");
 
     printf("Included highways  : %s\n",HighwaysNameList(OSMWays->file.highways));
@@ -315,14 +323,14 @@ int main(int argc,char** argv)
        else if(!strcmp(argv[arg],"--turn-relation=all"))
          {
           for(item=0;item<OSMRelations->file.trnumber;item++)
-             print_turnrelation(OSMRelations,item,OSMSegments,OSMNodes);
+             print_turn_relation(OSMRelations,item,OSMSegments,OSMNodes);
          }
        else if(!strncmp(argv[arg],"--turn-relation=",16))
          {
           item=atoi(&argv[arg][16]);
 
           if(item>=0 && item<OSMRelations->file.trnumber)
-             print_turnrelation(OSMRelations,item,OSMSegments,OSMNodes);
+             print_turn_relation(OSMRelations,item,OSMSegments,OSMNodes);
           else
              printf("Invalid turn relation number; minimum=0, maximum=%"Pindex_t".\n",OSMRelations->file.trnumber-1);
          }
@@ -351,7 +359,7 @@ int main(int argc,char** argv)
              print_segment_osm(OSMSegments,item,OSMWays);
 
        for(item=0;item<OSMRelations->file.trnumber;item++)
-          print_turnrelation_osm(OSMRelations,item,OSMSegments,OSMNodes);
+          print_turn_relation_osm(OSMRelations,item,OSMSegments,OSMNodes);
       }
 
     print_tail_osm();
@@ -371,16 +379,16 @@ int main(int argc,char** argv)
 
 static void print_node(Nodes *nodes,index_t item)
 {
- Node *node=LookupNode(nodes,item,1);
+ Node *nodep=LookupNode(nodes,item,1);
  double latitude,longitude;
 
  GetLatLong(nodes,item,&latitude,&longitude);
 
  printf("Node %"Pindex_t"\n",item);
- printf("  firstseg=%"Pindex_t"\n",node->firstseg);
- printf("  latoffset=%d lonoffset=%d (latitude=%.6f longitude=%.6f)\n",node->latoffset,node->lonoffset,radians_to_degrees(latitude),radians_to_degrees(longitude));
- printf("  allow=%02x (%s)\n",node->allow,AllowedNameList(node->allow));
- if(IsSuperNode(node))
+ printf("  firstseg=%"Pindex_t"\n",nodep->firstseg);
+ printf("  latoffset=%d lonoffset=%d (latitude=%.6f longitude=%.6f)\n",nodep->latoffset,nodep->lonoffset,radians_to_degrees(latitude),radians_to_degrees(longitude));
+ printf("  allow=%02x (%s)\n",nodep->allow,AllowedNameList(nodep->allow));
+ if(IsSuperNode(nodep))
     printf("  Super-Node\n");
 }
 
@@ -395,20 +403,20 @@ static void print_node(Nodes *nodes,index_t item)
 
 static void print_segment(Segments *segments,index_t item)
 {
- Segment *segment=LookupSegment(segments,item,1);
+ Segment *segmentp=LookupSegment(segments,item,1);
 
  printf("Segment %"Pindex_t"\n",item);
- printf("  node1=%"Pindex_t" node2=%"Pindex_t"\n",segment->node1,segment->node2);
- printf("  next2=%"Pindex_t"\n",segment->next2);
- printf("  way=%"Pindex_t"\n",segment->way);
- printf("  distance=%d (%.3f km)\n",DISTANCE(segment->distance),distance_to_km(DISTANCE(segment->distance)));
- if(IsSuperSegment(segment) && IsNormalSegment(segment))
+ printf("  node1=%"Pindex_t" node2=%"Pindex_t"\n",segmentp->node1,segmentp->node2);
+ printf("  next2=%"Pindex_t"\n",segmentp->next2);
+ printf("  way=%"Pindex_t"\n",segmentp->way);
+ printf("  distance=%d (%.3f km)\n",DISTANCE(segmentp->distance),distance_to_km(DISTANCE(segmentp->distance)));
+ if(IsSuperSegment(segmentp) && IsNormalSegment(segmentp))
     printf("  Super-Segment AND normal Segment\n");
- else if(IsSuperSegment(segment) && !IsNormalSegment(segment))
+ else if(IsSuperSegment(segmentp) && !IsNormalSegment(segmentp))
     printf("  Super-Segment\n");
- if(IsOnewayTo(segment,segment->node1))
+ if(IsOnewayTo(segmentp,segmentp->node1))
     printf("  One-Way from node2 to node1\n");
- if(IsOnewayTo(segment,segment->node2))
+ if(IsOnewayTo(segmentp,segmentp->node2))
     printf("  One-Way from node1 to node2\n");
 }
 
@@ -423,26 +431,26 @@ static void print_segment(Segments *segments,index_t item)
 
 static void print_way(Ways *ways,index_t item)
 {
- Way *way=LookupWay(ways,item,1);
- char *name=WayName(ways,way);
+ Way *wayp=LookupWay(ways,item,1);
+ char *name=WayName(ways,wayp);
 
  printf("Way %"Pindex_t"\n",item);
  if(*name)
     printf("  name=%s\n",name);
- printf("  type=%02x (%s%s%s)\n",way->type,HighwayName(HIGHWAY(way->type)),way->type&Way_OneWay?",One-Way":"",way->type&Way_Roundabout?",Roundabout":"");
- printf("  allow=%02x (%s)\n",way->allow,AllowedNameList(way->allow));
- if(way->props)
-    printf("  props=%02x (%s)\n",way->props,PropertiesNameList(way->props));
- if(way->speed)
-    printf("  speed=%d (%d km/hr)\n",way->speed,speed_to_kph(way->speed));
- if(way->weight)
-    printf("  weight=%d (%.1f tonnes)\n",way->weight,weight_to_tonnes(way->weight));
- if(way->height)
-    printf("  height=%d (%.1f m)\n",way->height,height_to_metres(way->height));
- if(way->width)
-    printf("  width=%d (%.1f m)\n",way->width,width_to_metres(way->width));
- if(way->length)
-    printf("  length=%d (%.1f m)\n",way->length,length_to_metres(way->length));
+ printf("  type=%02x (%s%s%s)\n",wayp->type,HighwayName(HIGHWAY(wayp->type)),wayp->type&Highway_OneWay?",One-Way":"",wayp->type&Highway_Roundabout?",Roundabout":"");
+ printf("  allow=%02x (%s)\n",wayp->allow,AllowedNameList(wayp->allow));
+ if(wayp->props)
+    printf("  props=%02x (%s)\n",wayp->props,PropertiesNameList(wayp->props));
+ if(wayp->speed)
+    printf("  speed=%d (%d km/hr)\n",wayp->speed,speed_to_kph(wayp->speed));
+ if(wayp->weight)
+    printf("  weight=%d (%.1f tonnes)\n",wayp->weight,weight_to_tonnes(wayp->weight));
+ if(wayp->height)
+    printf("  height=%d (%.1f m)\n",wayp->height,height_to_metres(wayp->height));
+ if(wayp->width)
+    printf("  width=%d (%.1f m)\n",wayp->width,width_to_metres(wayp->width));
+ if(wayp->length)
+    printf("  length=%d (%.1f m)\n",wayp->length,length_to_metres(wayp->length));
 }
 
 
@@ -458,42 +466,42 @@ static void print_way(Ways *ways,index_t item)
   Nodes *nodes The set of nodes to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_turnrelation(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
+static void print_turn_relation(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
 {
- Segment *segment;
- TurnRelation *relation=LookupTurnRelation(relations,item,1);
- Node *node=LookupNode(nodes,relation->via,1);
+ Segment *segmentp;
+ TurnRelation *relationp=LookupTurnRelation(relations,item,1);
+ Node *nodep=LookupNode(nodes,relationp->via,1);
  index_t from_way=NO_WAY,to_way=NO_WAY;
  index_t from_node=NO_NODE,to_node=NO_NODE;
 
- segment=FirstSegment(segments,node,1);
+ segmentp=FirstSegment(segments,nodep,1);
 
  do
    {
-    index_t seg=IndexSegment(segments,segment);
+    index_t seg=IndexSegment(segments,segmentp);
 
-    if(seg==relation->from)
+    if(seg==relationp->from)
       {
-       from_node=OtherNode(segment,relation->from);
-       from_way=segment->way;
+       from_node=OtherNode(segmentp,relationp->from);
+       from_way=segmentp->way;
       }
 
-    if(seg==relation->to)
+    if(seg==relationp->to)
       {
-       to_node=OtherNode(segment,relation->to);
-       to_way=segment->way;
+       to_node=OtherNode(segmentp,relationp->to);
+       to_way=segmentp->way;
       }
 
-    segment=NextSegment(segments,segment,relation->via);
+    segmentp=NextSegment(segments,segmentp,relationp->via);
    }
- while(segment);
+ while(segmentp);
 
  printf("Relation %"Pindex_t"\n",item);
- printf("  from=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relation->from,from_way,from_node);
- printf("  via=%"Pindex_t" (node)\n",relation->via);
- printf("  to=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relation->to,to_way,to_node);
- if(relation->except)
-    printf("  except=%02x (%s)\n",relation->except,AllowedNameList(relation->except));
+ printf("  from=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relationp->from,from_way,from_node);
+ printf("  via=%"Pindex_t" (node)\n",relationp->via);
+ printf("  to=%"Pindex_t" (segment) = %"Pindex_t" (way) = %"Pindex_t" (node)\n",relationp->to,to_way,to_node);
+ if(relationp->except)
+    printf("  except=%02x (%s)\n",relationp->except,AllowedNameList(relationp->except));
 }
 
 
@@ -574,40 +582,40 @@ static void print_region_osm(Nodes *nodes,Segments *segments,Ways *ways,Relation
 
        for(item=index1;item<index2;item++)
          {
-          Node *node=LookupNode(nodes,item,1);
-          double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(node->latoffset));
-          double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(node->lonoffset));
+          Node *nodep=LookupNode(nodes,item,1);
+          double lat=latlong_to_radians(bin_to_latlong(nodes->file.latzero+latb)+off_to_latlong(nodep->latoffset));
+          double lon=latlong_to_radians(bin_to_latlong(nodes->file.lonzero+lonb)+off_to_latlong(nodep->lonoffset));
 
           if(lat>latmin && lat<latmax && lon>lonmin && lon<lonmax)
             {
-             Segment *segment;
+             Segment *segmentp;
 
              print_node_osm(nodes,item);
 
-             segment=FirstSegment(segments,node,1);
+             segmentp=FirstSegment(segments,nodep,1);
 
-             while(segment)
+             while(segmentp)
                {
                 double olat,olon;
-                index_t oitem=OtherNode(segment,item);
+                index_t oitem=OtherNode(segmentp,item);
 
                 GetLatLong(nodes,oitem,&olat,&olon);
 
                 if(olat>latmin && olat<latmax && olon>lonmin && olon<lonmax)
                    if(item>oitem)
-                      if(!option_no_super || IsNormalSegment(segment))
-                         print_segment_osm(segments,IndexSegment(segments,segment),ways);
+                      if(!option_no_super || IsNormalSegment(segmentp))
+                         print_segment_osm(segments,IndexSegment(segments,segmentp),ways);
 
-                segment=NextSegment(segments,segment,item);
+                segmentp=NextSegment(segments,segmentp,item);
                }
 
-             if(IsTurnRestrictedNode(node))
+             if(IsTurnRestrictedNode(nodep))
                {
                 index_t relindex=FindFirstTurnRelation1(relations,item);
 
                 while(relindex!=NO_RELATION)
                   {
-                   print_turnrelation_osm(relations,relindex,segments,nodes);
+                   print_turn_relation_osm(relations,relindex,segments,nodes);
 
                    relindex=FindNextTurnRelation1(relations,relindex);
                   }
@@ -628,32 +636,32 @@ static void print_region_osm(Nodes *nodes,Segments *segments,Ways *ways,Relation
 
 static void print_node_osm(Nodes *nodes,index_t item)
 {
- Node *node=LookupNode(nodes,item,1);
+ Node *nodep=LookupNode(nodes,item,1);
  double latitude,longitude;
  int i;
 
  GetLatLong(nodes,item,&latitude,&longitude);
 
- if(node->allow==Transports_ALL && node->flags==0)
+ if(nodep->allow==Transports_ALL && nodep->flags==0)
     printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1' />\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
  else
    {
     printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1'>\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
 
-    if(node->flags & NODE_SUPER)
+    if(nodep->flags & NODE_SUPER)
        printf("    <tag k='routino:super' v='yes' />\n");
 
-    if(node->flags & NODE_UTURN)
+    if(nodep->flags & NODE_UTURN)
        printf("    <tag k='routino:uturn' v='yes' />\n");
 
-    if(node->flags & NODE_MINIRNDBT)
+    if(nodep->flags & NODE_MINIRNDBT)
        printf("    <tag k='highway' v='mini_roundabout' />\n");
 
-    if(node->flags & NODE_TURNRSTRCT)
+    if(nodep->flags & NODE_TURNRSTRCT)
        printf("    <tag k='routino:turnrestriction' v='yes' />\n");
 
     for(i=1;i<Transport_Count;i++)
-       if(!(node->allow & TRANSPORTS(i)))
+       if(!(nodep->allow & TRANSPORTS(i)))
           printf("    <tag k='%s' v='no' />\n",TransportName(i));
 
     printf("  </node>\n");
@@ -673,61 +681,61 @@ static void print_node_osm(Nodes *nodes,index_t item)
 
 static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
 {
- Segment *segment=LookupSegment(segments,item,1);
- Way *way=LookupWay(ways,segment->way,1);
- char *name=WayName(ways,way);
+ Segment *segmentp=LookupSegment(segments,item,1);
+ Way *wayp=LookupWay(ways,segmentp->way,1);
+ char *name=WayName(ways,wayp);
  int i;
 
  printf("  <way id='%lu' version='1'>\n",(unsigned long)item+1);
 
- if(IsOnewayTo(segment,segment->node1))
+ if(IsOnewayTo(segmentp,segmentp->node1))
    {
-    printf("    <nd ref='%lu' />\n",(unsigned long)segment->node2+1);
-    printf("    <nd ref='%lu' />\n",(unsigned long)segment->node1+1);
+    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node2+1);
+    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node1+1);
    }
  else
    {
-    printf("    <nd ref='%lu' />\n",(unsigned long)segment->node1+1);
-    printf("    <nd ref='%lu' />\n",(unsigned long)segment->node2+1);
+    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node1+1);
+    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node2+1);
    }
 
- if(IsSuperSegment(segment))
+ if(IsSuperSegment(segmentp))
     printf("    <tag k='routino:super' v='yes' />\n");
- if(IsNormalSegment(segment))
+ if(IsNormalSegment(segmentp))
     printf("    <tag k='routino:normal' v='yes' />\n");
 
- printf("    <tag k='routino:distance' v='%.3f' />\n",distance_to_km(DISTANCE(segment->distance)));
+ printf("    <tag k='routino:distance' v='%.3f' />\n",distance_to_km(DISTANCE(segmentp->distance)));
 
- if(way->type & Way_OneWay)
+ if(wayp->type & Highway_OneWay)
     printf("    <tag k='oneway' v='yes' />\n");
 
- if(way->type & Way_Roundabout)
+ if(wayp->type & Highway_Roundabout)
     printf("    <tag k='roundabout' v='yes' />\n");
 
- printf("    <tag k='highway' v='%s' />\n",HighwayName(HIGHWAY(way->type)));
+ printf("    <tag k='highway' v='%s' />\n",HighwayName(HIGHWAY(wayp->type)));
 
- if(IsNormalSegment(segment) && *name)
+ if(IsNormalSegment(segmentp) && *name)
     printf("    <tag k='name' v='%s' />\n",ParseXML_Encode_Safe_XML(name));
 
  for(i=1;i<Transport_Count;i++)
-    if(way->allow & TRANSPORTS(i))
+    if(wayp->allow & TRANSPORTS(i))
        printf("    <tag k='%s' v='yes' />\n",TransportName(i));
 
  for(i=1;i<Property_Count;i++)
-    if(way->props & PROPERTIES(i))
+    if(wayp->props & PROPERTIES(i))
        printf("    <tag k='%s' v='yes' />\n",PropertyName(i));
 
- if(way->speed)
-    printf("    <tag k='maxspeed' v='%d' />\n",speed_to_kph(way->speed));
+ if(wayp->speed)
+    printf("    <tag k='maxspeed' v='%d' />\n",speed_to_kph(wayp->speed));
 
- if(way->weight)
-    printf("    <tag k='maxweight' v='%.1f' />\n",weight_to_tonnes(way->weight));
- if(way->height)
-    printf("    <tag k='maxheight' v='%.1f' />\n",height_to_metres(way->height));
- if(way->width)
-    printf("    <tag k='maxwidth' v='%.1f' />\n",width_to_metres(way->width));
- if(way->length)
-    printf("    <tag k='maxlength' v='%.1f' />\n",length_to_metres(way->length));
+ if(wayp->weight)
+    printf("    <tag k='maxweight' v='%.1f' />\n",weight_to_tonnes(wayp->weight));
+ if(wayp->height)
+    printf("    <tag k='maxheight' v='%.1f' />\n",height_to_metres(wayp->height));
+ if(wayp->width)
+    printf("    <tag k='maxwidth' v='%.1f' />\n",width_to_metres(wayp->width));
+ if(wayp->length)
+    printf("    <tag k='maxlength' v='%.1f' />\n",length_to_metres(wayp->length));
 
  printf("  </way>\n");
 }
@@ -745,14 +753,14 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
   Nodes *nodes The set of nodes to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void print_turnrelation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
+static void print_turn_relation_osm(Relations *relations,index_t item,Segments *segments,Nodes *nodes)
 {
- TurnRelation *relation=LookupTurnRelation(relations,item,1);
+ TurnRelation *relationp=LookupTurnRelation(relations,item,1);
 
- Segment *segment_from=LookupSegment(segments,relation->from,1);
- Segment *segment_to  =LookupSegment(segments,relation->to  ,2);
+ Segment *segmentp_from=LookupSegment(segments,relationp->from,1);
+ Segment *segmentp_to  =LookupSegment(segments,relationp->to  ,2);
 
- double angle=TurnAngle(nodes,segment_from,segment_to,relation->via);
+ double angle=TurnAngle(nodes,segmentp_from,segmentp_to,relationp->via);
 
  char *restriction;
 
@@ -769,12 +777,12 @@ static void print_turnrelation_osm(Relations *relations,index_t item,Segments *s
  printf("    <tag k='type' v='restriction' />\n");
  printf("    <tag k='restriction' v='%s'/>\n",restriction);
 
- if(relation->except)
-    printf("    <tag k='except' v='%s' />\n",AllowedNameList(relation->except));
+ if(relationp->except)
+    printf("    <tag k='except' v='%s' />\n",AllowedNameList(relationp->except));
 
- printf("    <member type='way' ref='%lu' role='from' />\n",(unsigned long)relation->from+1);
- printf("    <member type='node' ref='%lu' role='via' />\n",(unsigned long)relation->via+1);
- printf("    <member type='way' ref='%lu' role='to' />\n",(unsigned long)relation->to+1);
+ printf("    <member type='way' ref='%lu' role='from' />\n",(unsigned long)relationp->from+1);
+ printf("    <member type='node' ref='%lu' role='via' />\n",(unsigned long)relationp->via+1);
+ printf("    <member type='way' ref='%lu' role='to' />\n",(unsigned long)relationp->to+1);
 
  printf("  </relation>\n");
 }
@@ -855,7 +863,7 @@ static void print_usage(int detail,const char *argerr,const char *err)
          "                                --data=<data-type>]\n"
          "                  [--dump [--node=<node> ...]\n"
          "                          [--segment=<segment> ...]\n"
-         "                          [--way=<way> ...]]\n"
+         "                          [--way=<way> ...]\n"
          "                          [--turn-relation=<rel> ...]]\n"
          "                  [--dump-osm [--no-super]\n"
          "                              [--latmin=<latmin> --latmax=<latmax>\n"
@@ -889,15 +897,18 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "  --data=<data-type>      * the type of data to select.\n"
             "\n"
             "  <data-type> can be selected from:\n"
-            "      junctions = segment count at each junction.\n"
-            "      super     = super-node and super-segments.\n"
-            "      oneway    = oneway segments.\n"
-            "      turns     = turn restrictions.\n"
-            "      speed     = speed limits.\n"
-            "      weight    = weight limits.\n"
-            "      height    = height limits.\n"
-            "      width     = width limits.\n"
-            "      length    = length limits.\n"
+            "      junctions   = segment count at each junction.\n"
+            "      super       = super-node and super-segments.\n"
+            "      oneway      = oneway segments.\n"
+            "      highway-*   = segments of the specified highway type.\n"
+            "      transport-* = segments allowing the specified transport type.\n"
+            "      barrier-*   = nodes disallowing the specified transport type.\n"
+            "      turns       = turn restrictions.\n"
+            "      speed       = speed limits.\n"
+            "      weight      = weight limits.\n"
+            "      height      = height limits.\n"
+            "      width       = width limits.\n"
+            "      length      = length limits.\n"
             "\n"
             "--dump                    Dump selected contents of the database.\n"
             "  --node=<node>           * the node with the selected index.\n"
