@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -170,20 +170,20 @@ void *MapFileWriteable(const char *filename)
 
   void *UnmapFile Returns NULL (for similarity to the MapFile function).
 
-  const char *filename The name of the file when it was opened.
+  const void *address The address of the mapped file in memory.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void *UnmapFile(const char *filename)
+void *UnmapFile(const void *address)
 {
  int i;
 
  for(i=0;i<nmappedfiles;i++)
-    if(!strcmp(mappedfiles[i].filename,filename))
+    if(mappedfiles[i].address==address)
        break;
 
  if(i==nmappedfiles)
    {
-    fprintf(stderr,"The file '%s' was not mapped using MapFile().\n",filename);
+    fprintf(stderr,"The data at address %p was not mapped using MapFile().\n",address);
     exit(EXIT_FAILURE);
    }
 
@@ -372,12 +372,30 @@ int CloseFile(int fd)
 
   int DeleteFile Returns 0 if OK.
 
-  char *filename The name of the file to delete.
+  const char *filename The name of the file to delete.
   ++++++++++++++++++++++++++++++++++++++*/
 
-int DeleteFile(char *filename)
+int DeleteFile(const char *filename)
 {
  unlink(filename);
 
  return(0);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Rename a file on disk.
+
+  int RenameFile Returns 0 if OK.
+
+  const char *oldfilename The old name of the file before renaming.
+
+  const char *newfilename The new name of the file after renaming.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int RenameFile(const char *oldfilename,const char *newfilename)
+{
+ rename(oldfilename,newfilename);
+
+ return(0);
+}
