@@ -29,10 +29,11 @@
 #define HAVE_PREAD_PWRITE 1
 
 
-#include <assert.h>
 #include <unistd.h>
 #include <sys/types.h>
 
+#include "logging.h"
+
 
 /* Functions in files.c */
 
@@ -40,7 +41,7 @@ char *FileName(const char *dirname,const char *prefix, const char *name);
 
 void *MapFile(const char *filename);
 void *MapFileWriteable(const char *filename);
-void *UnmapFile(const char *filename);
+void *UnmapFile(const void *address);
 
 int OpenFileNew(const char *filename);
 int OpenFileAppend(const char *filename);
@@ -60,7 +61,9 @@ static int SeekFile(int fd,off_t position);
 
 int CloseFile(int fd);
 
-int DeleteFile(char *filename);
+int DeleteFile(const char *filename);
+
+int RenameFile(const char *oldfilename,const char *newfilename);
 
 
 /* Inline the frequently called functions */
@@ -79,7 +82,7 @@ int DeleteFile(char *filename);
 
 static inline int WriteFile(int fd,const void *address,size_t length)
 {
- assert(fd!=-1);
+ logassert(fd!=-1,"File descriptor is in error - report a bug");
 
  /* Write the data */
 
@@ -104,7 +107,7 @@ static inline int WriteFile(int fd,const void *address,size_t length)
 
 static inline int ReadFile(int fd,void *address,size_t length)
 {
- assert(fd!=-1);
+ logassert(fd!=-1,"File descriptor is in error - report a bug");
 
  /* Read the data */
 
@@ -127,7 +130,7 @@ static inline int ReadFile(int fd,void *address,size_t length)
 
 static inline int SeekFile(int fd,off_t position)
 {
- assert(fd!=-1);
+ logassert(fd!=-1,"File descriptor is in error - report a bug");
 
  /* Seek the data */
 
@@ -154,7 +157,7 @@ static inline int SeekFile(int fd,off_t position)
 
 static inline int SeekWriteFile(int fd,const void *address,size_t length,off_t position)
 {
- assert(fd!=-1);
+ logassert(fd!=-1,"File descriptor is in error - report a bug");
 
  /* Seek and write the data */
 
@@ -193,7 +196,7 @@ static inline int SeekWriteFile(int fd,const void *address,size_t length,off_t p
 
 static inline int SeekReadFile(int fd,void *address,size_t length,off_t position)
 {
- assert(fd!=-1);
+ logassert(fd!=-1,"File descriptor is in error - report a bug");
 
  /* Seek and read the data */
 
