@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "logging.h"
 
@@ -34,14 +35,22 @@
 /*+ The option to print the output in a way that allows logging to a file. +*/
 int option_loggable=0;
 
+/*+ The option to print timestamps with the output. +*/
+int option_logtime=0;
+
+
 /* Local functions */
 
 static void vfprintf_first(FILE *file,const char *format,va_list ap);
 static void vfprintf_middle(FILE *file,const char *format,va_list ap);
 static void vfprintf_last(FILE *file,const char *format,va_list ap);
 
+
 /* Local variables */
 
+/*+ The time that printf_first was called. +*/
+static struct timeval start_time;
+
 /*+ The length of the string printed out last time. +*/
 static int printed_length=0;
 
@@ -61,6 +70,9 @@ void printf_first(const char *format, ...)
 {
  va_list ap;
 
+ if(option_logtime)
+    gettimeofday(&start_time,NULL);
+
  if(option_loggable)
     return;
 
@@ -129,6 +141,9 @@ void fprintf_first(FILE *file,const char *format, ...)
 {
  va_list ap;
 
+ if(option_logtime)
+    gettimeofday(&start_time,NULL);
+
  if(option_loggable)
     return;
 
@@ -201,6 +216,9 @@ static void vfprintf_first(FILE *file,const char *format,va_list ap)
 {
  int retval;
 
+ if(option_logtime)
+    fprintf_elapsed_time(file,&start_time);
+
  retval=vfprintf(file,format,ap);
  fflush(file);
 
@@ -223,7 +241,11 @@ static void vfprintf_middle(FILE *file,const char *format,va_list ap)
 {
  int retval;
 
- putchar('\r');
+ fputc('\r',file);
+
+ if(option_logtime)
+    fprintf_elapsed_time(file,&start_time);
+
  retval=vfprintf(file,format,ap);
  fflush(file);
 
@@ -232,7 +254,7 @@ static void vfprintf_middle(FILE *file,const char *format,va_list ap)
     int new_printed_length=retval;
 
     while(retval++<printed_length)
-       putchar(' ');
+       fputc(' ',file);
 
     printed_length=new_printed_length;
    }
@@ -254,18 +276,48 @@ static void vfprintf_last(FILE *file,const char *format,va_list ap)
  int retval;
 
  if(!option_loggable)
-    putchar('\r');
+    fputc('\r',file);
+
+ if(option_logtime)
+    fprintf_elapsed_time(file,&start_time);
+
  retval=vfprintf(file,format,ap);
 
  if(retval>0)
     while(retval++<printed_length)
-       putchar(' ');
+       fputc(' ',file);
 
- putchar('\n');
+ fputc('\n',file);
  fflush(file);
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Print the elapsed time without a following newline.
+
+  FILE *file The file to print to.
+
+  struct timeval *start The start time from which the elapsed time is to be printed.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void fprintf_elapsed_time(FILE *file,struct timeval *start)
+{
+ struct timeval finish,elapsed;
+
+ gettimeofday(&finish,NULL);
+
+ elapsed.tv_sec =finish.tv_sec -start->tv_sec;
+ elapsed.tv_usec=finish.tv_usec-start->tv_usec;
+ if(elapsed.tv_usec<0)
+   {
+    elapsed.tv_sec -=1;
+    elapsed.tv_usec+=1000000;
+   }
+
+ fprintf(file,"[%2ld:%02ld.%03ld] ",elapsed.tv_sec/60,elapsed.tv_sec%60,elapsed.tv_usec/10000);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Create the error log file.
 
@@ -318,3 +370,21 @@ void logerror(const char *format, ...)
 
  va_end(ap);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Log a fatal error and exit
+
+  const char *message The error message.
+
+  const char *file The file in which the error occured.
+
+  int line The line number in the file at which the error occured.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void _logassert(const char *message,const char *file,int line)
+{
+ fprintf(stderr,"Routino Fatal Error (%s:%d): %s\n",file,line,message);
+
+ exit(EXIT_FAILURE);
+}
