@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -37,21 +36,24 @@
 #include "sorting.h"
 
 
-/* Variables */
+/* Global variables */
 
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
-/*+ A temporary file-local variable for use by the sort functions. +*/
-static NodesX *sortnodesx;
+/* Local variables */
 
-/* Functions */
+/*+ Temporary file-local variables for use by the sort functions. +*/
+static NodesX *sortnodesx;
+static latlong_t lat_min,lat_max,lon_min,lon_max;
+
+/* Local functions */
 
 static int sort_by_id(NodeX *a,NodeX *b);
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index);
 
 static int sort_by_lat_long(NodeX *a,NodeX *b);
-static int delete_pruned_and_index_by_lat_long(NodeX *nodex,index_t index);
+static int index_by_lat_long(NodeX *nodex,index_t index);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -59,36 +61,43 @@ static int delete_pruned_and_index_by_lat_long(NodeX *nodex,index_t index);
 
   NodesX *NewNodeList Returns a pointer to the node list.
 
-  int append Set to 1 if the file is to be opened for appending (now or later).
+  int append Set to 1 if the file is to be opened for appending.
+
+  int readonly Set to 1 if the file is to be opened for reading.
   ++++++++++++++++++++++++++++++++++++++*/
 
-NodesX *NewNodeList(int append)
+NodesX *NewNodeList(int append,int readonly)
 {
  NodesX *nodesx;
 
  nodesx=(NodesX*)calloc(1,sizeof(NodesX));
 
- assert(nodesx); /* Check calloc() worked */
+ logassert(nodesx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
- nodesx->filename=(char*)malloc(strlen(option_tmpdirname)+32);
+ nodesx->filename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ nodesx->filename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
+
+ sprintf(nodesx->filename    ,"%s/nodesx.parsed.mem",option_tmpdirname);
+ sprintf(nodesx->filename_tmp,"%s/nodesx.%p.tmp"    ,option_tmpdirname,(void*)nodesx);
+
+ if(append || readonly)
+    if(ExistsFile(nodesx->filename))
+      {
+       off_t size;
+
+       size=SizeFile(nodesx->filename);
+
+       nodesx->number=size/sizeof(NodeX);
+
+       RenameFile(nodesx->filename,nodesx->filename_tmp);
+      }
 
  if(append)
-    sprintf(nodesx->filename,"%s/nodesx.input.tmp",option_tmpdirname);
+    nodesx->fd=OpenFileAppend(nodesx->filename_tmp);
+ else if(!readonly)
+    nodesx->fd=OpenFileNew(nodesx->filename_tmp);
  else
-    sprintf(nodesx->filename,"%s/nodesx.%p.tmp",option_tmpdirname,(void*)nodesx);
-
- if(append)
-   {
-    off_t size;
-
-    nodesx->fd=OpenFileAppend(nodesx->filename);
-
-    size=SizeFile(nodesx->filename);
-
-    nodesx->number=size/sizeof(NodeX);
-   }
- else
-    nodesx->fd=OpenFileNew(nodesx->filename);
+    nodesx->fd=-1;
 
  return(nodesx);
 }
@@ -99,15 +108,18 @@ NodesX *NewNodeList(int append)
 
   NodesX *nodesx The set of nodes to be freed.
 
-  int keep Set to 1 if the file is to be kept (for appending later).
+  int keep If set then the results file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void FreeNodeList(NodesX *nodesx,int keep)
 {
- if(!keep)
-    DeleteFile(nodesx->filename);
+ if(keep)
+    RenameFile(nodesx->filename_tmp,nodesx->filename);
+ else
+    DeleteFile(nodesx->filename_tmp);
 
  free(nodesx->filename);
+ free(nodesx->filename_tmp);
 
  if(nodesx->idata)
     free(nodesx->idata);
@@ -115,6 +127,9 @@ void FreeNodeList(NodesX *nodesx,int keep)
  if(nodesx->gdata)
     free(nodesx->gdata);
 
+ if(nodesx->pdata)
+    free(nodesx->pdata);
+
  if(nodesx->super)
     free(nodesx->super);
 
@@ -135,10 +150,10 @@ void FreeNodeList(NodesX *nodesx,int keep)
 
   transports_t allow The allowed traffic types through the node.
 
-  uint16_t flags The flags to set for this node.
+  nodeflags_t flags The flags to set for this node.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,uint16_t flags)
+void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,nodeflags_t flags)
 {
  NodeX nodex;
 
@@ -152,7 +167,79 @@ void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transp
 
  nodesx->number++;
 
- assert(nodesx->number<NODE_FAKE); /* NODE_FAKE marks the high-water mark for real nodes. */
+ logassert(nodesx->number<NODE_FAKE,"Too many nodes (change index_t to 64-bits?)"); /* NODE_FAKE marks the high-water mark for real nodes. */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finish appending nodes and change the filename over.
+
+  NodesX *nodesx The nodes that have been appended.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void FinishNodeList(NodesX *nodesx)
+{
+ if(nodesx->fd!=-1)
+    nodesx->fd=CloseFile(nodesx->fd);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Find a particular node index.
+
+  index_t IndexNodeX Returns the index of the extended node with the specified id.
+
+  NodesX *nodesx The set of nodes to use.
+
+  node_t id The node id to look for.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+index_t IndexNodeX(NodesX *nodesx,node_t id)
+{
+ index_t start=0;
+ index_t end=nodesx->number-1;
+ index_t mid;
+
+ if(nodesx->number==0)          /* No nodes */
+    return(NO_NODE);
+
+ if(id<nodesx->idata[start])    /* Key is before start */
+    return(NO_NODE);
+
+ if(id>nodesx->idata[end])      /* Key is after end */
+    return(NO_NODE);
+
+ /* Binary search - search key exact match only is required.
+  *
+  *  # <- start  |  Check mid and move start or end if it doesn't match
+  *  #           |
+  *  #           |  Since an exact match is wanted we can set end=mid-1
+  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
+  *  #           |
+  *  #           |  Eventually either end=start or end=start+1 and one of
+  *  # <- end    |  start or end is the wanted one.
+  */
+
+ do
+   {
+    mid=(start+end)/2;             /* Choose mid point */
+
+    if(nodesx->idata[mid]<id)      /* Mid point is too low */
+       start=mid+1;
+    else if(nodesx->idata[mid]>id) /* Mid point is too high */
+       end=mid?(mid-1):mid;
+    else                           /* Mid point is correct */
+       return(mid);
+   }
+ while((end-start)>1);
+
+ if(nodesx->idata[start]==id)      /* Start is correct */
+    return(start);
+
+ if(nodesx->idata[end]==id)        /* End is correct */
+    return(end);
+
+ return(NO_NODE);
 }
 
 
@@ -171,32 +258,29 @@ void SortNodeList(NodesX *nodesx)
 
  printf_first("Sorting Nodes");
 
- /* Close the file (finished appending) */
-
- nodesx->fd=CloseFile(nodesx->fd);
-
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
- DeleteFile(nodesx->filename);
+ DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename);
+ fd=OpenFileNew(nodesx->filename_tmp);
 
  /* Allocate the array of indexes */
 
  nodesx->idata=(node_t*)malloc(nodesx->number*sizeof(node_t));
 
- assert(nodesx->idata); /* Check malloc() worked */
+ logassert(nodesx->idata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
- /* Sort by node indexes */
+ /* Sort the nodes by ID and index them */
 
  xnumber=nodesx->number;
- nodesx->number=0;
 
  sortnodesx=nodesx;
 
- nodesx->number=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_and_index_by_id);
+ nodesx->number=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),NULL,
+                                                           (int (*)(const void*,const void*))sort_by_id,
+                                                           (int (*)(void*,index_t))deduplicate_and_index_by_id);
 
  /* Close the files */
 
@@ -229,7 +313,7 @@ static int sort_by_id(NodeX *a,NodeX *b)
  else if(a_id>b_id)
     return(1);
  else
-    return(0);
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
 }
 
 
@@ -240,23 +324,173 @@ static int sort_by_id(NodeX *a,NodeX *b)
 
   NodeX *nodex The extended node.
 
-  index_t index The index of this node in the total.
+  index_t index The number of sorted nodes that have already been written to the output file.
   ++++++++++++++++++++++++++++++++++++++*/
 
 static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 {
- if(index==0 || sortnodesx->idata[index-1]!=nodex->id)
-   {
-    sortnodesx->idata[index]=nodex->id;
+ static node_t previd=NO_NODE_ID;
 
-    return(1);
+ if(nodex->id!=previd)
+   {
+    previd=nodex->id;
+
+    if(nodex->flags&NODE_DELETED)
+       return(0);
+    else
+      {
+       sortnodesx->idata[index]=nodex->id;
+
+       return(1);
+      }
    }
  else
-   {
-    logerror("Node %"Pnode_t" is duplicated\n",nodex->id);
-
     return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove any nodes that are not part of a highway.
+
+  NodesX *nodesx The set of nodes to modify.
+
+  SegmentsX *segmentsx The set of segments to use.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,int keep)
+{
+ NodeX nodex;
+ index_t total=0,highway=0,nothighway=0;
+ int fd;
+
+ /* Print the start message */
+
+ printf_first("Checking Nodes: Nodes=0");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+
+ if(keep)
+    RenameFile(nodesx->filename_tmp,nodesx->filename);
+ else
+    DeleteFile(nodesx->filename_tmp);
+
+ fd=OpenFileNew(nodesx->filename_tmp);
+
+ /* Modify the on-disk image */
+
+ while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+   {
+    if(!IsBitSet(segmentsx->usednode,total))
+       nothighway++;
+    else
+      {
+       nodex.id=highway;
+       nodesx->idata[highway]=nodesx->idata[total];
+
+       WriteFile(fd,&nodex,sizeof(NodeX));
+
+       highway++;
+      }
+
+    total++;
+
+    if(!(total%10000))
+       printf_middle("Checking Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
    }
+
+ nodesx->number=highway;
+
+ /* Close the files */
+
+ nodesx->fd=CloseFile(nodesx->fd);
+ CloseFile(fd);
+
+ /* Free the now-unneeded index */
+
+ free(segmentsx->usednode);
+ segmentsx->usednode=NULL;
+
+ /* Print the final message */
+
+ printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove any nodes that have been pruned.
+
+  NodesX *nodesx The set of nodes to prune.
+
+  SegmentsX *segmentsx The set of segments to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx)
+{
+ NodeX nodex;
+ index_t total=0,pruned=0,notpruned=0;
+ int fd;
+
+ if(nodesx->number==0)
+    return;
+
+ /* Print the start message */
+
+ printf_first("Deleting Pruned Nodes: Nodes=0 Pruned=0");
+
+ /* Allocate the array of indexes */
+
+ nodesx->pdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
+
+ logassert(nodesx->pdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+
+ /* Re-open the file read-only and a new file writeable */
+
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+
+ DeleteFile(nodesx->filename_tmp);
+
+ fd=OpenFileNew(nodesx->filename_tmp);
+
+ /* Modify the on-disk image */
+
+ while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
+   {
+    if(segmentsx->firstnode[total]==NO_SEGMENT)
+      {
+       pruned++;
+
+       nodesx->pdata[total]=NO_NODE;
+      }
+    else
+      {
+       nodex.id=notpruned;
+       nodesx->pdata[total]=notpruned;
+
+       WriteFile(fd,&nodex,sizeof(NodeX));
+
+       notpruned++;
+      }
+
+    total++;
+
+    if(!(total%10000))
+       printf_middle("Deleting Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
+   }
+
+ nodesx->number=notpruned;
+
+ /* Close the files */
+
+ nodesx->fd=CloseFile(nodesx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Deleted Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
 }
 
 
@@ -269,41 +503,70 @@ static int deduplicate_and_index_by_id(NodeX *nodex,index_t index)
 void SortNodeListGeographically(NodesX *nodesx)
 {
  int fd;
- index_t kept;
+ ll_bin_t lat_min_bin,lat_max_bin,lon_min_bin,lon_max_bin;
+
+ if(nodesx->number==0)
+    return;
+
+ /* While we are here we can work out the range of data */
+
+ lat_min=radians_to_latlong( 2);
+ lat_max=radians_to_latlong(-2);
+ lon_min=radians_to_latlong( 4);
+ lon_max=radians_to_latlong(-4);
 
  /* Print the start message */
 
- printf_first("Sorting Nodes Geographically (Deleting Pruned)");
+ printf_first("Sorting Nodes Geographically");
 
  /* Allocate the memory for the geographical index array */
 
  nodesx->gdata=(index_t*)malloc(nodesx->number*sizeof(index_t));
 
- assert(nodesx->gdata); /* Check malloc() worked */
+ logassert(nodesx->gdata,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
- DeleteFile(nodesx->filename);
+ DeleteFile(nodesx->filename_tmp);
 
- fd=OpenFileNew(nodesx->filename);
+ fd=OpenFileNew(nodesx->filename_tmp);
 
- /* Sort geographically */
+ /* Sort nodes geographically and index them */
 
  sortnodesx=nodesx;
 
- kept=filesort_fixed(nodesx->fd,fd,sizeof(NodeX),(int (*)(const void*,const void*))sort_by_lat_long,(int (*)(void*,index_t))delete_pruned_and_index_by_lat_long);
+ filesort_fixed(nodesx->fd,fd,sizeof(NodeX),NULL,
+                                            (int (*)(const void*,const void*))sort_by_lat_long,
+                                            (int (*)(void*,index_t))index_by_lat_long);
 
  /* Close the files */
 
  nodesx->fd=CloseFile(nodesx->fd);
  CloseFile(fd);
 
+ /* Free the memory */
+
+ free(nodesx->super);
+ nodesx->super=NULL;
+
+ /* Work out the number of bins */
+
+ lat_min_bin=latlong_to_bin(lat_min);
+ lon_min_bin=latlong_to_bin(lon_min);
+ lat_max_bin=latlong_to_bin(lat_max);
+ lon_max_bin=latlong_to_bin(lon_max);
+
+ nodesx->latzero=lat_min_bin;
+ nodesx->lonzero=lon_min_bin;
+
+ nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
+ nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
+
  /* Print the final message */
 
- printf_last("Sorted Nodes Geographically: Nodes=%"Pindex_t" Deleted=%"Pindex_t,kept,nodesx->number-kept);
- nodesx->number=kept;
+ printf_last("Sorted Nodes Geographically: Nodes=%"Pindex_t,nodesx->number);
 }
 
 
@@ -321,302 +584,83 @@ void SortNodeListGeographically(NodesX *nodesx)
 
 static int sort_by_lat_long(NodeX *a,NodeX *b)
 {
- int a_pruned=IsPrunedNodeX(a);
- int b_pruned=IsPrunedNodeX(b);
+ ll_bin_t a_lon=latlong_to_bin(a->longitude);
+ ll_bin_t b_lon=latlong_to_bin(b->longitude);
 
- if(a_pruned && b_pruned)
-    return(0);
- else if(a_pruned)
-    return(1);
- else if(b_pruned)
+ if(a_lon<b_lon)
     return(-1);
+ else if(a_lon>b_lon)
+    return(1);
  else
    {
-    ll_bin_t a_lon=latlong_to_bin(a->longitude);
-    ll_bin_t b_lon=latlong_to_bin(b->longitude);
+    ll_bin_t a_lat=latlong_to_bin(a->latitude);
+    ll_bin_t b_lat=latlong_to_bin(b->latitude);
 
-    if(a_lon<b_lon)
+    if(a_lat<b_lat)
        return(-1);
-    else if(a_lon>b_lon)
+    else if(a_lat>b_lat)
        return(1);
     else
       {
-       ll_bin_t a_lat=latlong_to_bin(a->latitude);
-       ll_bin_t b_lat=latlong_to_bin(b->latitude);
-
-       if(a_lat<b_lat)
+       if(a->longitude<b->longitude)
           return(-1);
-       else if(a_lat>b_lat)
+       else if(a->longitude>b->longitude)
           return(1);
        else
          {
-          if(a->longitude<b->longitude)
+          if(a->latitude<b->latitude)
              return(-1);
-          else if(a->longitude>b->longitude)
+          else if(a->latitude>b->latitude)
              return(1);
-          else
-            {
-             if(a->latitude<b->latitude)
-                return(-1);
-             else if(a->latitude>b->latitude)
-                return(1);
-            }
-
-          return(0);
          }
+
+       return(FILESORT_PRESERVE_ORDER(a,b));
       }
    }
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Delete the pruned nodes and create the index between the sorted and unsorted nodes.
+  Create the index between the sorted and unsorted nodes.
 
-  int delete_pruned_and_index_by_lat_long Return 1 if the value is to be kept, otherwise 0.
+  int index_by_lat_long Return 1 if the value is to be kept, otherwise 0.
 
   NodeX *nodex The extended node.
 
-  index_t index The index of this node in the total.
+  index_t index The number of sorted nodes that have already been written to the output file.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int delete_pruned_and_index_by_lat_long(NodeX *nodex,index_t index)
+static int index_by_lat_long(NodeX *nodex,index_t index)
 {
- if(IsPrunedNodeX(nodex))
-    return(0);
-
- /* Create the index from the previous sort to the current one */
-
  sortnodesx->gdata[nodex->id]=index;
 
+ if(IsBitSet(sortnodesx->super,nodex->id))
+    nodex->flags|=NODE_SUPER;
+
+ if(nodex->latitude<lat_min)
+    lat_min=nodex->latitude;
+ if(nodex->latitude>lat_max)
+    lat_max=nodex->latitude;
+ if(nodex->longitude<lon_min)
+    lon_min=nodex->longitude;
+ if(nodex->longitude>lon_max)
+    lon_max=nodex->longitude;
+
  return(1);
 }
 
 
-/*++++++++++++++++++++++++++++++++++++++
-  Find a particular node index.
-
-  index_t IndexNodeX Returns the index of the extended node with the specified id.
-
-  NodesX *nodesx The set of nodes to use.
-
-  node_t id The node id to look for.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-index_t IndexNodeX(NodesX *nodesx,node_t id)
-{
- index_t start=0;
- index_t end=nodesx->number-1;
- index_t mid;
-
- /* Binary search - search key exact match only is required.
-  *
-  *  # <- start  |  Check mid and move start or end if it doesn't match
-  *  #           |
-  *  #           |  Since an exact match is wanted we can set end=mid-1
-  *  # <- mid    |  or start=mid+1 because we know that mid doesn't match.
-  *  #           |
-  *  #           |  Eventually either end=start or end=start+1 and one of
-  *  # <- end    |  start or end is the wanted one.
-  */
-
- if(end<start)                        /* There are no nodes */
-    return(NO_NODE);
- else if(id<nodesx->idata[start])     /* Check key is not before start */
-    return(NO_NODE);
- else if(id>nodesx->idata[end])       /* Check key is not after end */
-    return(NO_NODE);
- else
-   {
-    do
-      {
-       mid=(start+end)/2;             /* Choose mid point */
-
-       if(nodesx->idata[mid]<id)      /* Mid point is too low */
-          start=mid+1;
-       else if(nodesx->idata[mid]>id) /* Mid point is too high */
-          end=mid?(mid-1):mid;
-       else                           /* Mid point is correct */
-          return(mid);
-      }
-    while((end-start)>1);
-
-    if(nodesx->idata[start]==id)      /* Start is correct */
-       return(start);
-
-    if(nodesx->idata[end]==id)        /* End is correct */
-       return(end);
-   }
-
- return(NO_NODE);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Remove any nodes that are not part of a highway.
-
-  NodesX *nodesx The set of nodes to modify.
-
-  SegmentsX *segmentsx The set of segments to use.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx)
-{
- NodeX nodex;
- index_t total=0,highway=0,nothighway=0;
- ll_bin_t lat_min_bin,lat_max_bin,lon_min_bin,lon_max_bin;
- latlong_t lat_min,lat_max,lon_min,lon_max;
- int fd;
-
- /* Print the start message */
-
- printf_first("Checking Nodes: Nodes=0");
-
- /* While we are here we can work out the range of data */
-
- lat_min=radians_to_latlong( 2);
- lat_max=radians_to_latlong(-2);
- lon_min=radians_to_latlong( 4);
- lon_max=radians_to_latlong(-4);
-
- /* Re-open the file read-only and a new file writeable */
-
- nodesx->fd=ReOpenFile(nodesx->filename);
-
- DeleteFile(nodesx->filename);
-
- fd=OpenFileNew(nodesx->filename);
-
- /* Modify the on-disk image */
-
- while(!ReadFile(nodesx->fd,&nodex,sizeof(NodeX)))
-   {
-    if(!IsBitSet(segmentsx->usednode,total))
-       nothighway++;
-    else
-      {
-       nodex.id=highway;
-
-       WriteFile(fd,&nodex,sizeof(NodeX));
-
-       nodesx->idata[highway]=nodesx->idata[total];
-       highway++;
-
-       if(nodex.latitude<lat_min)
-          lat_min=nodex.latitude;
-       if(nodex.latitude>lat_max)
-          lat_max=nodex.latitude;
-       if(nodex.longitude<lon_min)
-          lon_min=nodex.longitude;
-       if(nodex.longitude>lon_max)
-          lon_max=nodex.longitude;
-      }
-
-    total++;
-
-    if(!(total%10000))
-       printf_middle("Checking Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
-   }
-
- nodesx->number=highway;
-
- /* Close the files */
-
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Work out the number of bins */
-
- lat_min_bin=latlong_to_bin(lat_min);
- lon_min_bin=latlong_to_bin(lon_min);
- lat_max_bin=latlong_to_bin(lat_max);
- lon_max_bin=latlong_to_bin(lon_max);
-
- nodesx->latzero=lat_min_bin;
- nodesx->lonzero=lon_min_bin;
-
- nodesx->latbins=(lat_max_bin-lat_min_bin)+1;
- nodesx->lonbins=(lon_max_bin-lon_min_bin)+1;
-
- /* Free the now-unneeded index */
-
- free(segmentsx->usednode);
- segmentsx->usednode=NULL;
-
- /* Print the final message */
-
- printf_last("Checked Nodes: Nodes=%"Pindex_t" Highway=%"Pindex_t" not-Highway=%"Pindex_t,total,highway,nothighway);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Insert the super-node flag and the first segment indexes after geographical sorting.
-
-  NodesX *nodesx The set of nodes to modify.
-
-  SegmentsX *segmentsx The set of segments to use.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx)
-{
- index_t i;
- int fd;
-
- /* Print the start message */
-
- printf_first("Updating Super Nodes: Nodes=0");
-
- /* Re-open the file read-only and a new file writeable */
-
- nodesx->fd=ReOpenFile(nodesx->filename);
-
- DeleteFile(nodesx->filename);
-
- fd=OpenFileNew(nodesx->filename);
-
- /* Modify the on-disk image */
-
- for(i=0;i<nodesx->number;i++)
-   {
-    NodeX nodex;
-
-    ReadFile(nodesx->fd,&nodex,sizeof(NodeX));
-
-    if(IsBitSet(nodesx->super,nodex.id))
-       nodex.flags|=NODE_SUPER;
-
-    nodex.id=segmentsx->firstnode[nodesx->gdata[nodex.id]];
-
-    WriteFile(fd,&nodex,sizeof(NodeX));
-
-    if(!((i+1)%10000))
-       printf_middle("Updating Super Nodes: Nodes=%"Pindex_t,i+1);
-   }
-
- /* Close the files */
-
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Free the memory */
-
- free(nodesx->super);
- nodesx->super=NULL;
-
- /* Print the final message */
-
- printf_last("Updated Super Nodes: Nodes=%"Pindex_t,nodesx->number);
-}
-
-
 /*++++++++++++++++++++++++++++++++++++++
   Save the final node list database to a file.
 
   NodesX *nodesx The set of nodes to save.
 
   const char *filename The name of the file to save.
+
+  SegmentsX *segmentsx The set of segments to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SaveNodeList(NodesX *nodesx,const char *filename)
+void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx)
 {
  index_t i;
  int fd;
@@ -633,13 +677,13 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
 
  offsets=(index_t*)malloc((nodesx->latbins*nodesx->lonbins+1)*sizeof(index_t));
 
- assert(offsets); /* Check malloc() worked */
+ logassert(offsets,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  latlonbin=0;
 
  /* Re-open the file */
 
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 
  /* Write out the nodes data */
 
@@ -660,7 +704,7 @@ void SaveNodeList(NodesX *nodesx,const char *filename)
 
     node.latoffset=latlong_to_off(nodex.latitude);
     node.lonoffset=latlong_to_off(nodex.longitude);
-    node.firstseg=nodex.id;
+    node.firstseg=segmentsx->firstnode[nodesx->gdata[nodex.id]];
     node.allow=nodex.allow;
     node.flags=nodex.flags;
 
