@@ -45,15 +45,16 @@ struct _NodeX
  latlong_t    longitude;        /*+ The node longitude. +*/
 
  transports_t allow;            /*+ The node allowed traffic. +*/
-
- uint16_t     flags;            /*+ The node flags. +*/
+ nodeflags_t  flags;            /*+ The node flags. +*/
 };
 
 /*+ A structure containing a set of nodes (memory format). +*/
 struct _NodesX
 {
- char     *filename;            /*+ The name of the temporary file. +*/
- int       fd;                  /*+ The file descriptor of the temporary file. +*/
+ char     *filename;            /*+ The name of the intermediate file (for the NodesX). +*/
+ char     *filename_tmp;        /*+ The name of the temporary file (for the NodesX). +*/
+
+ int       fd;                  /*+ The file descriptor of the open file (for the NodesX). +*/
 
  index_t   number;              /*+ The number of extended nodes still being considered. +*/
 
@@ -70,6 +71,8 @@ struct _NodesX
 
  node_t   *idata;               /*+ The extended node IDs (sorted by ID). +*/
 
+ index_t  *pdata;               /*+ The node indexes after pruning. +*/
+
  index_t  *gdata;               /*+ The final node indexes (sorted geographically). +*/
 
  BitMask  *super;               /*+ A bit-mask marker for super nodes (same order as sorted nodes). +*/
@@ -84,30 +87,27 @@ struct _NodesX
 
 /* Functions in nodesx.c */
 
-NodesX *NewNodeList(int append);
+NodesX *NewNodeList(int append,int readonly);
 void FreeNodeList(NodesX *nodesx,int keep);
 
-void SaveNodeList(NodesX *nodesx,const char *filename);
+void AppendNodeList(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,nodeflags_t flags);
+void FinishNodeList(NodesX *nodesx);
 
 index_t IndexNodeX(NodesX *nodesx,node_t id);
 
-void AppendNode(NodesX *nodesx,node_t id,double latitude,double longitude,transports_t allow,uint16_t flags);
-
 void SortNodeList(NodesX *nodesx);
 
+void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,int keep);
+
+void RemovePrunedNodes(NodesX *nodesx,SegmentsX *segmentsx);
+
 void SortNodeListGeographically(NodesX *nodesx);
 
-void RemoveNonHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx);
-
-void UpdateNodes(NodesX *nodesx,SegmentsX *segmentsx);
+void SaveNodeList(NodesX *nodesx,const char *filename,SegmentsX *segmentsx);
 
 
 /* Macros and inline functions */
 
-/*+ Return true if this is a pruned node. +*/
-#define IsPrunedNodeX(xxx)   ((xxx)->latitude==NO_LATLONG)
-
-
 #if !SLIM
 
 #define LookupNodeX(nodesx,index,position)      &(nodesx)->data[index]
