@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,8 +48,18 @@
 #define ISFALSE(xx) (!strcmp(xx,"false") || !strcmp(xx,"no") || !strcmp(xx,"0"))
 
 
+/* Constants */
+
+#define MODE_NORMAL  3
+#define MODE_CREATE  2
+#define MODE_MODIFY  1
+#define MODE_DELETE -1
+
+
 /* Local variables */
 
+static int mode=MODE_NORMAL;
+
 static index_t nnodes=0;
 static index_t nways=0;
 static index_t nrelations=0;
@@ -91,11 +100,16 @@ static double parse_length(way_t id,const char *k,const char *v);
 
 //static int xmlDeclaration_function(const char *_tag_,int _type_,const char *version,const char *encoding);
 static int osmType_function(const char *_tag_,int _type_,const char *version);
+static int osmChangeType_function(const char *_tag_,int _type_,const char *version);
+static int deleteType_function(const char *_tag_,int _type_);
+static int createType_function(const char *_tag_,int _type_);
+static int modifyType_function(const char *_tag_,int _type_);
 static int relationType_function(const char *_tag_,int _type_,const char *id);
 static int wayType_function(const char *_tag_,int _type_,const char *id);
 static int memberType_function(const char *_tag_,int _type_,const char *type,const char *ref,const char *role);
 static int ndType_function(const char *_tag_,int _type_,const char *ref);
 static int nodeType_function(const char *_tag_,int _type_,const char *id,const char *lat,const char *lon);
+static int changesetType_function(const char *_tag_,int _type_);
 static int tagType_function(const char *_tag_,int _type_,const char *k,const char *v);
 //static int boundType_function(const char *_tag_,int _type_);
 //static int boundsType_function(const char *_tag_,int _type_);
@@ -124,6 +138,13 @@ static xmltag tagType_tag=
                tagType_function,
                {NULL}};
 
+/*+ The changesetType type tag. +*/
+static xmltag changesetType_tag=
+              {"changeset",
+               0, {NULL},
+               changesetType_function,
+               {&tagType_tag,NULL}};
+
 /*+ The nodeType type tag. +*/
 static xmltag nodeType_tag=
               {"node",
@@ -159,12 +180,40 @@ static xmltag relationType_tag=
                relationType_function,
                {&memberType_tag,&tagType_tag,NULL}};
 
+/*+ The deleteType type tag. +*/
+static xmltag deleteType_tag=
+              {"delete",
+               0, {NULL},
+               deleteType_function,
+               {&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
+
+/*+ The createType type tag. +*/
+static xmltag createType_tag=
+              {"create",
+               0, {NULL},
+               createType_function,
+               {&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
+
+/*+ The modifyType type tag. +*/
+static xmltag modifyType_tag=
+              {"modify",
+               0, {NULL},
+               modifyType_function,
+               {&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
+
+/*+ The osmChangeType type tag. +*/
+static xmltag osmChangeType_tag=
+              {"osmChange",
+               1, {"version"},
+               osmChangeType_function,
+               {&boundsType_tag,&modifyType_tag,&createType_tag,&deleteType_tag,NULL}};
+
 /*+ The osmType type tag. +*/
 static xmltag osmType_tag=
               {"osm",
                1, {"version"},
                osmType_function,
-               {&boundsType_tag,&boundType_tag,&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
+               {&boundsType_tag,&boundType_tag,&changesetType_tag,&nodeType_tag,&wayType_tag,&relationType_tag,NULL}};
 
 /*+ The xmlDeclaration type tag. +*/
 static xmltag xmlDeclaration_tag=
@@ -174,8 +223,11 @@ static xmltag xmlDeclaration_tag=
                {NULL}};
 
 
-/*+ The complete set of tags at the top level. +*/
-static xmltag *xml_toplevel_tags[]={&xmlDeclaration_tag,&osmType_tag,NULL};
+/*+ The complete set of tags at the top level for OSM. +*/
+static xmltag *xml_osm_toplevel_tags[]={&xmlDeclaration_tag,&osmType_tag,NULL};
+
+/*+ The complete set of tags at the top level for OSC. +*/
+static xmltag *xml_osc_toplevel_tags[]={&xmlDeclaration_tag,&osmChangeType_tag,NULL};
 
 
 /* The XML tag processing functions */
@@ -241,6 +293,24 @@ static int tagType_function(const char *_tag_,int _type_,const char *k,const cha
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the changesetType XSD type is seen
+
+  int changesetType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int changesetType_function(const char *_tag_,int _type_)
+{
+ current_tags=NULL;
+
+ return(0);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The function that is called when the nodeType XSD type is seen
 
@@ -277,10 +347,13 @@ static int nodeType_function(const char *_tag_,int _type_,const char *id,const c
 
     XMLPARSE_ASSERT_INTEGER(_tag_,id);   llid=atoll(id); /* need long long conversion */
     node_id=(node_t)llid;
-    assert((long long)node_id==llid);      /* check node id can be stored in node_t data type. */
+    logassert((long long)node_id==llid,"Node ID too large (change node_t to 64-bits?)"); /* check node id can be stored in node_t data type. */
 
-    XMLPARSE_ASSERT_FLOATING(_tag_,lat); latitude =atof(lat);
-    XMLPARSE_ASSERT_FLOATING(_tag_,lon); longitude=atof(lon);
+    if(mode!=MODE_DELETE)
+      {
+       XMLPARSE_ASSERT_FLOATING(_tag_,lat); latitude =atof(lat);
+       XMLPARSE_ASSERT_FLOATING(_tag_,lon); longitude=atof(lon);
+      }
    }
 
  if(_type_&XMLPARSE_TAG_END)
@@ -289,7 +362,7 @@ static int nodeType_function(const char *_tag_,int _type_,const char *id,const c
 
     process_node_tags(result,node_id,latitude,longitude);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -318,7 +391,7 @@ static int ndType_function(const char *_tag_,int _type_,const char *ref)
 
     XMLPARSE_ASSERT_INTEGER(_tag_,ref); llid=atoll(ref); /* need long long conversion */
     node_id=(node_t)llid;
-    assert((long long)node_id==llid);      /* check node id can be stored in node_t data type. */
+    logassert((long long)node_id==llid,"Node ID too large (change node_t to 64-bits?)"); /* check node id can be stored in node_t data type. */
 
     if(way_nnodes && (way_nnodes%256)==0)
        way_nodes=(node_t*)realloc((void*)way_nodes,(way_nnodes+256)*sizeof(node_t));
@@ -360,7 +433,7 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
        node_t node_id;
 
        node_id=(node_t)llid;
-       assert((long long)node_id==llid);   /* check node id can be stored in node_t data type. */
+       logassert((long long)node_id==llid,"Node ID too large (change node_t to 64-bits?)"); /* check node id can be stored in node_t data type. */
 
        if(relation_nnodes && (relation_nnodes%256)==0)
           relation_nodes=(node_t*)realloc((void*)relation_nodes,(relation_nnodes+256)*sizeof(node_t));
@@ -378,7 +451,7 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
        way_t way_id;
 
        way_id=(way_t)llid;
-       assert((long long)way_id==llid);   /* check way id can be stored in way_t data type. */
+       logassert((long long)way_id==llid,"Way ID too large (change way_t to 64-bits?)"); /* check way id can be stored in way_t data type. */
 
        if(relation_nways && (relation_nways%256)==0)
           relation_ways=(way_t*)realloc((void*)relation_ways,(relation_nways+256)*sizeof(way_t));
@@ -398,7 +471,7 @@ static int memberType_function(const char *_tag_,int _type_,const char *type,con
        relation_t relation_id;
 
        relation_id=(relation_t)llid;
-       assert((long long)relation_id==llid);   /* check relation id can be stored in relation_t data type. */
+       logassert((long long)relation_id==llid,"Relation ID too large (change relation_t to 64-bits?)"); /* check relation id can be stored in relation_t data type. */
 
        if(relation_nrelations && (relation_nrelations%256)==0)
           relation_relations=(relation_t*)realloc((void*)relation_relations,(relation_nrelations+256)*sizeof(relation_t));
@@ -445,7 +518,7 @@ static int wayType_function(const char *_tag_,int _type_,const char *id)
     XMLPARSE_ASSERT_INTEGER(_tag_,id); llid=atoll(id); /* need long long conversion */
 
     way_id=(way_t)llid;
-    assert((long long)way_id==llid);   /* check way id can be stored in way_t data type. */
+    logassert((long long)way_id==llid,"Way ID too large (change way_t to 64-bits?)"); /* check way id can be stored in way_t data type. */
    }
 
  if(_type_&XMLPARSE_TAG_END)
@@ -454,7 +527,7 @@ static int wayType_function(const char *_tag_,int _type_,const char *id)
 
     process_way_tags(result,way_id);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -500,7 +573,7 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
     XMLPARSE_ASSERT_INTEGER(_tag_,id); llid=atoll(id); /* need long long conversion */
 
     relation_id=(relation_t)llid;
-    assert((long long)relation_id==llid);   /* check relation id can be stored in relation_t data type. */
+    logassert((long long)relation_id==llid,"Relation ID too large (change relation_t to 64-bits?)"); /* check relation id can be stored in relation_t data type. */
    }
 
  if(_type_&XMLPARSE_TAG_END)
@@ -509,7 +582,7 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
 
     process_relation_tags(result,relation_id);
 
-    DeleteTagList(current_tags);
+    DeleteTagList(current_tags); current_tags=NULL;
     DeleteTagList(result);
    }
 
@@ -517,6 +590,87 @@ static int relationType_function(const char *_tag_,int _type_,const char *id)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the deleteType XSD type is seen
+
+  int deleteType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deleteType_function(const char *_tag_,int _type_)
+{
+ if(_type_&XMLPARSE_TAG_START)
+    mode=MODE_DELETE;
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the createType XSD type is seen
+
+  int createType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int createType_function(const char *_tag_,int _type_)
+{
+ if(_type_&XMLPARSE_TAG_START)
+    mode=MODE_CREATE;
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the modifyType XSD type is seen
+
+  int modifyType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int modifyType_function(const char *_tag_,int _type_)
+{
+ if(_type_&XMLPARSE_TAG_START)
+    mode=MODE_MODIFY;
+
+ return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  The function that is called when the osmChangeType XSD type is seen
+
+  int osmChangeType_function Returns 0 if no error occured or something else otherwise.
+
+  const char *_tag_ Set to the name of the element tag that triggered this function call.
+
+  int _type_ Set to XMLPARSE_TAG_START at the start of a tag and/or XMLPARSE_TAG_END at the end of a tag.
+
+  const char *version The contents of the 'version' attribute (or NULL if not defined).
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int osmChangeType_function(const char *_tag_,int _type_,const char *version)
+{
+ if(_type_&XMLPARSE_TAG_START)
+   {
+    if(!version || strcmp(version,"0.6"))
+       XMLPARSE_MESSAGE(_tag_,"Invalid value for 'version' (only '0.6' accepted)");
+   }
+
+ return(0);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The function that is called when the osmType XSD type is seen
 
@@ -533,6 +687,8 @@ static int osmType_function(const char *_tag_,int _type_,const char *version)
 {
  if(_type_&XMLPARSE_TAG_START)
    {
+    mode=MODE_NORMAL;
+
     if(!version || strcmp(version,"0.6"))
        XMLPARSE_MESSAGE(_tag_,"Invalid value for 'version' (only '0.6' accepted)");
    }
@@ -600,7 +756,60 @@ int ParseOSM(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,R
 
  printf_first("Reading: Lines=0 Nodes=0 Ways=0 Relations=0");
 
- retval=ParseXML(file,xml_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
+ retval=ParseXML(file,xml_osm_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
+
+ printf_last("Read: Lines=%llu Nodes=%"Pindex_t" Ways=%"Pindex_t" Relations=%"Pindex_t,ParseXML_LineNumber(),nnodes,nways,nrelations);
+
+ free(way_nodes);
+
+ free(relation_nodes);
+ free(relation_ways);
+ free(relation_relations);
+
+ return(retval);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Parse an OSC XML file (from planet download).
+
+  int ParseOSC Returns 0 if OK or something else in case of an error.
+
+  FILE *file The file to read from.
+
+  NodesX *OSMNodes The data structure of nodes to fill in.
+
+  SegmentsX *OSMSegments The data structure of segments to fill in.
+
+  WaysX *OSMWays The data structure of ways to fill in.
+
+  RelationsX *OSMRelations The data structure of relations to fill in.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int ParseOSC(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,RelationsX *OSMRelations)
+{
+ int retval;
+
+ /* Copy the function parameters and initialise the variables. */
+
+ nodes=OSMNodes;
+ segments=OSMSegments;
+ ways=OSMWays;
+ relations=OSMRelations;
+
+ way_nodes=(node_t*)malloc(256*sizeof(node_t));
+
+ relation_nodes    =(node_t    *)malloc(256*sizeof(node_t));
+ relation_ways     =(way_t     *)malloc(256*sizeof(way_t));
+ relation_relations=(relation_t*)malloc(256*sizeof(relation_t));
+
+ /* Parse the file */
+
+ nnodes=0,nways=0,nrelations=0;
+
+ printf_first("Reading: Lines=0 Nodes=0 Ways=0 Relations=0");
+
+ retval=ParseXML(file,xml_osc_toplevel_tags,XMLPARSE_UNKNOWN_ATTR_IGNORE);
 
  printf_last("Read: Lines=%llu Nodes=%"Pindex_t" Ways=%"Pindex_t" Relations=%"Pindex_t,ParseXML_LineNumber(),nnodes,nways,nrelations);
 
@@ -629,13 +838,23 @@ int ParseOSM(FILE *file,NodesX *OSMNodes,SegmentsX *OSMSegments,WaysX *OSMWays,R
 static void process_node_tags(TagList *tags,node_t id,double latitude,double longitude)
 {
  transports_t allow=Transports_ALL;
- uint16_t flags=0;
+ nodeflags_t flags=0;
  int i;
 
+ /* Delete */
+
+ if(mode==MODE_DELETE)
+   {
+    AppendNodeList(nodes,id,degrees_to_radians(latitude),degrees_to_radians(longitude),allow,NODE_DELETED);
+
+    return;
+   }
+
  /* Parse the tags */
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
@@ -648,6 +867,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Bicycle;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'bicycle' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -659,6 +879,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Foot;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'foot' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -670,6 +891,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Goods;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'goods' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -677,7 +899,10 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
       case 'h':
        if(!strcmp(k,"highway"))
           if(!strcmp(v,"mini_roundabout"))
+            {
              flags|=NODE_MINIRNDBT;
+             recognised=1; break;
+            }
 
        if(!strcmp(k,"horse"))
          {
@@ -685,6 +910,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Horse;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'horse' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"hgv"))
@@ -693,6 +919,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_HGV;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'hgv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -704,6 +931,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Moped;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'moped' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorbike"))
@@ -712,6 +940,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Motorbike;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'motorbike' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorcar"))
@@ -720,6 +949,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Motorcar;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'motorcar' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -731,6 +961,7 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_PSV;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'psv' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -742,18 +973,22 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
              allow&=~Transports_Wheelchair;
           else if(!ISTRUE(v))
              logerror("Node %"Pnode_t" has an unrecognised tag value 'wheelchair' = '%s' (after tagging rules); using 'yes'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       default:
-       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Node %"Pnode_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the node */
 
- AppendNode(nodes,id,degrees_to_radians(latitude),degrees_to_radians(longitude),allow,flags);
+ AppendNodeList(nodes,id,degrees_to_radians(latitude),degrees_to_radians(longitude),allow,flags);
 }
 
 
@@ -767,10 +1002,41 @@ static void process_node_tags(TagList *tags,node_t id,double latitude,double lon
 
 static void process_way_tags(TagList *tags,way_t id)
 {
- Way   way={0};
- int   oneway=0,roundabout=0,area=0;
+ Way way={0};
+ distance_t oneway=0,area=0;
+ int roundabout=0;
  char *name=NULL,*ref=NULL,*refname=NULL;
- int i;
+ int i,j;
+
+ /* Delete */
+
+ if(mode==MODE_DELETE || mode==MODE_MODIFY)
+   {
+    way.type=WAY_DELETED;
+
+    AppendWayList(ways,id,&way,"");
+
+    way.type=Highway_None;
+
+    AppendSegmentList(segments,id,NO_NODE_ID,NO_NODE_ID,0);
+   }
+
+ if(mode==MODE_DELETE)
+    return;
+
+ /* Sanity check */
+
+ if(way_nnodes==0)
+   {
+    logerror("Way %"Pway_t" has no nodes.\n",id);
+    return;
+   }
+
+ if(way_nnodes==1)
+   {
+    logerror("Way %"Pway_t" has only one node.\n",id);
+    return;
+   }
 
  /* Parse the tags - just look for highway */
 
@@ -783,20 +1049,23 @@ static void process_way_tags(TagList *tags,way_t id)
       {
        way.type=HighwayType(v);
 
-       if(way.type==Way_Count)
+       if(way.type==Highway_None)
           logerror("Way %"Pway_t" has an unrecognised highway type '%s' (after tagging rules); ignoring it.\n",id,v);
+
+       break;
       }
    }
 
  /* Don't continue if this is not a highway (bypass error logging) */
 
- if(way.type==0 || way.type==Way_Count)
+ if(way.type==Highway_None)
     return;
 
  /* Parse the tags - look for the others */
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
@@ -806,9 +1075,10 @@ static void process_way_tags(TagList *tags,way_t id)
        if(!strcmp(k,"area"))
          {
           if(ISTRUE(v))
-             area=1;
+             area=SEGMENT_AREA;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'area' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -820,6 +1090,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Bicycle;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bicycle' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"bicycleroute"))
@@ -828,6 +1099,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_BicycleRoute;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"bridge"))
@@ -836,6 +1108,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Bridge;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'bridge' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -847,6 +1120,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Foot;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'foot' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"footroute"))
@@ -855,6 +1129,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_FootRoute;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -866,13 +1141,14 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Goods;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'goods' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'h':
        if(!strcmp(k,"highway"))
-          ;
+         {recognised=1; break;}
 
        if(!strcmp(k,"horse"))
          {
@@ -880,6 +1156,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Horse;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'horse' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"hgv"))
@@ -888,25 +1165,44 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_HGV;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'hgv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'm':
-       if(!strcmp(k,"maxspeed"))
-          way.speed=kph_to_speed(parse_speed(id,k,v));
+       if(!strncmp(k,"max",3))
+         {
+          if(!strcmp(k+3,"speed"))
+            {
+             way.speed=kph_to_speed(parse_speed(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxweight"))
-          way.weight=tonnes_to_weight(parse_weight(id,k,v));
+          if(!strcmp(k+3,"weight"))
+            {
+             way.weight=tonnes_to_weight(parse_weight(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxheight"))
-          way.height=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"height"))
+            {
+             way.height=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxwidth"))
-          way.width=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"width"))
+            {
+             way.width=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
 
-       if(!strcmp(k,"maxlength"))
-          way.length=metres_to_height(parse_length(id,k,v));
+          if(!strcmp(k+3,"length"))
+            {
+             way.length=metres_to_height(parse_length(id,k,v));
+             recognised=1; break;
+            }
+         }
 
        if(!strcmp(k,"moped"))
          {
@@ -914,6 +1210,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Moped;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'moped' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorbike"))
@@ -922,6 +1219,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Motorbike;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'motorbike' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"motorcar"))
@@ -930,6 +1228,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Motorcar;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'motorcar' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"multilane"))
@@ -938,13 +1237,17 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Multilane;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'multilane' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'n':
        if(!strcmp(k,"name"))
+         {
           name=v;
+          recognised=1; break;
+         }
 
        break;
 
@@ -952,11 +1255,12 @@ static void process_way_tags(TagList *tags,way_t id)
        if(!strcmp(k,"oneway"))
          {
           if(ISTRUE(v))
-             oneway=1;
+             oneway=ONEWAY_1TO2;
           else if(!strcmp(v,"-1"))
-             oneway=-1;
+             oneway=ONEWAY_2TO1;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'oneway' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -968,6 +1272,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Paved;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'paved' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        if(!strcmp(k,"psv"))
@@ -976,13 +1281,17 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_PSV;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'psv' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       case 'r':
        if(!strcmp(k,"ref"))
+         {
           ref=v;
+          recognised=1; break;
+         }
 
        if(!strcmp(k,"roundabout"))
          {
@@ -990,6 +1299,7 @@ static void process_way_tags(TagList *tags,way_t id)
              roundabout=1;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'roundabout' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1001,6 +1311,7 @@ static void process_way_tags(TagList *tags,way_t id)
              way.props|=Properties_Tunnel;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'tunnel' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1012,13 +1323,17 @@ static void process_way_tags(TagList *tags,way_t id)
              way.allow|=Transports_Wheelchair;
           else if(!ISFALSE(v))
              logerror("Way %"Pway_t" has an unrecognised tag value 'wheelchair' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
 
       default:
-       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Way %"Pway_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the way */
@@ -1026,18 +1341,11 @@ static void process_way_tags(TagList *tags,way_t id)
  if(!way.allow)
     return;
 
- if(way_nnodes==0)
-   {
-    logerror("Way %"Pway_t" has no nodes.\n",id);
-
-    return;
-   }
-
  if(oneway)
-    way.type|=Way_OneWay;
+    way.type|=Highway_OneWay;
 
  if(roundabout)
-    way.type|=Way_Roundabout;
+    way.type|=Highway_Roundabout;
 
  if(ref && name)
    {
@@ -1051,7 +1359,7 @@ static void process_way_tags(TagList *tags,way_t id)
  else /* if(!ref && !name) */
     refname="";
 
- AppendWay(ways,id,&way,refname);
+ AppendWayList(ways,id,&way,refname);
 
  if(ref && name)
     free(refname);
@@ -1061,12 +1369,16 @@ static void process_way_tags(TagList *tags,way_t id)
     node_t from=way_nodes[i-1];
     node_t to  =way_nodes[i];
 
-    if(oneway>0)
-       AppendSegment(segments,id,from,to,area+ONEWAY_1TO2);
-    else if(oneway<0)
-       AppendSegment(segments,id,from,to,area+ONEWAY_2TO1);
-    else
-       AppendSegment(segments,id,from,to,area);
+    for(j=1;j<i;j++)
+      {
+       node_t n1=way_nodes[j-1];
+       node_t n2=way_nodes[j];
+
+       if((n1==from && n2==to) || (n2==from && n1==to))
+          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" in way %"Pway_t" is duplicated.\n",n1,n2,id);
+      }
+
+    AppendSegmentList(segments,id,from,to,area+oneway);
    }
 }
 
@@ -1087,10 +1399,35 @@ static void process_relation_tags(TagList *tags,relation_t id)
  TurnRestriction restriction=TurnRestrict_None;
  int i;
 
+ /* Delete */
+
+ if(mode==MODE_DELETE || mode==MODE_MODIFY)
+   {
+    AppendRouteRelationList(relations,id,RELATION_DELETED,
+                            relation_ways,relation_nways,
+                            relation_relations,relation_nrelations);
+
+    AppendTurnRelationList(relations,id,
+                           relation_from,relation_to,relation_via,
+                           restriction,RELATION_DELETED);
+   }
+
+ if(mode==MODE_DELETE)
+    return;
+
+ /* Sanity check */
+
+ if(relation_nnodes==0 && relation_nways==0 && relation_nrelations==0)
+   {
+    logerror("Relation %"Prelation_t" has no nodes, ways or relations.\n",id);
+    return;
+   }
+
  /* Parse the tags */
 
  for(i=0;i<tags->ntags;i++)
    {
+    int recognised=0;
     char *k=tags->k[i];
     char *v=tags->v[i];
 
@@ -1103,6 +1440,7 @@ static void process_relation_tags(TagList *tags,relation_t id)
              routes|=Transports_Bicycle;
           else if(!ISFALSE(v))
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'bicycleroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1116,6 +1454,8 @@ static void process_relation_tags(TagList *tags,relation_t id)
 
           if(except==Transports_None)
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'except' = '%s' (after tagging rules); ignoring it.\n",id,v);
+
+          recognised=1; break;
          }
 
        break;
@@ -1127,6 +1467,7 @@ static void process_relation_tags(TagList *tags,relation_t id)
              routes|=Transports_Foot;
           else if(!ISFALSE(v))
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'footroute' = '%s' (after tagging rules); using 'no'.\n",id,v);
+          recognised=1; break;
          }
 
        break;
@@ -1144,19 +1485,30 @@ static void process_relation_tags(TagList *tags,relation_t id)
 
           if(restriction==TurnRestrict_None)
              logerror("Relation %"Prelation_t" has an unrecognised tag value 'restriction' = '%s' (after tagging rules); ignoring it.\n",id,v);
+
+          recognised=1; break;
          }
 
        break;
 
       case 't':
        if(!strcmp(k,"type"))
+         {
           if(!strcmp(v,"restriction"))
              relation_turn_restriction=1;
+
+          /* Don't log an error for relations of types that we don't handle - there are so many */
+          recognised=1; break;
+         }
+
        break;
 
       default:
-       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
+       break;
       }
+
+    if(!recognised)
+       logerror("Relation %"Prelation_t" has an unrecognised tag '%s' = '%s' (after tagging rules); ignoring it.\n",id,k,v);
    }
 
  /* Create the route relation (must store all relations that have ways or
@@ -1164,9 +1516,9 @@ static void process_relation_tags(TagList *tags,relation_t id)
     other relations that are routes) */
 
  if((relation_nways || relation_nrelations) && !relation_turn_restriction)
-    AppendRouteRelation(relations,id,routes,
-                        relation_ways,relation_nways,
-                        relation_relations,relation_nrelations);
+    AppendRouteRelationList(relations,id,routes,
+                            relation_ways,relation_nways,
+                            relation_relations,relation_nrelations);
 
  /* Create the turn restriction relation. */
 
@@ -1179,9 +1531,9 @@ static void process_relation_tags(TagList *tags,relation_t id)
     else if(relation_via==NO_NODE_ID)
        logerror("Relation %"Prelation_t" is a turn restriction but has no 'via' node.\n",id);
     else
-       AppendTurnRestrictRelation(relations,id,
-                                  relation_from,relation_to,relation_via,
-                                  restriction,except);
+       AppendTurnRelationList(relations,id,
+                              relation_from,relation_to,relation_via,
+                              restriction,except);
    }
 }
 
@@ -1287,6 +1639,12 @@ static double parse_length(way_t id,const char *k,const char *v)
     if(sscanf(v,"%d' %d\"%n",&feet,&inches,&en)==2 && en && !v[en])
        return((feet+(double)inches/12.0)*0.254);
 
+    if(sscanf(v,"%d'%d\"%n",&feet,&inches,&en)==2 && en && !v[en])
+       return((feet+(double)inches/12.0)*0.254);
+
+    if(sscanf(v,"%d'-%d\"%n",&feet,&inches,&en)==2 && en && !v[en])
+       return((feet+(double)inches/12.0)*0.254);
+
     if(sscanf(v,"%d - %d%n",&feet,&inches,&en)==2 && en && !v[en])
        return((feet+(double)inches/12.0)*0.254);
 
