@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <sys/time.h>
 
 #include "types.h"
 #include "ways.h"
@@ -51,6 +52,9 @@ char *option_tmpdirname=NULL;
 /*+ The amount of RAM to use for filesorting. +*/
 size_t option_filesort_ramsize=0;
 
+/*+ The number of threads to use for filesorting. +*/
+int option_filesort_threads=1;
+
 
 /* Local functions */
 
@@ -63,6 +67,7 @@ static void print_usage(int detail,const char *argerr,const char *err);
 
 int main(int argc,char** argv)
 {
+ struct timeval start_time;
  NodesX     *Nodes;
  SegmentsX  *Segments,*SuperSegments=NULL,*MergedSegments=NULL;
  WaysX      *Ways;
@@ -71,10 +76,13 @@ int main(int argc,char** argv)
  int         max_iterations=5;
  char       *dirname=NULL,*prefix=NULL,*tagging=NULL,*errorlog=NULL;
  int         option_parse_only=0,option_process_only=0;
+ int         option_append=0,option_keep=0,option_changes=0;
  int         option_filenames=0;
  int         option_prune_isolated=500,option_prune_short=5,option_prune_straight=3;
  int         arg;
 
+ gettimeofday(&start_time,NULL);
+
  /* Parse the command line arguments */
 
  for(arg=1;arg<argc;arg++)
@@ -83,6 +91,10 @@ int main(int argc,char** argv)
        print_usage(1,NULL,NULL);
     else if(!strncmp(argv[arg],"--sort-ram-size=",16))
        option_filesort_ramsize=atoi(&argv[arg][16]);
+#if defined(USE_PTHREADS) && USE_PTHREADS
+    else if(!strncmp(argv[arg],"--sort-threads=",15))
+       option_filesort_threads=atoi(&argv[arg][15]);
+#endif
     else if(!strncmp(argv[arg],"--dir=",6))
        dirname=&argv[arg][6];
     else if(!strncmp(argv[arg],"--tmpdir=",9))
@@ -93,8 +105,16 @@ int main(int argc,char** argv)
        option_parse_only=1;
     else if(!strcmp(argv[arg],"--process-only"))
        option_process_only=1;
+    else if(!strcmp(argv[arg],"--append"))
+       option_append=1;
+    else if(!strcmp(argv[arg],"--keep"))
+       option_keep=1;
+    else if(!strcmp(argv[arg],"--changes"))
+       option_changes=1;
     else if(!strcmp(argv[arg],"--loggable"))
        option_loggable=1;
+    else if(!strcmp(argv[arg],"--logtime"))
+       option_logtime=1;
     else if(!strcmp(argv[arg],"--errorlog"))
        errorlog="error.log";
     else if(!strncmp(argv[arg],"--errorlog=",11))
@@ -127,6 +147,9 @@ int main(int argc,char** argv)
  if(option_parse_only && option_process_only)
     print_usage(0,NULL,"Cannot use '--parse-only' and '--process-only' at the same time.");
 
+ if(option_append && option_process_only)
+    print_usage(0,NULL,"Cannot use '--append' and '--process-only' at the same time.");
+
  if(option_filenames && option_process_only)
     print_usage(0,NULL,"Cannot use '--process-only' and filenames at the same time.");
 
@@ -149,84 +172,119 @@ int main(int argc,char** argv)
        option_tmpdirname=dirname;
    }
 
- if(tagging)
+ if(!option_process_only)
    {
-    if(!ExistsFile(tagging))
+    if(tagging)
       {
-       fprintf(stderr,"Error: The '--tagging' option specifies a file that does not exist.\n");
-       return(1);
+       if(!ExistsFile(tagging))
+         {
+          fprintf(stderr,"Error: The '--tagging' option specifies a file that does not exist.\n");
+          return(1);
+         }
       }
-   }
- else
-   {
-    if(ExistsFile(FileName(dirname,prefix,"tagging.xml")))
-       tagging=FileName(dirname,prefix,"tagging.xml");
-    else if(ExistsFile(FileName(DATADIR,NULL,"tagging.xml")))
-       tagging=FileName(DATADIR,NULL,"tagging.xml");
     else
       {
-       fprintf(stderr,"Error: The '--tagging' option was not used and the default 'tagging.xml' does not exist.\n");
+       if(ExistsFile(FileName(dirname,prefix,"tagging.xml")))
+          tagging=FileName(dirname,prefix,"tagging.xml");
+       else if(ExistsFile(FileName(DATADIR,NULL,"tagging.xml")))
+          tagging=FileName(DATADIR,NULL,"tagging.xml");
+       else
+         {
+          fprintf(stderr,"Error: The '--tagging' option was not used and the default 'tagging.xml' does not exist.\n");
+          return(1);
+         }
+      }
+
+    if(ParseXMLTaggingRules(tagging))
+      {
+       fprintf(stderr,"Error: Cannot read the tagging rules in the file '%s'.\n",tagging);
        return(1);
       }
    }
 
- if(ParseXMLTaggingRules(tagging))
-   {
-    fprintf(stderr,"Error: Cannot read the tagging rules in the file '%s'.\n",tagging);
-    return(1);
-   }
-
  /* Create new node, segment, way and relation variables */
 
- Nodes=NewNodeList(option_parse_only||option_process_only);
+ Nodes=NewNodeList(option_append||option_changes,option_process_only);
 
- Segments=NewSegmentList(option_parse_only||option_process_only);
+ Segments=NewSegmentList(option_append||option_changes,option_process_only);
 
- Ways=NewWayList(option_parse_only||option_process_only);
+ Ways=NewWayList(option_append||option_changes,option_process_only);
 
- Relations=NewRelationList(option_parse_only||option_process_only);
+ Relations=NewRelationList(option_append||option_changes,option_process_only);
 
  /* Create the error log file */
 
  if(errorlog)
-    open_errorlog(FileName(dirname,prefix,errorlog),option_parse_only||option_process_only);
+    open_errorlog(FileName(dirname,prefix,errorlog),option_append||option_changes||option_process_only);
 
  /* Parse the file */
 
- if(option_filenames)
-   {
-    for(arg=1;arg<argc;arg++)
-      {
-       FILE *file;
+if(!option_process_only)
+  {
+   if(option_filenames)
+     {
+      for(arg=1;arg<argc;arg++)
+        {
+         FILE *file;
 
-       if(argv[arg][0]=='-' && argv[arg][1]=='-')
-          continue;
+         if(argv[arg][0]=='-' && argv[arg][1]=='-')
+            continue;
 
-       file=fopen(argv[arg],"rb");
+         file=fopen(argv[arg],"rb");
 
-       if(!file)
-         {
-          fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
-          exit(EXIT_FAILURE);
-         }
+         if(!file)
+           {
+            fprintf(stderr,"Cannot open file '%s' for reading [%s].\n",argv[arg],strerror(errno));
+            exit(EXIT_FAILURE);
+           }
 
-       printf("\nParse OSM Data [%s]\n==============\n\n",argv[arg]);
-       fflush(stdout);
+         if(option_changes)
+           {
+            printf("\nParse OSC Data [%s]\n==============\n\n",argv[arg]);
+            fflush(stdout);
 
-       if(ParseOSM(file,Nodes,Segments,Ways,Relations))
-          exit(EXIT_FAILURE);
+            if(ParseOSC(file,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
+         else
+           {
+            printf("\nParse OSM Data [%s]\n==============\n\n",argv[arg]);
+            fflush(stdout);
 
-       fclose(file);
-      }
-   }
- else if(!option_process_only)
-   {
-    printf("\nParse OSM Data\n==============\n\n");
-    fflush(stdout);
+            if(ParseOSM(file,Nodes,Segments,Ways,Relations))
+               exit(EXIT_FAILURE);
+           }
 
-    if(ParseOSM(stdin,Nodes,Segments,Ways,Relations))
-       exit(EXIT_FAILURE);
-   }
+         fclose(file);
+        }
+     }
+   else
+     {
+      if(option_changes)
+        {
+         printf("\nParse OSC Data\n==============\n\n");
+         fflush(stdout);
+
+         if(ParseOSC(stdin,Nodes,Segments,Ways,Relations))
+            exit(EXIT_FAILURE);
+        }
+      else
+        {
+         printf("\nParse OSM Data\n==============\n\n");
+         fflush(stdout);
+
+         if(ParseOSM(stdin,Nodes,Segments,Ways,Relations))
+            exit(EXIT_FAILURE);
+        }
+     }
+
+   DeleteXMLTaggingRules();
+  }
+
+ FinishNodeList(Nodes);
+ FinishSegmentList(Segments);
+ FinishWayList(Ways);
+ FinishRelationList(Relations);
 
  if(option_parse_only)
    {
@@ -238,40 +296,47 @@ int main(int argc,char** argv)
     return(0);
    }
 
- DeleteXMLTaggingRules();
 
- /* Process the data */
+ /* Sort the data */
 
- printf("\nProcess OSM Data\n================\n\n");
+ printf("\nSort OSM Data\n=============\n\n");
  fflush(stdout);
 
  /* Sort the nodes, segments, ways and relations */
 
  SortNodeList(Nodes);
 
- SortSegmentList(Segments,0);
+ if(option_changes)
+    ApplySegmentChanges(Segments);
+
+ SortSegmentList(Segments);
 
  SortWayList(Ways);
 
  SortRelationList(Relations);
 
- /* Remove bad segments (must be after sorting the nodes and segments) */
+ /* Process the data */
 
- RemoveBadSegments(Nodes,Segments);
+ printf("\nProcess OSM Data\n================\n\n");
+ fflush(stdout);
+
+ /* Extract the way names (must be before using the ways) */
+
+ ExtractWayNames(Ways,option_keep||option_changes);
+
+ /* Remove bad segments (must be after sorting the nodes, segments and ways) */
+
+ RemoveBadSegments(Segments,Nodes,Ways,option_keep||option_changes);
 
  /* Remove non-highway nodes (must be after removing the bad segments) */
 
- RemoveNonHighwayNodes(Nodes,Segments);
+ RemoveNonHighwayNodes(Nodes,Segments,option_keep||option_changes);
 
  /* Process the route relations and first part of turn relations (must be before compacting the ways) */
 
- ProcessRouteRelations(Relations,Ways);
+ ProcessRouteRelations(Relations,Ways,option_keep||option_changes);
 
- ProcessTurnRelations1(Relations,Nodes,Ways);
-
- /* Compact the ways (must be before measuring the segments) */
-
- CompactWayList(Ways);
+ ProcessTurnRelations1(Relations,Nodes,Ways,option_keep||option_changes);
 
  /* Measure the segments and replace node/way id with index (must be after removing non-highway nodes) */
 
@@ -279,16 +344,27 @@ int main(int argc,char** argv)
 
  /* Index the segments */
 
- IndexSegments(Segments,Nodes);
+ IndexSegments(Segments,Nodes,Ways);
 
  /* Convert the turn relations from ways into nodes */
 
  ProcessTurnRelations2(Relations,Nodes,Segments,Ways);
 
+ /* Compact the ways (must be after turn relations 2) */
+
+ CompactWayList(Ways,Segments);
+
+ /* Index the segments */
+
+ IndexSegments(Segments,Nodes,Ways);
+
  /* Prune unwanted nodes/segments. */
 
  if(option_prune_straight || option_prune_isolated || option_prune_short)
    {
+    printf("\nPrune Unneeded Data\n===================\n\n");
+    fflush(stdout);
+
     StartPruning(Nodes,Segments,Ways);
 
     if(option_prune_straight)
@@ -301,6 +377,14 @@ int main(int argc,char** argv)
        PruneShortSegments(Nodes,Segments,Ways,option_prune_short);
 
     FinishPruning(Nodes,Segments,Ways);
+
+    /* Remove the pruned nodes and segments and update the indexes */
+
+    RemovePrunedNodes(Nodes,Segments);
+    RemovePrunedSegments(Segments,Ways);
+    CompactWayList(Ways,Segments);
+    RemovePrunedTurnRelations(Relations,Nodes);
+    IndexSegments(Segments,Nodes,Ways);
    }
 
  /* Repeated iteration on Super-Nodes and Super-Segments */
@@ -343,17 +427,13 @@ int main(int argc,char** argv)
        SuperSegments=SuperSegments2;
       }
 
-    /* Sort the super-segments */
+    /* Sort the super-segments and remove duplicates */
 
-    SortSegmentList(SuperSegments,0);
-
-    /* Remove duplicated super-segments */
-
-    DeduplicateSegments(SuperSegments,Nodes,Ways);
+    DeduplicateSuperSegments(SuperSegments,Ways);
 
     /* Index the segments */
 
-    IndexSegments(SuperSegments,Nodes);
+    IndexSegments(SuperSegments,Nodes,Ways);
 
     /* Check for end condition */
 
@@ -382,38 +462,28 @@ int main(int argc,char** argv)
 
  Segments=MergedSegments;
 
- /* Sort and re-index the segments */
+ /* Re-index the merged segments */
 
- SortSegmentList(Segments,0);
-
- IndexSegments(Segments,Nodes);
+ IndexSegments(Segments,Nodes,Ways);
 
  /* Cross reference the nodes and segments */
 
  printf("\nCross-Reference Nodes and Segments\n==================================\n\n");
  fflush(stdout);
 
- /* Sort the nodes geographically and update the segment indexes accordingly */
+ /* Sort the nodes and segments geographically */
 
  SortNodeListGeographically(Nodes);
 
- UpdateSegments(Segments,Nodes,Ways);
+ SortSegmentListGeographically(Segments,Nodes);
 
- /* Sort the segments geographically and re-index them */
+ /* Re-index the segments */
 
- SortSegmentList(Segments,0);
+ IndexSegments(Segments,Nodes,Ways);
 
- IndexSegments(Segments,Nodes);
+ /* Sort the turn relations geographically */
 
- /* Update the nodes */
-
- UpdateNodes(Nodes,Segments);
-
- /* Fix the turn relations after sorting nodes geographically */
-
- UpdateTurnRelations(Relations,Nodes,Segments);
-
- SortTurnRelationList(Relations);
+ SortTurnRelationListGeographically(Relations,Nodes,Segments);
 
  /* Output the results */
 
@@ -422,7 +492,7 @@ int main(int argc,char** argv)
 
  /* Write out the nodes */
 
- SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"));
+ SaveNodeList(Nodes,FileName(dirname,prefix,"nodes.mem"),Segments);
 
  FreeNodeList(Nodes,0);
 
@@ -449,6 +519,16 @@ int main(int argc,char** argv)
  if(errorlog)
     close_errorlog();
 
+ /* Print the total time */
+
+ if(option_logtime)
+   {
+    printf("\n");
+    fprintf_elapsed_time(stdout,&start_time);
+    printf("Complete\n");
+    fflush(stdout);
+   }
+
  return(0);
 }
 
@@ -468,11 +548,17 @@ static void print_usage(int detail,const char *argerr,const char *err)
  fprintf(stderr,
          "Usage: planetsplitter [--help]\n"
          "                      [--dir=<dirname>] [--prefix=<name>]\n"
+#if defined(USE_PTHREADS) && USE_PTHREADS
+         "                      [--sort-ram-size=<size>] [--sort-threads=<number>]\n"
+#else
          "                      [--sort-ram-size=<size>]\n"
+#endif
          "                      [--tmpdir=<dirname>]\n"
          "                      [--tagging=<filename>]\n"
-         "                      [--loggable] [--errorlog[=<name>]]\n"
+         "                      [--loggable] [--logtime]\n"
+         "                      [--errorlog[=<name>]]\n"
          "                      [--parse-only | --process-only]\n"
+         "                      [--append] [--keep] [--changes]\n"
          "                      [--max-iterations=<number>]\n"
          "                      [--prune-none]\n"
          "                      [--prune-isolated=<len>]\n"
@@ -504,6 +590,10 @@ static void print_usage(int detail,const char *argerr,const char *err)
 #else
             "                          (defaults to 256MB otherwise.)\n"
 #endif
+#if defined(USE_PTHREADS) && USE_PTHREADS
+            "--sort-threads=<number>   The number of threads to use for data sorting.\n"
+#endif
+            "\n"
             "--tmpdir=<dirname>        The directory name for temporary files.\n"
             "                          (defaults to the '--dir' option directory.)\n"
             "\n"
@@ -513,18 +603,22 @@ static void print_usage(int detail,const char *argerr,const char *err)
             "                           '" DATADIR "').\n"
             "\n"
             "--loggable                Print progress messages suitable for logging to file.\n"
+            "--logtime                 Print the elapsed time for each processing step.\n"
             "--errorlog[=<name>]       Log parsing errors to 'error.log' or the given name\n"
             "                          (the '--dir' and '--prefix' options are applied).\n"
             "\n"
-            "--parse-only              Parse the input OSM files and store the results.\n"
+            "--parse-only              Parse the OSM/OSC file(s) and store the results.\n"
             "--process-only            Process the stored results from previous option.\n"
+            "--append                  Parse the OSM file(s) and append to existing results.\n"
+            "--keep                    Keep the intermediate files after parsing & sorting.\n"
+            "--changes                 Parse the data as an OSC file and apply the changes.\n"
             "\n"
             "--max-iterations=<number> The number of iterations for finding super-nodes\n"
             "                          (defaults to 5).\n"
             "\n"
             "--prune-none              Disable the prune options below, they are re-enabled\n"
             "                          by adding them to the command line after this option.\n"
-            "--prune-isolated=<len>    Remove small disconnected groups of segments\n"
+            "--prune-isolated=<len>    Remove access from small disconnected segment groups\n"
             "                          (defaults to removing groups under 500m).\n"
             "--prune-short=<len>       Remove short segments (defaults to removing segments\n"
             "                          up to a maximum length of 5m).\n"
