@@ -21,7 +21,6 @@
 
 
 #include <stdlib.h>
-#include <assert.h>
 
 #include "types.h"
 #include "segments.h"
@@ -37,6 +36,11 @@
 #include "logging.h"
 
 
+/* Global variables */
+
+/*+ The command line '--tmpdir' option or its default value. +*/
+extern char *option_tmpdirname;
+
 /* Local functions */
 
 static void prune_segment(SegmentsX *segmentsx,SegmentX *segmentx);
@@ -63,6 +67,9 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
  SegmentX segmentx;
  index_t index=0,lastnode1=NO_NODE;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Adding Extra Segment Indexes: Segments=0");
@@ -71,11 +78,11 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
  segmentsx->next1=(index_t*)calloc(segmentsx->number,sizeof(index_t));
 
- assert(segmentsx->next1); /* Check malloc() worked */
+ logassert(segmentsx->next1,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  /* Open the file read-only */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 
  /* Read the on-disk image */
 
@@ -121,61 +128,10 @@ void StartPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
 void FinishPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
- index_t i,pruned=0;
- int fd;
+ if(segmentsx->next1)
+    free(segmentsx->next1);
 
-
- /* Delete the pruned segments */
-
- free(segmentsx->next1);
  segmentsx->next1=NULL;
-
- SortSegmentList(segmentsx,1);
-
- IndexSegments(segmentsx,nodesx);
-
-
- /* Delete the pruned nodes */
-
- printf_first("Marking Pruned Nodes: Nodes=0 Pruned=0");
-
- /* Re-open the file read-only and a new file writeable */
-
- nodesx->fd=ReOpenFile(nodesx->filename);
-
- DeleteFile(nodesx->filename);
-
- fd=OpenFileNew(nodesx->filename);
-
- /* Modify the on-disk image */
-
- for(i=0;i<nodesx->number;i++)
-   {
-    NodeX nodex;
-
-    ReadFile(nodesx->fd,&nodex,sizeof(NodeX));
-
-    if(segmentsx->firstnode[i]==NO_SEGMENT)
-      {
-       pruned++;
-       nodex.latitude=NO_LATLONG;
-       nodex.longitude=NO_LATLONG;
-      }
-
-    WriteFile(fd,&nodex,sizeof(NodeX));
-
-    if(!((i+1)%10000))
-       printf_middle("Marking Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,i+1,pruned);
-   }
-
- /* Close the files */
-
- nodesx->fd=CloseFile(nodesx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- printf_last("Marked Pruned Nodes: Nodes=%"Pindex_t" Pruned=%"Pindex_t,nodesx->number,pruned);
 }
 
 
@@ -194,166 +150,232 @@ void FinishPruning(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 
 void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distance_t minimum)
 {
- index_t i,j;
- index_t nregions=0,npruned=0;
+ transport_t transport;
  BitMask *connected,*region;
  index_t *regionsegments,*othersegments;
  int nallocregionsegments,nallocothersegments;
+ index_t nnewways=0;
+ int fd;
 
  if(nodesx->number==0 || segmentsx->number==0)
     return;
 
- /* Print the start message */
-
- printf_first("Pruning Isolated Regions: Segments=0 Pruned=0");
-
  /* Map into memory / open the files */
 
 #if !SLIM
- nodesx->data=MapFile(nodesx->filename);
- segmentsx->data=MapFileWriteable(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ nodesx->data=MapFile(nodesx->filename_tmp);
+ segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
 #endif
 
+ fd=ReOpenFileWriteable(waysx->filename_tmp);
+
  connected=AllocBitMask(segmentsx->number);
  region   =AllocBitMask(segmentsx->number);
 
- assert(connected); /* Check AllocBitMask() worked */
- assert(region);    /* Check AllocBitMask() worked */
+ logassert(connected,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
+ logassert(region,"Failed to allocate memory (try using slim mode?)");    /* Check AllocBitMask() worked */
 
  regionsegments=(index_t*)malloc((nallocregionsegments=1024)*sizeof(index_t));
  othersegments =(index_t*)malloc((nallocothersegments =1024)*sizeof(index_t));
 
- assert(regionsegments); /* Check malloc() worked */
- assert(othersegments);  /* Check malloc() worked */
+ logassert(regionsegments,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
+ logassert(othersegments,"Failed to allocate memory (try using slim mode?)");  /* Check malloc() worked */
 
- /* Loop through the segments and find the disconnected ones */
+ /* Loop through the transport types */
 
- for(i=0;i<segmentsx->number;i++)
+ for(transport=Transport_None+1;transport<Transport_Count;transport++)
    {
-    int nregionsegments=0,nothersegments=0;
-    distance_t total=0;
-    SegmentX *segmentx;
+    index_t i,j;
+    index_t nregions=0,npruned=0,nadjusted=0;
+    const char *transport_str=TransportName(transport);
+    transports_t transports=TRANSPORTS(transport);
 
-    segmentx=LookupSegmentX(segmentsx,i,1);
+    /* Print the start message */
 
-    if(IsPrunedSegmentX(segmentx))
-       goto endloop;
+    printf_first("Pruning Isolated Regions (%s): Segments=0 Adjusted=0 Pruned=0",transport_str);
 
-    if(IsBitSet(connected,i))
-       goto endloop;
+    /* Loop through the segments and find the disconnected ones */
 
-    othersegments[nothersegments++]=i;
-    SetBit(region,i);
+    ClearAllBits(connected,segmentsx->number);
+    ClearAllBits(region   ,segmentsx->number);
 
-    do
+    for(i=0;i<segmentsx->number;i++)
       {
-       index_t thissegment,nodes[2];
-       WayX *wayx;
+       int nregionsegments=0,nothersegments=0;
+       distance_t total=0;
+       SegmentX *segmentx;
+       WayX *wayx,tmpwayx;
 
-       thissegment=othersegments[--nothersegments];
+       if(IsBitSet(connected,i))
+          goto endloop;
 
-       if(nregionsegments==nallocregionsegments)
-          regionsegments=(index_t*)realloc(regionsegments,(nallocregionsegments+=1024)*sizeof(index_t));
+       segmentx=LookupSegmentX(segmentsx,i,1);
 
-       regionsegments[nregionsegments++]=thissegment;
+       if(IsPrunedSegmentX(segmentx))
+          goto endloop;
 
-       segmentx=LookupSegmentX(segmentsx,thissegment,1);
+       if(segmentx->way<waysx->number)
+          wayx=LookupWayX(waysx,segmentx->way,1);
+       else
+          SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
 
-       nodes[0]=segmentx->node1;
-       nodes[1]=segmentx->node2;
-       total+=DISTANCE(segmentx->distance);
+       if(!(wayx->way.allow&transports))
+          goto endloop;
 
-       wayx=LookupWayX(waysx,segmentx->way,1);
+       othersegments[nothersegments++]=i;
+       SetBit(region,i);
 
-       for(j=0;j<2;j++)
+       do
          {
-          NodeX *nodex=LookupNodeX(nodesx,nodes[j],1);
+          index_t thissegment,nodes[2];
 
-          if(!(nodex->allow&wayx->way.allow)) /* some type of traffic must be allowed */
-             continue;
+          thissegment=othersegments[--nothersegments];
 
-          segmentx=FirstSegmentX(segmentsx,nodes[j],1);
+          if(nregionsegments==nallocregionsegments)
+             regionsegments=(index_t*)realloc(regionsegments,(nallocregionsegments+=1024)*sizeof(index_t));
 
-          while(segmentx)
+          regionsegments[nregionsegments++]=thissegment;
+
+          segmentx=LookupSegmentX(segmentsx,thissegment,1);
+
+          nodes[0]=segmentx->node1;
+          nodes[1]=segmentx->node2;
+          total+=DISTANCE(segmentx->distance);
+
+          for(j=0;j<2;j++)
             {
-             index_t segment=IndexSegmentX(segmentsx,segmentx);
+             NodeX *nodex=LookupNodeX(nodesx,nodes[j],1);
 
-             if(segment!=thissegment)
+             if(!(nodex->allow&transports))
+                continue;
+
+             segmentx=FirstSegmentX(segmentsx,nodes[j],1);
+
+             while(segmentx)
                {
-                WayX *way2x=LookupWayX(waysx,segmentx->way,2);
+                index_t segment=IndexSegmentX(segmentsx,segmentx);
 
-                if(wayx->way.allow&way2x->way.allow) /* some type of traffic must be allowed */
+                if(segment!=thissegment)
                   {
-                   /* Already connected - finish */
+                   if(segmentx->way<waysx->number)
+                      wayx=LookupWayX(waysx,segmentx->way,1);
+                   else
+                      SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
 
-                   if(IsBitSet(connected,segment))
+                   if(wayx->way.allow&transports)
                      {
-                      total=minimum;
-                      goto foundconnection;
-                     }
+                      /* Already connected - finish */
 
-                   /* Not in region - add to list */
+                      if(IsBitSet(connected,segment))
+                        {
+                         total=minimum;
+                         goto foundconnection;
+                        }
 
-                   if(!IsBitSet(region,segment))
-                     {
-                      if(nothersegments==nallocothersegments)
-                         othersegments=(index_t*)realloc(othersegments,(nallocothersegments+=1024)*sizeof(index_t));
+                      /* Not in region - add to list */
 
-                      othersegments[nothersegments++]=segment;
-                      SetBit(region,segment);
+                      if(!IsBitSet(region,segment))
+                        {
+                         if(nothersegments==nallocothersegments)
+                            othersegments=(index_t*)realloc(othersegments,(nallocothersegments+=1024)*sizeof(index_t));
+
+                         othersegments[nothersegments++]=segment;
+                         SetBit(region,segment);
+                        }
                      }
                   }
-               }
 
-             segmentx=NextSegmentX(segmentsx,segmentx,nodes[j]);
+                segmentx=NextSegmentX(segmentsx,segmentx,nodes[j]);
+               }
             }
          }
-      }
-    while(nothersegments>0 && total<minimum);
+       while(nothersegments>0 && total<minimum);
 
-   foundconnection:
+      foundconnection:
 
-    /* Prune the segments or mark them as connected */
+       /* Prune the segments or mark them as connected */
 
-    if(total<minimum)
-      {
-       nregions++;
-
-       for(j=0;j<nregionsegments;j++)
+       if(total<minimum)        /* not connected - delete them */
          {
-          SegmentX *segmentx=LookupSegmentX(segmentsx,regionsegments[j],1);
+          nregions++;
 
-          SetBit(connected,regionsegments[j]);
+          for(j=0;j<nregionsegments;j++)
+            {
+             SegmentX *segmentx;
+             WayX *wayx,tmpwayx;
 
-          prune_segment(segmentsx,segmentx);
+             SetBit(connected,regionsegments[j]); /* not really connected, but don't need to check again */
+             ClearBit(region,regionsegments[j]);
 
-          npruned++;
+             segmentx=LookupSegmentX(segmentsx,regionsegments[j],1);
+
+             if(segmentx->way<waysx->number)
+                wayx=LookupWayX(waysx,segmentx->way,1);
+             else
+                SeekReadFile(fd,(wayx=&tmpwayx),sizeof(WayX),segmentx->way*sizeof(WayX));
+
+             if(wayx->way.allow==transports)
+               {
+                prune_segment(segmentsx,segmentx);
+
+                npruned++;
+               }
+             else
+               {
+                if(segmentx->way<waysx->number) /* create a new way */
+                  {
+                   tmpwayx=*wayx;
+
+                   tmpwayx.way.allow&=~transports;
+
+                   segmentx->way=waysx->number+nnewways;
+
+                   SeekWriteFile(fd,&tmpwayx,sizeof(WayX),segmentx->way*sizeof(WayX));
+
+                   nnewways++;
+
+                   PutBackSegmentX(segmentsx,segmentx);
+                  }
+                else            /* modify the existing one */
+                  {
+                   tmpwayx.way.allow&=~transports;
+
+                   SeekWriteFile(fd,&tmpwayx,sizeof(WayX),segmentx->way*sizeof(WayX));
+                  }
+
+                nadjusted++;
+               }
+            }
          }
-      }
-    else
-      {
-       for(j=0;j<nregionsegments;j++)
+       else                     /* connected - mark as part of the main region */
          {
-          SetBit(connected,regionsegments[j]);
-          ClearBit(region,regionsegments[j]);
+          for(j=0;j<nregionsegments;j++)
+            {
+             SetBit(connected,regionsegments[j]);
+             ClearBit(region,regionsegments[j]);
+            }
+
+          for(j=0;j<nothersegments;j++)
+            {
+             SetBit(connected,othersegments[j]);
+             ClearBit(region,othersegments[j]);
+            }
          }
 
-       for(j=0;j<nothersegments;j++)
-         {
-          SetBit(connected,othersegments[j]);
-          ClearBit(region,othersegments[j]);
-         }
+      endloop:
+
+       if(!((i+1)%10000))
+          printf_middle("Pruning Isolated Regions (%s): Segments=%"Pindex_t" Adjusted=%"Pindex_t" Pruned=%"Pindex_t" (%"Pindex_t" Regions)",transport_str,i+1,nadjusted,npruned,nregions);
       }
 
-   endloop:
+    /* Print the final message */
 
-    if(!((i+1)%10000))
-       printf_middle("Pruning Isolated Regions: Segments=%"Pindex_t" Pruned=%"Pindex_t" (%"Pindex_t" Regions)",i+1,npruned,nregions);
+    printf_last("Pruned Isolated Regions (%s): Segments=%"Pindex_t" Adjusted=%"Pindex_t" Pruned=%"Pindex_t" (%"Pindex_t" Regions)",transport_str,segmentsx->number,nadjusted,npruned,nregions);
    }
 
  /* Unmap from memory / close the files */
@@ -365,18 +387,18 @@ void PruneIsolatedRegions(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,dista
  free(othersegments);
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
  nodesx->fd=CloseFile(nodesx->fd);
  segmentsx->fd=CloseFile(segmentsx->fd);
  waysx->fd=CloseFile(waysx->fd);
 #endif
 
- /* Print the final message */
+ CloseFile(fd);
 
- printf_last("Pruned Isolated Regions: Segments=%"Pindex_t" Pruned=%"Pindex_t" (%"Pindex_t" Regions)",segmentsx->number,npruned,nregions);
+ waysx->number+=nnewways;
 }
 
 
@@ -407,13 +429,13 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
  /* Map into memory / open the files */
 
 #if !SLIM
- nodesx->data=MapFileWriteable(nodesx->filename);
- segmentsx->data=MapFileWriteable(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ nodesx->data=MapFileWriteable(nodesx->filename_tmp);
+ segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFileWriteable(nodesx->filename);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ nodesx->fd=ReOpenFileWriteable(nodesx->filename_tmp);
+ segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
 #endif
 
  /* Loop through the segments and find the short ones for possible modification */
@@ -498,6 +520,8 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
                 node1=OtherNode(segmentx,node2);
                }
             }
+          else if(segcount2>2)
+             break;
 
           segmentx=NextSegmentX(segmentsx,segmentx,node2);
          }
@@ -518,6 +542,8 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
                 node4=OtherNode(segmentx,node3);
                }
             }
+          else if(segcount3>2)
+             break;
 
           segmentx=NextSegmentX(segmentsx,segmentx,node3);
          }
@@ -630,32 +656,39 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
           segmentx1->distance+=DISTANCE(segmentx2->distance)/2;
           segmentx3->distance+=DISTANCE(segmentx2->distance)-DISTANCE(segmentx2->distance)/2;
 
-          PutBackSegmentX(segmentsx,segmentx1);
-          PutBackSegmentX(segmentsx,segmentx3);
-
-          prune_segment(segmentsx,segmentx2);
-
           if(segmentx1->node1==node1)
             {
              if(segmentx1->node2!=newnode)
                 modify_segment(segmentsx,segmentx1,node1,newnode);
+             else
+                PutBackSegmentX(segmentsx,segmentx1);
             }
           else /* if(segmentx1->node2==node1) */
             {
              if(segmentx1->node1!=newnode)
                 modify_segment(segmentsx,segmentx1,newnode,node1);
+             else
+                PutBackSegmentX(segmentsx,segmentx1);
             }
 
           if(segmentx3->node1==node4)
             {
              if(segmentx3->node2!=newnode)
                 modify_segment(segmentsx,segmentx3,node4,newnode);
+             else
+                PutBackSegmentX(segmentsx,segmentx3);
             }
           else /* if(segmentx3->node2==node4) */
             {
              if(segmentx3->node1!=newnode)
                 modify_segment(segmentsx,segmentx3,newnode,node4);
+             else
+                PutBackSegmentX(segmentsx,segmentx3);
             }
+
+          ReLookupSegmentX(segmentsx,segmentx2);
+
+          prune_segment(segmentsx,segmentx2);
          }
        else                     /* third case in diagram - prune one segment */
          {
@@ -724,14 +757,14 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
 
           segmentx1->distance+=DISTANCE(segmentx2->distance);
 
-          PutBackSegmentX(segmentsx,segmentx1);
-
-          prune_segment(segmentsx,segmentx2);
-
           if(segmentx1->node1==node1)
              modify_segment(segmentsx,segmentx1,node1,node3);
           else /* if(segmentx1->node2==node1) */
              modify_segment(segmentsx,segmentx1,node3,node1);
+
+          ReLookupSegmentX(segmentsx,segmentx2);
+
+          prune_segment(segmentsx,segmentx2);
          }
 
        npruned++;
@@ -746,9 +779,9 @@ void PruneShortSegments(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,distanc
  /* Unmap from memory / close the files */
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
  nodesx->fd=CloseFile(nodesx->fd);
  segmentsx->fd=CloseFile(segmentsx->fd);
@@ -795,30 +828,30 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  /* Map into memory / open the files */
 
 #if !SLIM
- nodesx->data=MapFile(nodesx->filename);
- segmentsx->data=MapFileWriteable(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ nodesx->data=MapFile(nodesx->filename_tmp);
+ segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename);
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
+ segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
 #endif
 
  checked=AllocBitMask(nodesx->number);
 
- assert(checked); /* Check AllocBitMask() worked */
+ logassert(checked,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
  nodes   =(index_t*)malloc((nalloc=1024)*sizeof(index_t));
  segments=(index_t*)malloc( nalloc      *sizeof(index_t));
 
- assert(nodes);    /* Check malloc() worked */
- assert(segments); /* Check malloc() worked */
+ logassert(nodes,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
+ logassert(segments,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  lats=(double*)malloc(nalloc*sizeof(double));
  lons=(double*)malloc(nalloc*sizeof(double));
 
- assert(lats);    /* Check malloc() worked */
- assert(lons);    /* Check malloc() worked */
+ logassert(lats,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
+ logassert(lons,"Failed to allocate memory (try using slim mode?)");    /* Check malloc() worked */
 
  /* Loop through the nodes and find stretchs of simple highway for possible modification */
 
@@ -845,7 +878,22 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
        int segcount=0;
        NodeX *nodex;
 
-       if(!IsBitSet(checked,nodes[current]))
+       /* Get the node data */
+
+       nodex=LookupNodeX(nodesx,nodes[current],1);
+
+       lats[current]=latlong_to_radians(nodex->latitude);
+       lons[current]=latlong_to_radians(nodex->longitude);
+
+       /* Count the segments at the node if not forced to be an end node */
+
+       if(IsBitSet(checked,nodes[current]))
+          ;
+       else if(nodex->flags&NODE_MINIRNDBT)
+          ;
+       else if(nodex->flags&NODE_TURNRSTRCT2 || nodex->flags&NODE_TURNRSTRCT)
+          ;
+       else
          {
           SegmentX *segmentx;
 
@@ -869,26 +917,13 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
                 node2=OtherNode(segmentx,nodes[current]);
                 way2=segmentx->way;
                }
+             else
+                break;
 
              segmentx=NextSegmentX(segmentsx,segmentx,nodes[current]);
             }
          }
 
-       /* Perform more checks */
-
-       nodex=LookupNodeX(nodesx,nodes[current],1);
-
-       lats[current]=latlong_to_radians(nodex->latitude);
-       lons[current]=latlong_to_radians(nodex->longitude);
-
-       /* Check if allowed due to mini-roundabout and turn restriction */
-
-       if(nodex->flags&NODE_MINIRNDBT)
-          segcount=0;
-
-       if(nodex->flags&NODE_TURNRSTRCT2 || nodex->flags&NODE_TURNRSTRCT)
-          segcount=0;
-
        /* Check if allowed due to one-way properties */
 
        if(segcount==2)
@@ -949,11 +984,11 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 
           if(lower==0)     /* move everything up by one */
             {
-             memmove(nodes+1   ,nodes   ,(upper-lower)*sizeof(index_t));
-             memmove(segments+1,segments,(upper-lower)*sizeof(index_t));
+             memmove(nodes+1   ,nodes   ,(1+upper-lower)*sizeof(index_t));
+             memmove(segments+1,segments,(1+upper-lower)*sizeof(index_t));
 
-             memmove(lats+1,lats,(upper-lower)*sizeof(double));
-             memmove(lons+1,lons,(upper-lower)*sizeof(double));
+             memmove(lats+1,lats,(1+upper-lower)*sizeof(double));
+             memmove(lons+1,lons,(1+upper-lower)*sizeof(double));
 
              current++;
              lower++;
@@ -1114,8 +1149,6 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
                {
                 segmentx->distance+=distance;
 
-                PutBackSegmentX(segmentsx,segmentx);
-
                 if(segmentx->node1==nodes[lower])
                    modify_segment(segmentsx,segmentx,nodes[lower],nodes[current]);
                 else /* if(segmentx->node2==nodes[lower]) */
@@ -1145,9 +1178,9 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
  free(lons);
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
  nodesx->fd=CloseFile(nodesx->fd);
  segmentsx->fd=CloseFile(segmentsx->fd);
@@ -1171,6 +1204,7 @@ void PruneStraightHighwayNodes(NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx,
 static void prune_segment(SegmentsX *segmentsx,SegmentX *segmentx)
 {
  unlink_segment_node1_refs(segmentsx,segmentx);
+
  unlink_segment_node2_refs(segmentsx,segmentx);
 
  segmentx->node1=NO_NODE;
@@ -1204,8 +1238,6 @@ static void modify_segment(SegmentsX *segmentsx,SegmentX *segmentx,index_t newno
     if(segmentx->distance&(ONEWAY_2TO1|ONEWAY_1TO2))
        segmentx->distance^=ONEWAY_2TO1|ONEWAY_1TO2;
 
-    PutBackSegmentX(segmentsx,segmentx);
-
     temp=newnode1;
     newnode1=newnode2;
     newnode2=temp;
@@ -1223,8 +1255,6 @@ static void modify_segment(SegmentsX *segmentsx,SegmentX *segmentx,index_t newno
 
     segmentsx->next1[thissegment]=segmentsx->firstnode[newnode1];
     segmentsx->firstnode[newnode1]=thissegment;
-
-    PutBackSegmentX(segmentsx,segmentx);
    }
 
  if(newnode2!=segmentx->node2) /* only modify it if the node has changed */
@@ -1233,9 +1263,9 @@ static void modify_segment(SegmentsX *segmentsx,SegmentX *segmentx,index_t newno
 
     segmentx->next2=segmentsx->firstnode[newnode2];
     segmentsx->firstnode[newnode2]=thissegment;
-
-    PutBackSegmentX(segmentsx,segmentx);
    }
+
+ PutBackSegmentX(segmentsx,segmentx);
 }
 
 
