@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -38,91 +37,115 @@
 #include "sorting.h"
 
 
-/* Local functions */
-
-static int sort_by_id(TurnRestrictRelX *a,TurnRestrictRelX *b);
-static int deduplicate_by_id(TurnRestrictRelX *relationx,index_t index);
-
-static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b);
-
-
-/* Variables */
+/* Global variables */
 
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
+/* Local variables */
+
+/*+ Temporary file-local variables for use by the sort functions. +*/
+static SegmentsX *sortsegmentsx;
+static NodesX *sortnodesx;
+
+/* Local functions */
+
+static int sort_route_by_id(RouteRelX *a,RouteRelX *b);
+static int deduplicate_route_by_id(RouteRelX *relationx,index_t index);
+
+static int sort_turn_by_id(TurnRelX *a,TurnRelX *b);
+static int deduplicate_turn_by_id(TurnRelX *relationx,index_t index);
+
+static int geographically_index(TurnRelX *relationx,index_t index);
+static int sort_by_via(TurnRelX *a,TurnRelX *b);
+
 
 /*++++++++++++++++++++++++++++++++++++++
   Allocate a new relation list (create a new file or open an existing one).
 
   RelationsX *NewRelationList Returns the relation list.
 
-  int append Set to 1 if the file is to be opened for appending (now or later).
+  int append Set to 1 if the file is to be opened for appending.
+
+  int readonly Set to 1 if the file is to be opened for reading.
   ++++++++++++++++++++++++++++++++++++++*/
 
-RelationsX *NewRelationList(int append)
+RelationsX *NewRelationList(int append,int readonly)
 {
  RelationsX *relationsx;
 
  relationsx=(RelationsX*)calloc(1,sizeof(RelationsX));
 
- assert(relationsx); /* Check calloc() worked */
+ logassert(relationsx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
 
  /* Route Relations */
 
- relationsx->rfilename=(char*)malloc(strlen(option_tmpdirname)+32);
+ relationsx->rfilename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ relationsx->rfilename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
 
- if(append)
-    sprintf(relationsx->rfilename,"%s/relationsx.route.input.tmp",option_tmpdirname);
- else
-    sprintf(relationsx->rfilename,"%s/relationsx.route.%p.tmp",option_tmpdirname,(void*)relationsx);
+ sprintf(relationsx->rfilename    ,"%s/relationsx.route.parsed.mem",option_tmpdirname);
+ sprintf(relationsx->rfilename_tmp,"%s/relationsx.route.%p.tmp"    ,option_tmpdirname,(void*)relationsx);
 
- if(append)
-   {
-    off_t size,position=0;
-
-    relationsx->rfd=OpenFileAppend(relationsx->rfilename);
-
-    size=SizeFile(relationsx->rfilename);
-
-    while(position<size)
+ if(append || readonly)
+    if(ExistsFile(relationsx->rfilename))
       {
-       FILESORT_VARINT relationsize;
+       off_t size,position=0;
+       int rfd;
 
-       SeekReadFile(relationsx->rfd,&relationsize,FILESORT_VARSIZE,position);
+       size=SizeFile(relationsx->rfilename);
 
-       relationsx->rnumber++;
-       position+=relationsize+FILESORT_VARSIZE;
+       rfd=ReOpenFile(relationsx->rfilename);
+
+       while(position<size)
+         {
+          FILESORT_VARINT relationsize;
+
+          SeekReadFile(rfd,&relationsize,FILESORT_VARSIZE,position);
+
+          relationsx->rnumber++;
+          position+=relationsize+FILESORT_VARSIZE;
+         }
+
+       CloseFile(rfd);
+
+       RenameFile(relationsx->rfilename ,relationsx->rfilename_tmp);
       }
 
-    SeekFile(relationsx->rfd,size);
-   }
+ if(append)
+    relationsx->rfd=OpenFileAppend(relationsx->rfilename_tmp);
+ else if(!readonly)
+    relationsx->rfd=OpenFileNew(relationsx->rfilename_tmp);
  else
-    relationsx->rfd=OpenFileNew(relationsx->rfilename);
+    relationsx->rfd=-1;
 
 
  /* Turn Restriction Relations */
 
- relationsx->trfilename=(char*)malloc(strlen(option_tmpdirname)+32);
+ relationsx->trfilename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ relationsx->trfilename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
+
+ sprintf(relationsx->trfilename    ,"%s/relationsx.turn.parsed.mem",option_tmpdirname);
+ sprintf(relationsx->trfilename_tmp,"%s/relationsx.turn.%p.tmp"    ,option_tmpdirname,(void*)relationsx);
+
+ if(append || readonly)
+    if(ExistsFile(relationsx->trfilename))
+      {
+       off_t size;
+
+       size=SizeFile(relationsx->trfilename);
+
+       relationsx->trnumber=size/sizeof(TurnRelX);
+
+       RenameFile(relationsx->trfilename,relationsx->trfilename_tmp);
+      }
 
  if(append)
-    sprintf(relationsx->trfilename,"%s/relationsx.turn.input.tmp",option_tmpdirname);
+    relationsx->trfd=OpenFileAppend(relationsx->trfilename_tmp);
+ else if(!readonly)
+    relationsx->trfd=OpenFileNew(relationsx->trfilename_tmp);
  else
-    sprintf(relationsx->trfilename,"%s/relationsx.turn.%p.tmp",option_tmpdirname,(void*)relationsx);
-
- if(append)
-   {
-    off_t size;
-
-    relationsx->trfd=OpenFileAppend(relationsx->trfilename);
-
-    size=SizeFile(relationsx->trfilename);
-
-    relationsx->trnumber=size/sizeof(TurnRestrictRelX);
-   }
- else
-    relationsx->trfd=OpenFileNew(relationsx->trfilename);
+    relationsx->trfd=-1;
 
  return(relationsx);
 }
@@ -133,25 +156,31 @@ RelationsX *NewRelationList(int append)
 
   RelationsX *relationsx The set of relations to be freed.
 
-  int keep Set to 1 if the file is to be kept (for appending later).
+  int keep If set then the results file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void FreeRelationList(RelationsX *relationsx,int keep)
 {
  /* Route relations */
 
- if(!keep)
-    DeleteFile(relationsx->rfilename);
+ if(keep)
+    RenameFile(relationsx->rfilename_tmp,relationsx->rfilename);
+ else
+    DeleteFile(relationsx->rfilename_tmp);
 
  free(relationsx->rfilename);
+ free(relationsx->rfilename_tmp);
 
 
  /* Turn Restriction relations */
 
- if(!keep)
-    DeleteFile(relationsx->trfilename);
+ if(keep)
+    RenameFile(relationsx->trfilename_tmp,relationsx->trfilename);
+ else
+    DeleteFile(relationsx->trfilename_tmp);
 
  free(relationsx->trfilename);
+ free(relationsx->trfilename_tmp);
 
  free(relationsx);
 }
@@ -175,15 +204,15 @@ void FreeRelationList(RelationsX *relationsx,int keep)
   int nrelations The number of relations that are members of the relation.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendRouteRelation(RelationsX* relationsx,relation_t id,
-                         transports_t routes,
-                         way_t *ways,int nways,
-                         relation_t *relations,int nrelations)
+void AppendRouteRelationList(RelationsX* relationsx,relation_t id,
+                             transports_t routes,
+                             way_t *ways,int nways,
+                             relation_t *relations,int nrelations)
 {
- RouteRelX relationx;
+ RouteRelX relationx={0};
  FILESORT_VARINT size;
- way_t noway=NO_WAY;
- relation_t norelation=NO_RELATION;
+ way_t noway=NO_WAY_ID;
+ relation_t norelation=NO_RELATION_ID;
 
  relationx.id=id;
  relationx.routes=routes;
@@ -201,7 +230,7 @@ void AppendRouteRelation(RelationsX* relationsx,relation_t id,
 
  relationsx->rnumber++;
 
- assert(!(relationsx->rnumber==0)); /* Zero marks the high-water mark for relations. */
+ logassert(relationsx->rnumber!=0,"Too many route relations (change index_t to 64-bits?)"); /* Zero marks the high-water mark for relations. */
 }
 
 
@@ -223,11 +252,11 @@ void AppendRouteRelation(RelationsX* relationsx,relation_t id,
   transports_t except The set of transports allowed to bypass the restriction.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
-                                way_t from,way_t to,node_t via,
-                                TurnRestriction restriction,transports_t except)
+void AppendTurnRelationList(RelationsX* relationsx,relation_t id,
+                            way_t from,way_t to,node_t via,
+                            TurnRestriction restriction,transports_t except)
 {
- TurnRestrictRelX relationx={0};
+ TurnRelX relationx={0};
 
  relationx.id=id;
  relationx.from=from;
@@ -236,11 +265,27 @@ void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
  relationx.restriction=restriction;
  relationx.except=except;
 
- WriteFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
+ WriteFile(relationsx->trfd,&relationx,sizeof(TurnRelX));
 
  relationsx->trnumber++;
 
- assert(!(relationsx->trnumber==0)); /* Zero marks the high-water mark for relations. */
+ logassert(relationsx->trnumber!=0,"Too many turn relations (change index_t to 64-bits?)"); /* Zero marks the high-water mark for relations. */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Finish appending relations and change the filename over.
+
+  RelationsX *relationsx The relations that have been appended.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void FinishRelationList(RelationsX *relationsx)
+{
+ if(relationsx->rfd!=-1)
+    relationsx->rfd =CloseFile(relationsx->rfd);
+
+ if(relationsx->trfd!=-1)
+    relationsx->trfd=CloseFile(relationsx->trfd);
 }
 
 
@@ -252,15 +297,42 @@ void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
 
 void SortRelationList(RelationsX* relationsx)
 {
- /* Close the files (finished appending) */
-
- relationsx->rfd=CloseFile(relationsx->rfd);
-
- relationsx->trfd=CloseFile(relationsx->trfd);
-
-
  /* Route Relations */
 
+ if(relationsx->rnumber)
+   {
+    index_t rxnumber;
+    int rfd;
+
+    /* Print the start message */
+
+    printf_first("Sorting Route Relations");
+
+    /* Re-open the file read-only and a new file writeable */
+
+    relationsx->rfd=ReOpenFile(relationsx->rfilename_tmp);
+
+    DeleteFile(relationsx->rfilename_tmp);
+
+    rfd=OpenFileNew(relationsx->rfilename_tmp);
+
+    /* Sort the relations */
+
+    rxnumber=relationsx->rnumber;
+
+    relationsx->rnumber=filesort_vary(relationsx->rfd,rfd,NULL,
+                                                          (int (*)(const void*,const void*))sort_route_by_id,
+                                                          (int (*)(void*,index_t))deduplicate_route_by_id);
+
+    /* Close the files */
+
+    relationsx->rfd=CloseFile(relationsx->rfd);
+    CloseFile(rfd);
+
+    /* Print the final message */
+
+    printf_last("Sorted Route Relations: Relations=%"Pindex_t" Duplicates=%"Pindex_t,rxnumber,rxnumber-relationsx->rnumber);
+   }
 
  /* Turn Restriction Relations. */
 
@@ -275,18 +347,19 @@ void SortRelationList(RelationsX* relationsx)
 
     /* Re-open the file read-only and a new file writeable */
 
-    relationsx->trfd=ReOpenFile(relationsx->trfilename);
+    relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
 
-    DeleteFile(relationsx->trfilename);
+    DeleteFile(relationsx->trfilename_tmp);
 
-    trfd=OpenFileNew(relationsx->trfilename);
+    trfd=OpenFileNew(relationsx->trfilename_tmp);
 
     /* Sort the relations */
 
     trxnumber=relationsx->trnumber;
-    relationsx->trnumber=0;
 
-    relationsx->trnumber=filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRestrictRelX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))deduplicate_by_id);
+    relationsx->trnumber=filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRelX),NULL,
+                                                                               (int (*)(const void*,const void*))sort_turn_by_id,
+                                                                               (int (*)(void*,index_t))deduplicate_turn_by_id);
 
     /* Close the files */
 
@@ -301,16 +374,16 @@ void SortRelationList(RelationsX* relationsx)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Sort the turn restriction relations into id order.
+  Sort the route relations into id order.
 
-  int sort_by_id Returns the comparison of the id fields.
+  int sort_route_by_id Returns the comparison of the id fields.
 
-  TurnRestrictRelX *a The first extended relation.
+  RouteRelX *a The first extended relation.
 
-  TurnRestrictRelX *b The second extended relation.
+  RouteRelX *b The second extended relation.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int sort_by_id(TurnRestrictRelX *a,TurnRestrictRelX *b)
+static int sort_route_by_id(RouteRelX *a,RouteRelX *b)
 {
  relation_t a_id=a->id;
  relation_t b_id=b->id;
@@ -320,120 +393,87 @@ static int sort_by_id(TurnRestrictRelX *a,TurnRestrictRelX *b)
  else if(a_id>b_id)
     return(1);
  else
-    return(0);
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Deduplicate the extended relations using the id after sorting.
+  Deduplicate the route relations using the id after sorting.
 
-  int deduplicate_by_id Return 1 if the value is to be kept, otherwise 0.
+  int deduplicate_route_by_id Return 1 if the value is to be kept, otherwise 0.
 
-  TurnRestrictRelX *relationx The extended relation.
+  RouteRelX *relationx The extended relation.
 
-  index_t index The index of this relation in the total.
+  index_t index The number of sorted relations that have already been written to the output file.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int deduplicate_by_id(TurnRestrictRelX *relationx,index_t index)
+static int deduplicate_route_by_id(RouteRelX *relationx,index_t index)
 {
- static relation_t previd;
+ static relation_t previd=NO_RELATION_ID;
 
- if(index==0 || relationx->id!=previd)
+ if(relationx->id!=previd)
    {
     previd=relationx->id;
 
-    return(1);
+    if(relationx->routes==RELATION_DELETED)
+       return(0);
+    else
+       return(1);
    }
  else
-   {
-    logerror("Relation %"Prelation_t" is duplicated.\n",relationx->id);
-
     return(0);
-   }
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Sort the list of turn relations.
+  Sort the turn restriction relations into id order.
 
-  RelationsX* relationsx The set of relations to process.
+  int sort_turn_by_id Returns the comparison of the id fields.
+
+  TurnRelX *a The first extended relation.
+
+  TurnRelX *b The second extended relation.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SortTurnRelationList(RelationsX* relationsx)
+static int sort_turn_by_id(TurnRelX *a,TurnRelX *b)
 {
- int trfd;
-
- if(relationsx->trnumber==0)
-    return;
-
- /* Print the start message */
-
- printf_first("Sorting Turn Relations");
-
- /* Re-open the file read-only and a new file writeable */
-
- relationsx->trfd=ReOpenFile(relationsx->trfilename);
-
- DeleteFile(relationsx->trfilename);
-
- trfd=OpenFileNew(relationsx->trfilename);
-
- /* Sort the relations */
-
- filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRestrictRelX),(int (*)(const void*,const void*))sort_by_via,NULL);
-
- /* Close the files */
-
- relationsx->trfd=CloseFile(relationsx->trfd);
- CloseFile(trfd);
-
- /* Print the final message */
-
- printf_last("Sorted Turn Relations: Relations=%"Pindex_t,relationsx->trnumber);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Sort the turn restriction relations into via index order (then by from and to segments).
-
-  int sort_by_via Returns the comparison of the via, from and to fields.
-
-  TurnRestrictRelX *a The first extended relation.
-
-  TurnRestrictRelX *b The second extended relation.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
-{
- index_t a_id=a->via;
- index_t b_id=b->via;
+ relation_t a_id=a->id;
+ relation_t b_id=b->id;
 
  if(a_id<b_id)
     return(-1);
  else if(a_id>b_id)
     return(1);
  else
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Deduplicate the turn restriction relations using the id after sorting.
+
+  int deduplicate_turn_by_id Return 1 if the value is to be kept, otherwise 0.
+
+  TurnRelX *relationx The extended relation.
+
+  index_t index The number of sorted relations that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deduplicate_turn_by_id(TurnRelX *relationx,index_t index)
+{
+ static relation_t previd=NO_RELATION_ID;
+
+ if(relationx->id!=previd)
    {
-    index_t a_id=a->from;
-    index_t b_id=b->from;
+    previd=relationx->id;
 
-    if(a_id<b_id)
-       return(-1);
-    else if(a_id>b_id)
-       return(1);
+    if(relationx->except==RELATION_DELETED)
+       return(0);
     else
-      {
-       index_t a_id=a->to;
-       index_t b_id=b->to;
-
-       if(a_id<b_id)
-          return(-1);
-       else if(a_id>b_id)
-          return(1);
-       else
-          return(0);
-      }
+       return(1);
    }
+ else
+    return(0);
 }
 
 
@@ -443,9 +483,11 @@ static int sort_by_via(TurnRestrictRelX *a,TurnRestrictRelX *b)
   RelationsX *relationsx The set of relations to use.
 
   WaysX *waysx The set of ways to modify.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
+void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx,int keep)
 {
  RouteRelX *unmatched=NULL,*lastunmatched=NULL;
  int nunmatched=0,lastnunmatched=0,iteration=1;
@@ -456,14 +498,14 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
  /* Map into memory / open the files */
 
 #if !SLIM
- waysx->data=MapFileWriteable(waysx->filename);
+ waysx->data=MapFileWriteable(waysx->filename_tmp);
 #else
- waysx->fd=ReOpenFileWriteable(waysx->filename);
+ waysx->fd=ReOpenFileWriteable(waysx->filename_tmp);
 #endif
 
  /* Re-open the file read-only */
 
- relationsx->rfd=ReOpenFile(relationsx->rfilename);
+ relationsx->rfd=ReOpenFile(relationsx->rfilename_tmp);
 
  /* Read through the file. */
 
@@ -524,7 +566,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
           /* Update the ways that are listed for the relation */
 
-          if(wayid==NO_WAY)
+          if(wayid==NO_WAY_ID)
              continue;
 
           if(routes)
@@ -536,10 +578,24 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
                 WayX *wayx=LookupWayX(waysx,way,1);
 
                 if(routes&Transports_Foot)
+                  {
+                   if(!(wayx->way.allow&Transports_Foot))
+                     {
+                      logerror("Route Relation %"Prelation_t" for Foot contains Way %"Pway_t" that does not allow Foot transport; overriding.\n",relationx.id,wayid);
+                      wayx->way.allow|=Transports_Foot;
+                     }
                    wayx->way.props|=Properties_FootRoute;
+                  }
 
                 if(routes&Transports_Bicycle)
+                  {
+                   if(!(wayx->way.allow&Transports_Bicycle))
+                     {
+                      logerror("Route Relation %"Prelation_t" for Bicycle contains Way %"Pway_t" that does not allow Bicycle transport; overriding.\n",relationx.id,wayid);
+                      wayx->way.allow|=Transports_Bicycle;
+                     }
                    wayx->way.props|=Properties_BicycleRoute;
+                  }
 
                 PutBackWayX(waysx,wayx);
 
@@ -549,7 +605,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
                 logerror("Route Relation %"Prelation_t" contains Way %"Pway_t" but it does not exist in the Routino database.\n",relationx.id,wayid);
             }
          }
-       while(wayid!=NO_WAY);
+       while(wayid!=NO_WAY_ID);
 
        /* Loop through the relations */
 
@@ -559,7 +615,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
           /* Add the relations that are listed for this relation to the list for next time */
 
-          if(relationid==NO_RELATION)
+          if(relationid==NO_RELATION_ID)
              continue;
 
           if(relationid==relationx.id)
@@ -575,7 +631,7 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
              nunmatched++;
             }
          }
-       while(relationid!=NO_RELATION);
+       while(relationid!=NO_RELATION_ID);
 
        if(!((i+1)%1000))
           printf_middle("Processing Route Relations (%d): Relations=%"Pindex_t" Modified Ways=%"Pindex_t,iteration,relations,ways);
@@ -603,10 +659,13 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
 
  relationsx->rfd=CloseFile(relationsx->rfd);
 
+ if(keep)
+    RenameFile(relationsx->rfilename_tmp,relationsx->rfilename);
+
  /* Unmap from memory / close the files */
 
 #if !SLIM
- waysx->data=UnmapFile(waysx->filename);
+ waysx->data=UnmapFile(waysx->data);
 #else
  waysx->fd=CloseFile(waysx->fd);
 #endif
@@ -621,9 +680,11 @@ void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx)
   NodesX *nodesx The set of nodes to use.
 
   WaysX *waysx The set of ways to use.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
+void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx,int keep)
 {
  int trfd;
  index_t i,deleted=0;
@@ -634,21 +695,23 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
 
  /* Re-open the file read-only and a new file writeable */
 
- relationsx->trfd=ReOpenFile(relationsx->trfilename);
+ relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
 
- DeleteFile(relationsx->trfilename);
+ if(keep)
+    RenameFile(relationsx->trfilename_tmp,relationsx->trfilename);
+ else
+    DeleteFile(relationsx->trfilename_tmp);
 
- trfd=OpenFileNew(relationsx->trfilename);
+ trfd=OpenFileNew(relationsx->trfilename_tmp);
 
  /* Process all of the relations */
 
  for(i=0;i<relationsx->trnumber;i++)
    {
-    TurnRestrictRelX relationx;
-    node_t via;
-    way_t from,to;
+    TurnRelX relationx;
+    index_t via,from,to;
 
-    ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
+    ReadFile(relationsx->trfd,&relationx,sizeof(TurnRelX));
 
     via =IndexNodeX(nodesx,relationx.via);
     from=IndexWayX(waysx,relationx.from);
@@ -670,10 +733,10 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
     if(relationx.via==NO_NODE || relationx.from==NO_WAY || relationx.to==NO_WAY)
        deleted++;
     else
-       WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
+       WriteFile(trfd,&relationx,sizeof(TurnRelX));
 
     if(!((i+1)%1000))
-       printf_middle("Processing Turn Relations (1): Relations=%"Pindex_t" Deleted=%"Pindex_t,i+1-deleted,deleted);
+       printf_middle("Processing Turn Relations (1): Relations=%"Pindex_t" Deleted=%"Pindex_t,i+1,deleted);
    }
 
  /* Close the files */
@@ -683,7 +746,7 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
 
  /* Print the final message */
 
- printf_last("Processed Turn Relations (1): Relations=%"Pindex_t" Deleted=%"Pindex_t,relationsx->trnumber-deleted,deleted);
+ printf_last("Processed Turn Relations (1): Relations=%"Pindex_t" Deleted=%"Pindex_t,relationsx->trnumber,deleted);
 
  relationsx->trnumber-=deleted;
 }
@@ -703,7 +766,7 @@ void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx)
 
 void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx)
 {
- TurnRestrictRelX relationx;
+ TurnRelX relationx;
  int trfd;
  index_t total=0,deleted=0;
 
@@ -717,26 +780,26 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
  /* Map into memory / open the files */
 
 #if !SLIM
- nodesx->data=MapFileWriteable(nodesx->filename);
- segmentsx->data=MapFile(segmentsx->filename);
- waysx->data=MapFile(waysx->filename);
+ nodesx->data=MapFileWriteable(nodesx->filename_tmp);
+ segmentsx->data=MapFile(segmentsx->filename_tmp);
+ waysx->data=MapFile(waysx->filename_tmp);
 #else
- nodesx->fd=ReOpenFileWriteable(nodesx->filename);
- segmentsx->fd=ReOpenFile(segmentsx->filename);
- waysx->fd=ReOpenFile(waysx->filename);
+ nodesx->fd=ReOpenFileWriteable(nodesx->filename_tmp);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
 #endif
 
  /* Re-open the file read-only and a new file writeable */
 
- relationsx->trfd=ReOpenFile(relationsx->trfilename);
+ relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
 
- DeleteFile(relationsx->trfilename);
+ DeleteFile(relationsx->trfilename_tmp);
 
- trfd=OpenFileNew(relationsx->trfilename);
+ trfd=OpenFileNew(relationsx->trfilename_tmp);
 
  /* Process all of the relations */
 
- while(!ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX)))
+ while(!ReadFile(relationsx->trfd,&relationx,sizeof(TurnRelX)))
    {
     NodeX *nodex;
     SegmentX *segmentx;
@@ -827,7 +890,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
        relationx.from=node_from;
        relationx.to  =node_to;
 
-       WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
+       WriteFile(trfd,&relationx,sizeof(TurnRelX));
 
        total++;
 
@@ -888,7 +951,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
                 ;  /* not allowed */
              else
                {
-                assert(nnodes_other<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
+                logassert(nnodes_other<MAX_SEG_PER_NODE,"Too many segments for one node (increase MAX_SEG_PER_NODE?)"); /* Only a limited amount of information stored. */
 
                 node_other[nnodes_other++]=OtherNode(segmentx,relationx.via);
                }
@@ -925,7 +988,7 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
           relationx.from=node_from;
           relationx.to  =node_other[i];
 
-          WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
+          WriteFile(trfd,&relationx,sizeof(TurnRelX));
 
           total++;
 
@@ -964,9 +1027,9 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
  /* Unmap from memory / close the files */
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
- segmentsx->data=UnmapFile(segmentsx->filename);
- waysx->data=UnmapFile(waysx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
+ segmentsx->data=UnmapFile(segmentsx->data);
+ waysx->data=UnmapFile(waysx->data);
 #else
  nodesx->fd=CloseFile(nodesx->fd);
  segmentsx->fd=CloseFile(segmentsx->fd);
@@ -982,7 +1045,72 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Update the node indexes after geographical sorting.
+  Remove pruned turn relations and update the node indexes after pruning nodes.
+
+  RelationsX *relationsx The set of relations to modify.
+
+  NodesX *nodesx The set of nodes to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemovePrunedTurnRelations(RelationsX *relationsx,NodesX *nodesx)
+{
+ TurnRelX relationx;
+ index_t total=0,pruned=0,notpruned=0;
+ int trfd;
+
+ if(relationsx->trnumber==0)
+    return;
+
+ /* Print the start message */
+
+ printf_first("Deleting Pruned Turn Relations: Relations=0 Pruned=0");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
+
+ DeleteFile(relationsx->trfilename_tmp);
+
+ trfd=OpenFileNew(relationsx->trfilename_tmp);
+
+ /* Process all of the relations */
+
+ while(!ReadFile(relationsx->trfd,&relationx,sizeof(TurnRelX)))
+   {
+    relationx.from=nodesx->pdata[relationx.from];
+    relationx.via =nodesx->pdata[relationx.via];
+    relationx.to  =nodesx->pdata[relationx.to];
+
+    if(relationx.from==NO_NODE || relationx.via==NO_NODE || relationx.to==NO_NODE)
+       pruned++;
+    else
+      {
+       WriteFile(trfd,&relationx,sizeof(TurnRelX));
+
+       notpruned++;
+      }
+
+    total++;
+
+    if(!(total%1000))
+       printf_middle("Deleting Pruned Turn Relations: Relations=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
+   }
+
+ relationsx->trnumber=notpruned;
+
+ /* Close the files */
+
+ relationsx->trfd=CloseFile(relationsx->trfd);
+ CloseFile(trfd);
+
+ /* Print the final message */
+
+ printf_last("Deleted Pruned Turn Relations: Relations=%"Pindex_t" Pruned=%"Pindex_t,total,pruned);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the turn relations geographically after updating the node indexes.
 
   RelationsX *relationsx The set of relations to modify.
 
@@ -991,70 +1119,41 @@ void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segm
   SegmentsX *segmentsx The set of segments to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx)
+void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx)
 {
  int trfd;
- index_t i,kept=0;
+
+ if(segmentsx->number==0)
+    return;
 
  /* Print the start message */
 
- printf_first("Updating Turn Relations (Deleting Pruned): Relations=0");
+ printf_first("Sorting Turn Relations Geographically");
 
  /* Map into memory / open the files */
 
 #if !SLIM
- segmentsx->data=MapFile(segmentsx->filename);
+ segmentsx->data=MapFile(segmentsx->filename_tmp);
 #else
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 #endif
 
  /* Re-open the file read-only and a new file writeable */
 
- relationsx->trfd=ReOpenFile(relationsx->trfilename);
+ relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
 
- DeleteFile(relationsx->trfilename);
+ DeleteFile(relationsx->trfilename_tmp);
 
- trfd=OpenFileNew(relationsx->trfilename);
+ trfd=OpenFileNew(relationsx->trfilename_tmp);
 
- /* Process all of the relations */
+ /* Update the segments with geographically sorted node indexes and sort them */
 
- for(i=0;i<relationsx->trnumber;i++)
-   {
-    TurnRestrictRelX relationx;
-    index_t from_node,via_node,to_node;
+ sortnodesx=nodesx;
+ sortsegmentsx=segmentsx;
 
-    ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
-
-    from_node=nodesx->gdata[relationx.from];
-    via_node =nodesx->gdata[relationx.via];
-    to_node  =nodesx->gdata[relationx.to];
-
-    if(from_node<nodesx->number && via_node<nodesx->number && to_node<nodesx->number)
-      {
-       SegmentX *segmentx=FirstSegmentX(segmentsx,via_node,1);
-
-       do
-         {
-          if(OtherNode(segmentx,via_node)==from_node)
-             relationx.from=IndexSegmentX(segmentsx,segmentx);
-
-          if(OtherNode(segmentx,via_node)==to_node)
-             relationx.to=IndexSegmentX(segmentsx,segmentx);
-
-          segmentx=NextSegmentX(segmentsx,segmentx,via_node);
-         }
-       while(segmentx);
-
-       relationx.via=via_node;
-
-       WriteFile(trfd,&relationx,sizeof(TurnRestrictRelX));
-
-       kept++;
-      }
-
-    if(!(relationsx->trnumber%1000))
-       printf_middle("Updating Turn Relations (Deleting Pruned): Relations=%"Pindex_t,relationsx->trnumber);
-   }
+ filesort_fixed(relationsx->trfd,trfd,sizeof(TurnRelX),(int (*)(void*,index_t))geographically_index,
+                                                       (int (*)(const void*,const void*))sort_by_via,
+                                                       NULL);
 
  /* Close the files */
 
@@ -1064,16 +1163,97 @@ void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmen
  /* Unmap from memory / close the files */
 
 #if !SLIM
- segmentsx->data=UnmapFile(segmentsx->filename);
+ segmentsx->data=UnmapFile(segmentsx->data);
 #else
  segmentsx->fd=CloseFile(segmentsx->fd);
 #endif
 
  /* Print the final message */
 
- printf_last("Updated Turn Relations: Relations=%"Pindex_t" Deleted=%"Pindex_t,kept,relationsx->trnumber-kept);
+ printf_last("Sorted Turn Relations Geographically: Turn Relations=%"Pindex_t,relationsx->trnumber);
+}
 
- relationsx->trnumber=kept;
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update the turn relation indexes.
+
+  int geographically_index Return 1 if the value is to be kept, otherwise 0.
+
+  TurnRelX *relationx The extended turn relation.
+
+  index_t index The number of unsorted turn relations that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int geographically_index(TurnRelX *relationx,index_t index)
+{
+ SegmentX *segmentx;
+ index_t from_node,via_node,to_node;
+
+ from_node=sortnodesx->gdata[relationx->from];
+ via_node =sortnodesx->gdata[relationx->via];
+ to_node  =sortnodesx->gdata[relationx->to];
+
+ segmentx=FirstSegmentX(sortsegmentsx,via_node,1);
+
+ do
+   {
+    if(OtherNode(segmentx,via_node)==from_node)
+       relationx->from=IndexSegmentX(sortsegmentsx,segmentx);
+
+    if(OtherNode(segmentx,via_node)==to_node)
+       relationx->to=IndexSegmentX(sortsegmentsx,segmentx);
+
+    segmentx=NextSegmentX(sortsegmentsx,segmentx,via_node);
+   }
+ while(segmentx);
+
+ relationx->via=via_node;
+
+ return(1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the turn restriction relations into via index order (then by from and to segments).
+
+  int sort_by_via Returns the comparison of the via, from and to fields.
+
+  TurnRelX *a The first extended relation.
+
+  TurnRelX *b The second extended relation.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_via(TurnRelX *a,TurnRelX *b)
+{
+ index_t a_id=a->via;
+ index_t b_id=b->via;
+
+ if(a_id<b_id)
+    return(-1);
+ else if(a_id>b_id)
+    return(1);
+ else
+   {
+    index_t a_id=a->from;
+    index_t b_id=b->from;
+
+    if(a_id<b_id)
+       return(-1);
+    else if(a_id>b_id)
+       return(1);
+    else
+      {
+       index_t a_id=a->to;
+       index_t b_id=b->to;
+
+       if(a_id<b_id)
+          return(-1);
+       else if(a_id>b_id)
+          return(1);
+       else
+          return(FILESORT_PRESERVE_ORDER(a,b));
+      }
+   }
 }
 
 
@@ -1097,7 +1277,7 @@ void SaveRelationList(RelationsX* relationsx,const char *filename)
 
  /* Re-open the file read-only */
 
- relationsx->trfd=ReOpenFile(relationsx->trfilename);
+ relationsx->trfd=ReOpenFile(relationsx->trfilename_tmp);
 
  /* Write out the relations data */
 
@@ -1107,10 +1287,10 @@ void SaveRelationList(RelationsX* relationsx,const char *filename)
 
  for(i=0;i<relationsx->trnumber;i++)
    {
-    TurnRestrictRelX relationx;
+    TurnRelX relationx;
     TurnRelation relation={0};
 
-    ReadFile(relationsx->trfd,&relationx,sizeof(TurnRestrictRelX));
+    ReadFile(relationsx->trfd,&relationx,sizeof(TurnRelX));
 
     relation.from=relationx.from;
     relation.via=relationx.via;
