@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2010-2011 Andrew M. Bishop
+ This file Copyright 2010-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -43,7 +43,7 @@ struct _RouteRelX
 
 
 /*+ An extended structure containing a single turn restriction relation. +*/
-struct _TurnRestrictRelX
+struct _TurnRelX
 {
  relation_t      id;           /*+ The relation identifier. +*/
 
@@ -61,15 +61,19 @@ struct _RelationsX
 {
  /* Route relations */
 
- char      *rfilename;         /*+ The name of the temporary file (for the RouteRelX). +*/
- int        rfd;               /*+ The file descriptor of the temporary file (for the RouteRelX). +*/
+ char      *rfilename;         /*+ The name of the intermediate file (for the RouteRelX). +*/
+ char      *rfilename_tmp;     /*+ The name of the temporary file (for the RouteRelX). +*/
+
+ int        rfd;               /*+ The file descriptor of the open file (for the RouteRelX). +*/
 
  index_t    rnumber;           /*+ The number of extended route relations. +*/
 
  /* Turn restriction relations */
 
- char      *trfilename;        /*+ The name of the temporary file (for the TurnRestrictRelX). +*/
- int        trfd;              /*+ The file descriptor of the temporary file (for the TurnRestrictRelX). +*/
+ char      *trfilename;        /*+ The name of the intermediate file (for the TurnRelX). +*/
+ char      *trfilename_tmp;    /*+ The name of the temporary file (for the TurnRelX). +*/
+
+ int        trfd;              /*+ The file descriptor of the temporary file (for the TurnRelX). +*/
 
  index_t    trnumber;          /*+ The number of extended turn restriction relations. +*/
 };
@@ -77,28 +81,28 @@ struct _RelationsX
 
 /* Functions in relationsx.c */
 
-RelationsX *NewRelationList(int append);
+RelationsX *NewRelationList(int append,int readonly);
 void FreeRelationList(RelationsX *relationsx,int keep);
 
-void AppendRouteRelation(RelationsX* relationsx,relation_t id,
-                         transports_t routes,
-                         way_t *ways,int nways,
-                         relation_t *relations,int nrelations);
-
-void AppendTurnRestrictRelation(RelationsX* relationsx,relation_t id,
-                                way_t from,way_t to,node_t via,
-                                TurnRestriction restriction,transports_t except);
+void AppendRouteRelationList(RelationsX* relationsx,relation_t id,
+                             transports_t routes,
+                             way_t *ways,int nways,
+                             relation_t *relations,int nrelations);
+void AppendTurnRelationList(RelationsX* relationsx,relation_t id,
+                            way_t from,way_t to,node_t via,
+                            TurnRestriction restriction,transports_t except);
+void FinishRelationList(RelationsX *relationsx);
 
 void SortRelationList(RelationsX *relationsx);
 
-void SortTurnRelationList(RelationsX* relationsx);
+void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx,int keep);
 
-void ProcessRouteRelations(RelationsX *relationsx,WaysX *waysx);
-
-void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx);
+void ProcessTurnRelations1(RelationsX *relationsx,NodesX *nodesx,WaysX *waysx,int keep);
 void ProcessTurnRelations2(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx,WaysX *waysx);
 
-void UpdateTurnRelations(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx);
+void RemovePrunedTurnRelations(RelationsX *relationsx,NodesX *nodesx);
+
+void SortTurnRelationListGeographically(RelationsX *relationsx,NodesX *nodesx,SegmentsX *segmentsx);
 
 void SaveRelationList(RelationsX* relationsx,const char *filename);
 
