@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2011 Andrew M. Bishop
+ This file Copyright 2008-2012 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -164,7 +164,7 @@ int main(int argc,char** argv)
    {
     if(ExistsFile(FileName(dirname,prefix,"profiles.xml")))
        profiles=FileName(dirname,prefix,"profiles.xml");
-    else if(ExistsFile(FileName(DATADIR,NULL,"tagging.xml")))
+    else if(ExistsFile(FileName(DATADIR,NULL,"profiles.xml")))
        profiles=FileName(DATADIR,NULL,"profiles.xml");
     else
       {
@@ -284,7 +284,7 @@ int main(int argc,char** argv)
 
        highway=HighwayType(string);
 
-       if(highway==Way_Count)
+       if(highway==Highway_None)
           print_usage(0,argv[arg],NULL);
 
        profile->highway[highway]=atof(equal+1);
@@ -305,7 +305,7 @@ int main(int argc,char** argv)
 
        highway=HighwayType(string);
 
-       if(highway==Way_Count)
+       if(highway==Highway_None)
           print_usage(0,argv[arg],NULL);
 
        profile->speed[highway]=kph_to_speed(atof(equal+1));
@@ -326,7 +326,7 @@ int main(int argc,char** argv)
 
        property=PropertyType(string);
 
-       if(property==Property_Count)
+       if(property==Property_None)
           print_usage(0,argv[arg],NULL);
 
        profile->props_yes[property]=atof(equal+1);
@@ -571,7 +571,7 @@ int main(int argc,char** argv)
             {
              if(!finish_result)
                {
-                fprintf(stderr,"Error: Cannot find route compatible with profile.\n");
+                fprintf(stderr,"Error: Cannot create combined route following super-route.\n");
                 return(1);
                }
             }
