@@ -20,7 +20,6 @@
  ***************************************/
 
 
-#include <assert.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -44,11 +43,27 @@
 /*+ The command line '--tmpdir' option or its default value. +*/
 extern char *option_tmpdirname;
 
+/* Local variables */
+
+/*+ Temporary file-local variables for use by the sort functions. +*/
+static NodesX *sortnodesx;
+static SegmentsX *sortsegmentsx;
+static WaysX *sortwaysx;
+
 /* Local functions */
 
+static int sort_by_way_id(SegmentX *a,SegmentX *b);
+static int apply_changes(SegmentX *segmentx,index_t index);
+
 static int sort_by_id(SegmentX *a,SegmentX *b);
+static int deduplicate(SegmentX *segmentx,index_t index);
+
 static int delete_pruned(SegmentX *segmentx,index_t index);
 
+static int deduplicate_super(SegmentX *segmentx,index_t index);
+
+static int geographically_index(SegmentX *segmentx,index_t index);
+
 static distance_t DistanceX(NodeX *nodex1,NodeX *nodex2);
 
 
@@ -57,36 +72,43 @@ static distance_t DistanceX(NodeX *nodex1,NodeX *nodex2);
 
   SegmentsX *NewSegmentList Returns the segment list.
 
-  int append Set to 1 if the file is to be opened for appending (now or later).
+  int append Set to 1 if the file is to be opened for appending.
+
+  int readonly Set to 1 if the file is to be opened for reading.
   ++++++++++++++++++++++++++++++++++++++*/
 
-SegmentsX *NewSegmentList(int append)
+SegmentsX *NewSegmentList(int append,int readonly)
 {
  SegmentsX *segmentsx;
 
  segmentsx=(SegmentsX*)calloc(1,sizeof(SegmentsX));
 
- assert(segmentsx); /* Check calloc() worked */
+ logassert(segmentsx,"Failed to allocate memory (try using slim mode?)"); /* Check calloc() worked */
 
- segmentsx->filename=(char*)malloc(strlen(option_tmpdirname)+32);
+ segmentsx->filename    =(char*)malloc(strlen(option_tmpdirname)+32);
+ segmentsx->filename_tmp=(char*)malloc(strlen(option_tmpdirname)+32);
+
+ sprintf(segmentsx->filename    ,"%s/segmentsx.parsed.mem",option_tmpdirname);
+ sprintf(segmentsx->filename_tmp,"%s/segmentsx.%p.tmp"    ,option_tmpdirname,(void*)segmentsx);
+
+ if(append || readonly)
+    if(ExistsFile(segmentsx->filename))
+      {
+       off_t size;
+
+       size=SizeFile(segmentsx->filename);
+
+       segmentsx->number=size/sizeof(SegmentX);
+
+       RenameFile(segmentsx->filename,segmentsx->filename_tmp);
+      }
 
  if(append)
-    sprintf(segmentsx->filename,"%s/segmentsx.input.tmp",option_tmpdirname);
+    segmentsx->fd=OpenFileAppend(segmentsx->filename_tmp);
+ else if(!readonly)
+    segmentsx->fd=OpenFileNew(segmentsx->filename_tmp);
  else
-    sprintf(segmentsx->filename,"%s/segmentsx.%p.tmp",option_tmpdirname,(void*)segmentsx);
-
- if(append)
-   {
-    off_t size;
-
-    segmentsx->fd=OpenFileAppend(segmentsx->filename);
-
-    size=SizeFile(segmentsx->filename);
-
-    segmentsx->number=size/sizeof(SegmentX);
-   }
- else
-    segmentsx->fd=OpenFileNew(segmentsx->filename);
+    segmentsx->fd=-1;
 
  return(segmentsx);
 }
@@ -97,15 +119,18 @@ SegmentsX *NewSegmentList(int append)
 
   SegmentsX *segmentsx The set of segments to be freed.
 
-  int keep Set to 1 if the file is to be kept (for appending later).
+  int keep If set then the results file is to be kept.
   ++++++++++++++++++++++++++++++++++++++*/
 
 void FreeSegmentList(SegmentsX *segmentsx,int keep)
 {
- if(!keep)
-    DeleteFile(segmentsx->filename);
+ if(keep)
+    RenameFile(segmentsx->filename_tmp,segmentsx->filename);
+ else
+    DeleteFile(segmentsx->filename_tmp);
 
  free(segmentsx->filename);
+ free(segmentsx->filename_tmp);
 
  if(segmentsx->firstnode)
     free(segmentsx->firstnode);
@@ -134,7 +159,7 @@ void FreeSegmentList(SegmentsX *segmentsx,int keep)
   distance_t distance The distance between the nodes (or just the flags).
   ++++++++++++++++++++++++++++++++++++++*/
 
-void AppendSegment(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,distance_t distance)
+void AppendSegmentList(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,distance_t distance)
 {
  SegmentX segmentx;
 
@@ -160,127 +185,20 @@ void AppendSegment(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,dist
 
  segmentsx->number++;
 
- assert(segmentsx->number<SEGMENT_FAKE); /* SEGMENT_FAKE marks the high-water mark for real segments. */
+ logassert(segmentsx->number<SEGMENT_FAKE,"Too many segments (change index_t to 64-bits?)"); /* SEGMENT_FAKE marks the high-water mark for real segments. */
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Sort the segment list.
+  Finish appending segments and change the filename over.
 
-  SegmentsX *segmentsx The set of segments to sort and modify.
-
-  int delete Set to true if pruned segments are to be deleted.
+  SegmentsX *segmentsx The segments that have been appended.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void SortSegmentList(SegmentsX *segmentsx,int delete)
+void FinishSegmentList(SegmentsX *segmentsx)
 {
- int fd;
- index_t kept;
-
- /* Print the start message */
-
- if(delete)
-    printf_first("Sorting Segments (Deleting Pruned)");
- else
-    printf_first("Sorting Segments");
-
- /* Close the file (finished appending) */
-
  if(segmentsx->fd!=-1)
     segmentsx->fd=CloseFile(segmentsx->fd);
-
- /* Re-open the file read-only and a new file writeable */
-
- segmentsx->fd=ReOpenFile(segmentsx->filename);
-
- DeleteFile(segmentsx->filename);
-
- fd=OpenFileNew(segmentsx->filename);
-
- /* Sort by node indexes */
-
- if(delete)
-    kept=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,(int (*)(void*,index_t))delete_pruned);
- else
-    filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(const void*,const void*))sort_by_id,NULL);
-
- /* Close the files */
-
- segmentsx->fd=CloseFile(segmentsx->fd);
- CloseFile(fd);
-
- /* Print the final message */
-
- if(delete)
-   {
-    printf_last("Sorted Segments: Segments=%"Pindex_t" Deleted=%"Pindex_t,kept,segmentsx->number-kept);
-    segmentsx->number=kept;
-   }
- else
-    printf_last("Sorted Segments: Segments=%"Pindex_t,segmentsx->number);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Sort the segments into id order, first by node1 then by node2, finally by distance.
-
-  int sort_by_id Returns the comparison of the node fields.
-
-  SegmentX *a The first segment.
-
-  SegmentX *b The second segment.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int sort_by_id(SegmentX *a,SegmentX *b)
-{
- node_t a_id1=a->node1;
- node_t b_id1=b->node1;
-
- if(a_id1<b_id1)
-    return(-1);
- else if(a_id1>b_id1)
-    return(1);
- else /* if(a_id1==b_id1) */
-   {
-    node_t a_id2=a->node2;
-    node_t b_id2=b->node2;
-
-    if(a_id2<b_id2)
-       return(-1);
-    else if(a_id2>b_id2)
-       return(1);
-    else
-      {
-       distance_t a_distance=DISTANCE(a->distance);
-       distance_t b_distance=DISTANCE(b->distance);
-
-       if(a_distance<b_distance)
-          return(-1);
-       else if(a_distance>b_distance)
-          return(1);
-       else
-          return(0);
-      }
-   }
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Delete the pruned segments.
-
-  int delete_pruned Return 1 if the value is to be kept, otherwise 0.
-
-  SegmentX *segmentx The extended segment.
-
-  index_t index The index of this segment in the total.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-static int delete_pruned(SegmentX *segmentx,index_t index)
-{
- if(IsPrunedSegmentX(segmentx))
-    return(0);
-
- return(1);
 }
 
 
@@ -375,38 +293,270 @@ SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t nodeindex
  
  
 /*++++++++++++++++++++++++++++++++++++++
-  Remove bad segments (duplicated, zero length or with missing nodes).
+  Apply the changes to the segments (no unique id to use).
 
-  NodesX *nodesx The set of nodes to use.
-
-  SegmentsX *segmentsx The set of segments to modify.
+  SegmentsX *segmentsx The set of segments to sort and modify.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
+void ApplySegmentChanges(SegmentsX *segmentsx)
 {
- index_t duplicate=0,loop=0,nonode=0,good=0,total=0;
- SegmentX segmentx;
  int fd;
- node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
- distance_t prevdist=0;
+ index_t xnumber;
 
  /* Print the start message */
 
- printf_first("Checking Segments: Segments=0 Duplicate=0 Loop=0 No-Node=0");
-
- /* Allocate the array of node flags */
-
- segmentsx->usednode=AllocBitMask(nodesx->number);
-
- assert(segmentsx->usednode); /* Check AllocBitMask() worked */
+ printf_first("Applying Segment Changes");
 
  /* Re-open the file read-only and a new file writeable */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 
- DeleteFile(segmentsx->filename);
+ DeleteFile(segmentsx->filename_tmp);
 
- fd=OpenFileNew(segmentsx->filename);
+ fd=OpenFileNew(segmentsx->filename_tmp);
+
+ /* Sort by node indexes */
+
+ xnumber=segmentsx->number;
+
+ segmentsx->number=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),NULL,
+                                                                    (int (*)(const void*,const void*))sort_by_way_id,
+                                                                    (int (*)(void*,index_t))apply_changes);
+
+ /* Close the files */
+
+ segmentsx->fd=CloseFile(segmentsx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Applying Segment Changes: Segments=%"Pindex_t" Changed=%"Pindex_t,xnumber,xnumber-segmentsx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the segments into way id order.
+
+  int sort_by_way_id Returns the comparison of the way fields.
+
+  SegmentX *a The first segment.
+
+  SegmentX *b The second segment.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_way_id(SegmentX *a,SegmentX *b)
+{
+ way_t a_id=a->way;
+ way_t b_id=b->way;
+
+ if(a_id<b_id)
+    return(-1);
+ else if(a_id>b_id)
+    return(1);
+ else /* if(a_id==b_id) */
+    return(-FILESORT_PRESERVE_ORDER(a,b)); /* latest version first */
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Apply the changes to the segments.
+
+  int apply_changes Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended segment.
+
+  index_t index The number of sorted segments that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int apply_changes(SegmentX *segmentx,index_t index)
+{
+ static way_t prevway=NO_WAY_ID;
+ static int deleted=0;
+
+ if(prevway!=segmentx->way)
+   {
+    prevway=segmentx->way;
+    deleted=0;
+   }
+
+ if(!deleted)
+    if(segmentx->node1==NO_NODE_ID)
+       deleted=1;
+
+ if(deleted)
+    return(0);
+ else
+    return(1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the segment list and deduplicate it.
+
+  SegmentsX *segmentsx The set of segments to sort and modify.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void SortSegmentList(SegmentsX *segmentsx)
+{
+ int fd;
+ index_t xnumber;
+
+ /* Print the start message */
+
+ printf_first("Sorting Segments");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+
+ DeleteFile(segmentsx->filename_tmp);
+
+ fd=OpenFileNew(segmentsx->filename_tmp);
+
+ /* Sort by node indexes */
+
+ xnumber=segmentsx->number;
+
+ segmentsx->number=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),NULL,
+                                                                    (int (*)(const void*,const void*))sort_by_id,
+                                                                    (int (*)(void*,index_t))deduplicate);
+
+ /* Close the files */
+
+ segmentsx->fd=CloseFile(segmentsx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted Segments: Segments=%"Pindex_t" Duplicates=%"Pindex_t,xnumber,xnumber-segmentsx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the segments into id order, first by node1 then by node2, finally by distance.
+
+  int sort_by_id Returns the comparison of the node fields.
+
+  SegmentX *a The first segment.
+
+  SegmentX *b The second segment.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int sort_by_id(SegmentX *a,SegmentX *b)
+{
+ node_t a_id1=a->node1;
+ node_t b_id1=b->node1;
+
+ if(a_id1<b_id1)
+    return(-1);
+ else if(a_id1>b_id1)
+    return(1);
+ else /* if(a_id1==b_id1) */
+   {
+    node_t a_id2=a->node2;
+    node_t b_id2=b->node2;
+
+    if(a_id2<b_id2)
+       return(-1);
+    else if(a_id2>b_id2)
+       return(1);
+    else
+      {
+       distance_t a_distance=DISTANCE(a->distance);
+       distance_t b_distance=DISTANCE(b->distance);
+
+       if(a_distance<b_distance)
+          return(-1);
+       else if(a_distance>b_distance)
+          return(1);
+       else
+         {
+          distance_t a_distflag=DISTFLAG(a->distance);
+          distance_t b_distflag=DISTFLAG(b->distance);
+
+          if(a_distflag<b_distflag)
+             return(-1);
+          else if(a_distflag>b_distflag)
+             return(1);
+          else
+             return(FILESORT_PRESERVE_ORDER(a,b)); /* preserve order */
+         }
+      }
+   }
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Discard duplicate segments.
+
+  int deduplicate Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended segment.
+
+  index_t index The number of sorted segments that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deduplicate(SegmentX *segmentx,index_t index)
+{
+ static node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
+ static way_t prevway=NO_WAY_ID;
+ static distance_t prevdist=0;
+
+ if(prevnode1!=segmentx->node1 || prevnode2!=segmentx->node2 || prevway!=segmentx->way || prevdist!=segmentx->distance)
+   {
+    prevnode1=segmentx->node1;
+    prevnode2=segmentx->node2;
+    prevway=segmentx->way;
+    prevdist=segmentx->distance;
+
+    return(1);
+   }
+ else
+    return(0);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove bad segments (duplicated, zero length or with missing nodes).
+
+  SegmentsX *segmentsx The set of segments to modify.
+
+  NodesX *nodesx The set of nodes to use.
+
+  WaysX *waysx The set of ways to use.
+
+  int keep If set to 1 then keep the old data file otherwise delete it.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep)
+{
+ index_t noway=0,loop=0,nonode=0,duplicate=0,good=0,total=0;
+ node_t prevnode1=NO_NODE_ID,prevnode2=NO_NODE_ID;
+ way_t prevway=NO_WAY_ID;
+ distance_t prevdist=0;
+ SegmentX segmentx;
+ int fd;
+
+ /* Print the start message */
+
+ printf_first("Checking Segments: Segments=0 Loop=0 No-Way=0 No-Node=0 Duplicate=0");
+
+ /* Allocate the node usage bitmask */
+
+ segmentsx->usednode=AllocBitMask(nodesx->number);
+
+ logassert(segmentsx->usednode,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
+
+ /* Re-open the file read-only and a new file writeable */
+
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+
+ if(keep)
+    RenameFile(segmentsx->filename_tmp,segmentsx->filename);
+ else
+    DeleteFile(segmentsx->filename_tmp);
+
+ fd=OpenFileNew(segmentsx->filename_tmp);
 
  /* Modify the on-disk image */
 
@@ -414,22 +564,13 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
    {
     index_t index1=IndexNodeX(nodesx,segmentx.node1);
     index_t index2=IndexNodeX(nodesx,segmentx.node2);
+    index_t indexw=IndexWayX(waysx,segmentx.way);
 
-    if(prevnode1==segmentx.node1 && prevnode2==segmentx.node2)
+    if(indexw==NO_WAY)
       {
-       if(!prevdist && !DISTANCE(segmentx.distance))
-          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",segmentx.node1,segmentx.node2);
+       logerror("Segment belongs to way %"Pway_t" but it doesn't exist.\n",segmentx.way);
 
-       if(!prevdist && DISTANCE(segmentx.distance))
-          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the area).\n",segmentx.node1,segmentx.node2);
-
-       if(prevdist && !DISTANCE(segmentx.distance))
-          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the non-area).\n",segmentx.node1,segmentx.node2);
-
-       if(prevdist && DISTANCE(segmentx.distance))
-          logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (both are areas).\n",segmentx.node1,segmentx.node2);
-
-       duplicate++;
+       noway++;
       }
     else if(segmentx.node1==segmentx.node2)
       {
@@ -450,6 +591,27 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
        nonode++;
       }
+    else if(prevnode1==segmentx.node1 && prevnode2==segmentx.node2)
+      {
+       if(prevway==segmentx.way)
+          ; /* already logged an error - only possible to get here for oneway opposite direction segments */
+       else
+         {
+          if(!(prevdist&SEGMENT_AREA) && !(segmentx.distance&SEGMENT_AREA))
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated.\n",segmentx.node1,segmentx.node2);
+
+          if(!(prevdist&SEGMENT_AREA) && (segmentx.distance&SEGMENT_AREA))
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the area).\n",segmentx.node1,segmentx.node2);
+
+          if((prevdist&SEGMENT_AREA) && !(segmentx.distance&SEGMENT_AREA))
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (discarded the non-area).\n",segmentx.node1,segmentx.node2);
+
+          if((prevdist&SEGMENT_AREA) && (segmentx.distance&SEGMENT_AREA))
+             logerror("Segment connecting nodes %"Pnode_t" and %"Pnode_t" is duplicated (both are areas).\n",segmentx.node1,segmentx.node2);
+         }
+
+       duplicate++;
+      }
     else
       {
        WriteFile(fd,&segmentx,sizeof(SegmentX));
@@ -457,17 +619,18 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
        SetBit(segmentsx->usednode,index1);
        SetBit(segmentsx->usednode,index2);
 
-       good++;
-
        prevnode1=segmentx.node1;
        prevnode2=segmentx.node2;
+       prevway=segmentx.way;
        prevdist=DISTANCE(segmentx.distance);
+
+       good++;
       }
 
     total++;
 
     if(!(total%10000))
-       printf_middle("Checking Segments: Segments=%"Pindex_t" Duplicate=%"Pindex_t" Loop=%"Pindex_t" No-Node=%"Pindex_t,total,duplicate,loop,nonode);
+       printf_middle("Checking Segments: Segments=%"Pindex_t" Loop=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,loop,noway,nonode,duplicate);
    }
 
  segmentsx->number=good;
@@ -479,7 +642,7 @@ void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx)
 
  /* Print the final message */
 
- printf_last("Checked Segments: Segments=%"Pindex_t" Duplicate=%"Pindex_t" Loop=%"Pindex_t" No-Node=%"Pindex_t,total,duplicate,loop,nonode);
+ printf_last("Checked Segments: Segments=%"Pindex_t" Loop=%"Pindex_t" No-Way=%"Pindex_t" No-Node=%"Pindex_t" Duplicate=%"Pindex_t,total,loop,noway,nonode,duplicate);
 }
 
 
@@ -499,6 +662,9 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
  int fd;
  SegmentX segmentx;
 
+ if(segmentsx->number==0)
+    return;
+
  /* Print the start message */
 
  printf_first("Measuring Segments: Segments=0");
@@ -506,18 +672,24 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
  /* Map into memory /  open the file */
 
 #if !SLIM
- nodesx->data=MapFile(nodesx->filename);
+ nodesx->data=MapFile(nodesx->filename_tmp);
 #else
- nodesx->fd=ReOpenFile(nodesx->filename);
+ nodesx->fd=ReOpenFile(nodesx->filename_tmp);
 #endif
 
+ /* Allocate the way usage bitmask */
+
+ segmentsx->usedway=AllocBitMask(waysx->number);
+
+ logassert(segmentsx->usedway,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
+
  /* Re-open the file read-only and a new file writeable */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 
- DeleteFile(segmentsx->filename);
+ DeleteFile(segmentsx->filename_tmp);
 
- fd=OpenFileNew(segmentsx->filename);
+ fd=OpenFileNew(segmentsx->filename_tmp);
 
  /* Modify the on-disk image */
 
@@ -536,9 +708,12 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
     segmentx.node2=node2;
     segmentx.way  =way;
 
-    /* Set the distance but preserve the other flags */
+    SetBit(segmentsx->usedway,segmentx.way);
 
-    segmentx.distance|=DISTANCE(DistanceX(nodex1,nodex2));
+    /* Set the distance but keep the other flags except for area */
+
+    segmentx.distance=DISTANCE(DistanceX(nodex1,nodex2))|DISTFLAG(segmentx.distance);
+    segmentx.distance&=~SEGMENT_AREA;
 
     /* Write the modified segment */
 
@@ -566,7 +741,7 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
  /* Unmap from memory / close the file */
 
 #if !SLIM
- nodesx->data=UnmapFile(nodesx->filename);
+ nodesx->data=UnmapFile(nodesx->data);
 #else
  nodesx->fd=CloseFile(nodesx->fd);
 #endif
@@ -578,7 +753,7 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Remove the duplicate segments.
+  Index the segments by creating the firstnode index and filling in the segment next2 parameter.
 
   SegmentsX *segmentsx The set of segments to modify.
 
@@ -587,130 +762,7 @@ void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
   WaysX *waysx The set of ways to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void DeduplicateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
-{
- index_t duplicate=0,good=0;
- index_t index=0;
- int fd,nprev=0;
- index_t prevnode1=NO_NODE,prevnode2=NO_NODE;
- SegmentX prevsegx[MAX_SEG_PER_NODE],segmentx;
- Way prevway[MAX_SEG_PER_NODE];
-
- /* Print the start message */
-
- printf_first("Deduplicating Segments: Segments=0 Duplicate=0");
-
- /* Map into memory / open the file */
-
-#if !SLIM
- waysx->data=MapFile(waysx->filename);
-#else
- waysx->fd=ReOpenFile(waysx->filename);
-#endif
-
- /* Re-open the file read-only and a new file writeable */
-
- segmentsx->fd=ReOpenFile(segmentsx->filename);
-
- DeleteFile(segmentsx->filename);
-
- fd=OpenFileNew(segmentsx->filename);
-
- /* Modify the on-disk image */
-
- while(!ReadFile(segmentsx->fd,&segmentx,sizeof(SegmentX)))
-   {
-    WayX *wayx=LookupWayX(waysx,segmentx.way,1);
-    int isduplicate=0;
-
-    if(segmentx.node1==prevnode1 && segmentx.node2==prevnode2)
-      {
-       int offset;
-
-       for(offset=0;offset<nprev;offset++)
-         {
-          if(DISTFLAG(segmentx.distance)==DISTFLAG(prevsegx[offset].distance))
-             if(!WaysCompare(&prevway[offset],&wayx->way))
-               {
-                isduplicate=1;
-                break;
-               }
-         }
-
-       if(isduplicate)
-         {
-          nprev--;
-
-          for(;offset<nprev;offset++)
-            {
-             prevsegx[offset]=prevsegx[offset+1];
-             prevway[offset] =prevway[offset+1];
-            }
-         }
-       else
-         {
-          assert(nprev<MAX_SEG_PER_NODE); /* Only a limited amount of information stored. */
-
-          prevsegx[nprev]=segmentx;
-          prevway[nprev] =wayx->way;
-
-          nprev++;
-         }
-      }
-    else
-      {
-       nprev=1;
-       prevnode1=segmentx.node1;
-       prevnode2=segmentx.node2;
-       prevsegx[0]=segmentx;
-       prevway[0] =wayx->way;
-      }
-
-    if(isduplicate)
-       duplicate++;
-    else
-      {
-       WriteFile(fd,&segmentx,sizeof(SegmentX));
-
-       good++;
-      }
-
-    index++;
-
-    if(!(index%10000))
-       printf_middle("Deduplicating Segments: Segments=%"Pindex_t" Duplicate=%"Pindex_t,index,duplicate);
-   }
-
- segmentsx->number=good;
-
- /* Close the files */
-
- segmentsx->fd=CloseFile(segmentsx->fd);
- CloseFile(fd);
-
- /* Unmap from memory / close the file */
-
-#if !SLIM
- waysx->data=UnmapFile(waysx->filename);
-#else
- waysx->fd=CloseFile(waysx->fd);
-#endif
-
- /* Print the final message */
-
- printf_last("Deduplicated Segments: Segments=%"Pindex_t" Duplicate=%"Pindex_t" Unique=%"Pindex_t,index,duplicate,good);
-}
-
-
-/*++++++++++++++++++++++++++++++++++++++
-  Index the segments by creating the firstnode index and filling in the segment next2 parameter.
-
-  SegmentsX *segmentsx The set of segments to modify.
-
-  NodesX *nodesx The sset of nodes to use.
-  ++++++++++++++++++++++++++++++++++++++*/
-
-void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
+void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
 {
  index_t index,i;
 
@@ -723,12 +775,12 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
 
  /* Allocate the array of indexes */
 
- if(!segmentsx->firstnode)
-   {
-    segmentsx->firstnode=(index_t*)malloc(nodesx->number*sizeof(index_t));
+ if(segmentsx->firstnode)
+    free(segmentsx->firstnode);
 
-    assert(segmentsx->firstnode); /* Check malloc() worked */
-   }
+ segmentsx->firstnode=(index_t*)malloc(nodesx->number*sizeof(index_t));
+
+ logassert(segmentsx->firstnode,"Failed to allocate memory (try using slim mode?)"); /* Check malloc() worked */
 
  for(i=0;i<nodesx->number;i++)
     segmentsx->firstnode[i]=NO_SEGMENT;
@@ -736,9 +788,9 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
  /* Map into memory / open the files */
 
 #if !SLIM
- segmentsx->data=MapFileWriteable(segmentsx->filename);
+ segmentsx->data=MapFileWriteable(segmentsx->filename_tmp);
 #else
- segmentsx->fd=ReOpenFileWriteable(segmentsx->filename);
+ segmentsx->fd=ReOpenFileWriteable(segmentsx->filename_tmp);
 #endif
 
  /* Read through the segments in reverse order */
@@ -747,6 +799,15 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
    {
     SegmentX *segmentx=LookupSegmentX(segmentsx,index,1);
 
+    if(nodesx->pdata)
+      {
+       segmentx->node1=nodesx->pdata[segmentx->node1];
+       segmentx->node2=nodesx->pdata[segmentx->node2];
+      }
+
+    if(waysx->cdata)
+       segmentx->way=waysx->cdata[segmentx->way];
+
     segmentx->next2=segmentsx->firstnode[segmentx->node2];
 
     PutBackSegmentX(segmentsx,segmentx);
@@ -761,11 +822,25 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
  /* Unmap from memory / close the files */
 
 #if !SLIM
- segmentsx->data=UnmapFile(segmentsx->filename);
+ segmentsx->data=UnmapFile(segmentsx->data);
 #else
  segmentsx->fd=CloseFile(segmentsx->fd);
 #endif
 
+ /* Free the memory */
+
+ if(nodesx->pdata)
+   {
+    free(nodesx->pdata);
+    nodesx->pdata=NULL;
+   }
+
+ if(waysx->cdata)
+   {
+    free(waysx->cdata);
+    waysx->cdata=NULL;
+   }
+
  /* Print the final message */
 
  printf_last("Indexed Segments: Segments=%"Pindex_t,segmentsx->number);
@@ -773,90 +848,287 @@ void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx)
 
 
 /*++++++++++++++++++++++++++++++++++++++
-  Update the segment indexes after geographical sorting.
+  Prune the deleted segments while resorting the list.
 
-  SegmentsX *segmentsx The set of segments to modify.
+  SegmentsX *segmentsx The set of segments to sort and modify.
 
-  NodesX *nodesx The set of nodes to use.
-
-  WaysX *waysx The set of ways to use.
+  WaysX *waysx The set of ways to check.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void UpdateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx)
+void RemovePrunedSegments(SegmentsX *segmentsx,WaysX *waysx)
 {
- index_t i;
  int fd;
+ index_t xnumber;
+
+ if(segmentsx->number==0)
+    return;
 
  /* Print the start message */
 
- printf_first("Updating Segments: Segments=0");
+ printf_first("Sorting and Pruning Segments");
 
- /* Map into memory / open the files */
+ /* Allocate the way usage bitmask */
 
-#if !SLIM
- waysx->data=MapFile(waysx->filename);
-#else
- waysx->fd=ReOpenFile(waysx->filename);
-#endif
+ segmentsx->usedway=AllocBitMask(waysx->number);
+
+ logassert(segmentsx->usedway,"Failed to allocate memory (try using slim mode?)"); /* Check AllocBitMask() worked */
 
  /* Re-open the file read-only and a new file writeable */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 
- DeleteFile(segmentsx->filename);
+ DeleteFile(segmentsx->filename_tmp);
 
- fd=OpenFileNew(segmentsx->filename);
+ fd=OpenFileNew(segmentsx->filename_tmp);
 
- /* Modify the on-disk image */
+ /* Sort by node indexes */
 
- for(i=0;i<segmentsx->number;i++)
-   {
-    SegmentX segmentx;
-    WayX *wayx;
+ xnumber=segmentsx->number;
 
-    ReadFile(segmentsx->fd,&segmentx,sizeof(SegmentX));
+ sortsegmentsx=segmentsx;
 
-    segmentx.node1=nodesx->gdata[segmentx.node1];
-    segmentx.node2=nodesx->gdata[segmentx.node2];
-
-    if(segmentx.node1>segmentx.node2)
-      {
-       index_t temp;
-
-       temp=segmentx.node1;
-       segmentx.node1=segmentx.node2;
-       segmentx.node2=temp;
-
-       if(segmentx.distance&(ONEWAY_2TO1|ONEWAY_1TO2))
-          segmentx.distance^=ONEWAY_2TO1|ONEWAY_1TO2;
-      }
-
-    wayx=LookupWayX(waysx,segmentx.way,1);
-
-    segmentx.way=wayx->prop;
-
-    WriteFile(fd,&segmentx,sizeof(SegmentX));
-
-    if(!((i+1)%10000))
-       printf_middle("Updating Segments: Segments=%"Pindex_t,i+1);
-   }
+ segmentsx->number=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(void*,index_t))delete_pruned,
+                                                                    (int (*)(const void*,const void*))sort_by_id,
+                                                                    NULL);
 
  /* Close the files */
 
  segmentsx->fd=CloseFile(segmentsx->fd);
  CloseFile(fd);
 
- /* Unmap from memory / close the files */
+ /* Print the final message */
+
+ printf_last("Sorted and Pruned Segments: Segments=%"Pindex_t" Deleted=%"Pindex_t,xnumber,xnumber-segmentsx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Delete the pruned segments.
+
+  int delete_pruned Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended segment.
+
+  index_t index The number of unsorted segments that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int delete_pruned(SegmentX *segmentx,index_t index)
+{
+ if(IsPrunedSegmentX(segmentx))
+    return(0);
+
+ SetBit(sortsegmentsx->usedway,segmentx->way);
+
+ return(1);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Remove the duplicate super-segments.
+
+  SegmentsX *segmentsx The set of super-segments to modify.
+
+  WaysX *waysx The set of ways to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void DeduplicateSuperSegments(SegmentsX *segmentsx,WaysX *waysx)
+{
+ int fd;
+ index_t xnumber;
+
+ if(waysx->number==0)
+    return;
+
+ /* Print the start message */
+
+ printf_first("Sorting and Deduplicating Super-Segments");
+
+ /* Map into memory / open the file */
 
 #if !SLIM
- waysx->data=UnmapFile(waysx->filename);
+ waysx->data=MapFile(waysx->filename_tmp);
+#else
+ waysx->fd=ReOpenFile(waysx->filename_tmp);
+#endif
+
+ /* Re-open the file read-only and a new file writeable */
+
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+
+ DeleteFile(segmentsx->filename_tmp);
+
+ fd=OpenFileNew(segmentsx->filename_tmp);
+
+ /* Sort by node indexes */
+
+ xnumber=segmentsx->number;
+
+ sortsegmentsx=segmentsx;
+ sortwaysx=waysx;
+
+ segmentsx->number=filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),NULL,
+                                                                    (int (*)(const void*,const void*))sort_by_id,
+                                                                    (int (*)(void*,index_t))deduplicate_super);
+
+ /* Close the files */
+
+ segmentsx->fd=CloseFile(segmentsx->fd);
+ CloseFile(fd);
+
+ /* Unmap from memory / close the file */
+
+#if !SLIM
+ waysx->data=UnmapFile(waysx->data);
 #else
  waysx->fd=CloseFile(waysx->fd);
 #endif
 
  /* Print the final message */
 
- printf_last("Updated Segments: Segments=%"Pindex_t,segmentsx->number);
+ printf_last("Sorted and Deduplicated Super-Segments: Super-Segments=%"Pindex_t" Duplicate=%"Pindex_t,xnumber,xnumber-segmentsx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  De-duplicate super-segments.
+
+  int deduplicate_super Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended super-segment.
+
+  index_t index The number of sorted super-segments that have already been written to the output file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int deduplicate_super(SegmentX *segmentx,index_t index)
+{
+ static int nprev=0;
+ static index_t prevnode1=NO_NODE,prevnode2=NO_NODE;
+ static SegmentX prevsegx[MAX_SEG_PER_NODE];
+ static Way prevway[MAX_SEG_PER_NODE];
+
+ WayX *wayx=LookupWayX(sortwaysx,segmentx->way,1);
+ int isduplicate=0;
+
+ if(index==0 || segmentx->node1!=prevnode1 || segmentx->node2!=prevnode2)
+   {
+    nprev=1;
+    prevnode1=segmentx->node1;
+    prevnode2=segmentx->node2;
+    prevsegx[0]=*segmentx;
+    prevway[0] =wayx->way;
+   }
+ else
+   {
+    int offset;
+
+    for(offset=0;offset<nprev;offset++)
+      {
+       if(DISTFLAG(segmentx->distance)==DISTFLAG(prevsegx[offset].distance))
+          if(!WaysCompare(&prevway[offset],&wayx->way))
+            {
+             isduplicate=1;
+             break;
+            }
+      }
+
+    if(isduplicate)
+      {
+       nprev--;
+
+       for(;offset<nprev;offset++)
+         {
+          prevsegx[offset]=prevsegx[offset+1];
+          prevway[offset] =prevway[offset+1];
+         }
+      }
+    else
+      {
+       logassert(nprev<MAX_SEG_PER_NODE,"Too many segments for one node (increase MAX_SEG_PER_NODE?)"); /* Only a limited amount of information stored. */
+
+       prevsegx[nprev]=*segmentx;
+       prevway[nprev] =wayx->way;
+
+       nprev++;
+      }
+   }
+
+ return(!isduplicate);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Sort the segments geographically after updating the node indexes.
+
+  SegmentsX *segmentsx The set of segments to modify.
+
+  NodesX *nodesx The set of nodes to use.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void SortSegmentListGeographically(SegmentsX *segmentsx,NodesX *nodesx)
+{
+ int fd;
+
+ if(segmentsx->number==0)
+    return;
+
+ /* Print the start message */
+
+ printf_first("Sorting Segments Geographically");
+
+ /* Re-open the file read-only and a new file writeable */
+
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
+
+ DeleteFile(segmentsx->filename_tmp);
+
+ fd=OpenFileNew(segmentsx->filename_tmp);
+
+ /* Update the segments with geographically sorted node indexes and sort them */
+
+ sortnodesx=nodesx;
+
+ filesort_fixed(segmentsx->fd,fd,sizeof(SegmentX),(int (*)(void*,index_t))geographically_index,
+                                                  (int (*)(const void*,const void*))sort_by_id,
+                                                  NULL);
+ /* Close the files */
+
+ segmentsx->fd=CloseFile(segmentsx->fd);
+ CloseFile(fd);
+
+ /* Print the final message */
+
+ printf_last("Sorted Segments Geographically: Segments=%"Pindex_t,segmentsx->number);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Update the segment indexes.
+
+  int geographically_index Return 1 if the value is to be kept, otherwise 0.
+
+  SegmentX *segmentx The extended segment.
+
+  index_t index The number of unsorted segments that have been read from the input file.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static int geographically_index(SegmentX *segmentx,index_t index)
+{
+ segmentx->node1=sortnodesx->gdata[segmentx->node1];
+ segmentx->node2=sortnodesx->gdata[segmentx->node2];
+
+ if(segmentx->node1>segmentx->node2)
+   {
+    index_t temp;
+
+    temp=segmentx->node1;
+    segmentx->node1=segmentx->node2;
+    segmentx->node2=temp;
+
+    if(segmentx->distance&(ONEWAY_2TO1|ONEWAY_1TO2))
+       segmentx->distance^=ONEWAY_2TO1|ONEWAY_1TO2;
+   }
+
+ return(1);
 }
 
 
@@ -881,7 +1153,7 @@ void SaveSegmentList(SegmentsX *segmentsx,const char *filename)
 
  /* Re-open the file */
 
- segmentsx->fd=ReOpenFile(segmentsx->filename);
+ segmentsx->fd=ReOpenFile(segmentsx->filename_tmp);
 
  /* Write out the segments data */
 
