@@ -52,8 +52,10 @@ struct _SegmentX
 /*+ A structure containing a set of segments (memory format). +*/
 struct _SegmentsX
 {
- char      *filename;           /*+ The name of the temporary file. +*/
- int        fd;                 /*+ The file descriptor of the temporary file. +*/
+ char      *filename;           /*+ The name of the intermediate file (for the SegmentsX). +*/
+ char      *filename_tmp;       /*+ The name of the temporary file (for the SegmentsX). +*/
+
+ int        fd;                 /*+ The file descriptor of the open file (for the SegmentsX). +*/
 
  index_t    number;             /*+ The number of extended segments still being considered. +*/
 
@@ -73,35 +75,39 @@ struct _SegmentsX
  index_t   *next1;              /*+ The index of the next segment with the same node1 (used while pruning). +*/
 
  BitMask   *usednode;           /*+ A flag to indicate if a node is used (used for removing bad segments). +*/
+
+ BitMask   *usedway;            /*+ A flag to indicate if a way is used (used for removing pruned ways). +*/
 };
 
 
 /* Functions in segmentsx.c */
 
-
-SegmentsX *NewSegmentList(int append);
+SegmentsX *NewSegmentList(int append,int readonly);
 void FreeSegmentList(SegmentsX *segmentsx,int keep);
 
-void SaveSegmentList(SegmentsX *segmentsx,const char *filename);
+void AppendSegmentList(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,distance_t distance);
+void FinishSegmentList(SegmentsX *segmentsx);
 
 SegmentX *FirstSegmentX(SegmentsX *segmentsx,index_t nodeindex,int position);
 SegmentX *NextSegmentX(SegmentsX *segmentsx,SegmentX *segmentx,index_t nodeindex);
 
-void AppendSegment(SegmentsX *segmentsx,way_t way,node_t node1,node_t node2,distance_t distance);
+void ApplySegmentChanges(SegmentsX *segmentsx);
 
-void SortSegmentList(SegmentsX *segmentsx,int delete);
+void SortSegmentList(SegmentsX *segmentsx);
 
-void RemoveBadSegments(NodesX *nodesx,SegmentsX *segmentsx);
+void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
+
+void RemoveBadSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx,int keep);
 
 void MeasureSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
 
-void DeduplicateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
+void RemovePrunedSegments(SegmentsX *segmentsx,WaysX *waysx);
 
-void CreateRealSegments(SegmentsX *segmentsx,WaysX *waysx);
+void DeduplicateSuperSegments(SegmentsX *segmentsx,WaysX *waysx);
 
-void IndexSegments(SegmentsX *segmentsx,NodesX *nodesx);
+void SortSegmentListGeographically(SegmentsX *segmentsx,NodesX *nodesx);
 
-void UpdateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
+void SaveSegmentList(SegmentsX *segmentsx,const char *filename);
 
 
 /* Macros / inline functions */
@@ -117,6 +123,8 @@ void UpdateSegments(SegmentsX *segmentsx,NodesX *nodesx,WaysX *waysx);
 #define IndexSegmentX(segmentsx,segmentx)                (index_t)((segmentx)-&(segmentsx)->data[0])
 
 #define PutBackSegmentX(segmentsx,segmentx)              /* nop */
+
+#define ReLookupSegmentX(segmentsx,segmentx)             /* nop */
   
 #else
 
@@ -126,6 +134,8 @@ static index_t IndexSegmentX(SegmentsX *segmentsx,SegmentX *segmentx);
 
 static void PutBackSegmentX(SegmentsX *segmentsx,SegmentX *segmentx);
 
+static void ReLookupSegmentX(SegmentsX *segmentsx,SegmentX *segmentx);
+
 
 /*++++++++++++++++++++++++++++++++++++++
   Lookup a particular extended segment with the specified id from the file on disk.
@@ -182,6 +192,22 @@ static inline void PutBackSegmentX(SegmentsX *segmentsx,SegmentX *segmentx)
  SeekWriteFile(segmentsx->fd,&segmentsx->cached[position1],sizeof(SegmentX),(off_t)segmentsx->incache[position1]*sizeof(SegmentX));
 }
 
+
+/*++++++++++++++++++++++++++++++++++++++
+  Lookup an extended segment's data from the disk into file again after the disk was updated.
+
+  SegmentsX *segmentsx The set of segments to use.
+
+  SegmentX *segmentx The extended segment to refresh.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static inline void ReLookupSegmentX(SegmentsX *segmentsx,SegmentX *segmentx)
+{
+ int position1=segmentx-&segmentsx->cached[0];
+
+ SeekReadFile(segmentsx->fd,&segmentsx->cached[position1],sizeof(SegmentX),(off_t)segmentsx->incache[position1]*sizeof(SegmentX));
+}
+
 #endif /* SLIM */
 
 
