@@ -33,13 +33,13 @@
 /* Constants and macros for handling them */
 
 /*+ An undefined node ID. +*/
-#define NO_NODE_ID     (~(node_t)0)
+#define NO_NODE_ID     ((node_t)~0)
 
 /*+ An undefined way ID. +*/
-#define NO_WAY_ID      (~(way_t)0)
+#define NO_WAY_ID      ((way_t)~0)
 
 /*+ An undefined relation ID. +*/
-#define NO_RELATION_ID (~(relation_t)0)
+#define NO_RELATION_ID ((relation_t)~0)
 
 /*+ The maximum number of segments per node (used to size temporary storage). +*/
 #define MAX_SEG_PER_NODE 32
@@ -49,14 +49,15 @@
 
 #define BitMask uint32_t
 
-#define AllocBitMask(xx)   (BitMask*)calloc((1+(xx)/32),sizeof(BitMask))
+#define AllocBitMask(xx)    (BitMask*)calloc((1+(xx)/32),sizeof(BitMask))
 
-#define SetAllBits0(xx,yy) memset((xx), 0,(1+(yy)/32)*sizeof(BitMask))
-#define SetAllBits1(xx,yy) memset((xx),~0,(1+(yy)/32)*sizeof(BitMask))
+#define ClearAllBits(xx,yy) memset((xx), 0,(1+(yy)/32)*sizeof(BitMask))
+#define SetAllBits(xx,yy)   memset((xx),~0,(1+(yy)/32)*sizeof(BitMask))
 
-#define ClearBit(xx,yy)    (xx)[(yy)/32]&=~(((BitMask)1)<<((yy)%32))
-#define SetBit(xx,yy)      (xx)[(yy)/32]|= (((BitMask)1)<<((yy)%32))
-#define IsBitSet(xx,yy)   ((xx)[(yy)/32]&  (((BitMask)1)<<((yy)%32)))
+#define ClearBit(xx,yy)     (xx)[(yy)/32]&=~(((BitMask)1)<<((yy)%32))
+#define SetBit(xx,yy)       (xx)[(yy)/32]|= (((BitMask)1)<<((yy)%32))
+
+#define IsBitSet(xx,yy)    ((xx)[(yy)/32]&  (((BitMask)1)<<((yy)%32)))
 
 
 /* Simple Types */
@@ -114,7 +115,7 @@ typedef struct _WaysX WaysX;
 
 typedef struct _RouteRelX RouteRelX;
 
-typedef struct _TurnRestrictRelX TurnRestrictRelX;
+typedef struct _TurnRelX TurnRelX;
 
 typedef struct _RelationsX RelationsX;
 
