@@ -22,7 +22,8 @@ var mapprops={ // contains all properties for the map to be displayed.
 
  // EDIT THIS above to change the visible map limits
 
- // EDIT THIS below to change the map URL(s) 
+
+ // EDIT THIS below to change the map URL(s)
 
     mapdata: [
         {
@@ -30,8 +31,16 @@ var mapprops={ // contains all properties for the map to be displayed.
             baseurl:  "http://tile.openstreetmap.org/",
             errorurl: "http://openstreetmap.org/openlayers/img/404.png"
         }
-    ]
+    ],
 
- // EDIT THIS above to change the map URL(s) 
+ // EDIT THIS above to change the map URL(s)
+
+
+ // EDIT THIS below to change the maximum number of markers
+
+ // The number of waypoints to include in the HTML
+    maxmarkers: 9
+
+ // EDIT THIS above to change the maximum number of markers
 
 }; // end of map properties
