@@ -3,7 +3,7 @@
 //
 // Part of the Routino routing software.
 //
-// This file Copyright 2008-2011 Andrew M. Bishop
+// This file Copyright 2008-2012 Andrew M. Bishop
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Affero General Public License as published by
@@ -28,6 +28,9 @@ var data_types=[
                 "junctions",
                 "super",
                 "oneway",
+                "highway",
+                "transport",
+                "barrier",
                 "turns",
                 "speed",
                 "weight",
@@ -74,6 +77,42 @@ var hex={0: "00", 1: "11",  2: "22",  3: "33",  4: "44",  5: "55",  6: "66",  7:
 var turn_restriction_style;
 
 
+////////////////////////////////////////////////////////////////////////////////
+/////////////////////////////// Initialisation /////////////////////////////////
+////////////////////////////////////////////////////////////////////////////////
+
+// Process the URL query string and extract the arguments
+
+var legal={"^lon"  : "^[-0-9.]+$",
+           "^lat"  : "^[-0-9.]+$",
+           "^zoom" : "^[0-9]+$"};
+
+var args={};
+
+if(location.search.length>1)
+  {
+   var query,queries;
+
+   query=location.search.replace(/^\?/,"");
+   query=query.replace(/;/g,'&');
+   queries=query.split('&');
+
+   for(var i=0;i<queries.length;i++)
+     {
+      queries[i].match(/^([^=]+)(=(.*))?$/);
+
+      k=RegExp.$1;
+      v=unescape(RegExp.$3);
+
+      for(var l in legal)
+        {
+         if(k.match(RegExp(l)) && v.match(RegExp(legal[l])))
+            args[k]=v;
+        }
+     }
+  }
+
+
 ////////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////// Map handling /////////////////////////////////
 ////////////////////////////////////////////////////////////////////////////////
@@ -81,7 +120,6 @@ var turn_restriction_style;
 var map;
 var layerMap=[], layerVectors, layerBoxes;
 var epsg4326, epsg900913;
-var map_args;
 
 var box;
 
@@ -89,8 +127,12 @@ var box;
 // Initialise the 'map' object
 //
 
-function map_init(lat,lon,zoom)
+function map_init()             // called from visualiser.html
 {
+ lon =args["lon"];
+ lat =args["lat"];
+ zoom=args["zoom"];
+
  // Map properties (North/South and East/West limits and zoom in/out limits) are now in mapprops.js
  // Map URLs are now in mapprops.js
 
@@ -115,16 +157,15 @@ function map_init(lat,lon,zoom)
 
                             minZoomLevel: mapprops.zoomout,
                             numZoomLevels: mapprops.zoomin-mapprops.zoomout+1,
-                            maxResolution: 156543.0339 / Math.pow(2,mapprops.zoomout),
+                            maxResolution: 156543.03390625 / Math.pow(2,mapprops.zoomout),
 
+                            // These two lines are not needed with OpenLayers 2.12
+                            units: "m",
                             maxExtent:        new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
-                            restrictedExtent: new OpenLayers.Bounds(mapprops.westedge,mapprops.southedge,mapprops.eastedge,mapprops.northedge).transform(epsg4326,epsg900913),
 
-                            units: "m"
+                            restrictedExtent: new OpenLayers.Bounds(mapprops.westedge,mapprops.southedge,mapprops.eastedge,mapprops.northedge).transform(epsg4326,epsg900913)
                            });
 
- map.events.register("moveend", map, mapMoved);
-
  // Add map tile layers
 
  for(var l=0;l < mapprops.mapdata.length;l++)
@@ -193,7 +234,7 @@ function map_init(lat,lon,zoom)
 
  // Move the map
 
- if(lon != 'lon' && lat != 'lat' && zoom != 'zoom')
+ if(lon != undefined && lat != undefined && zoom != undefined)
    {
     if(lon<mapprops.westedge) lon=mapprops.westedge;
     if(lon>mapprops.eastedge) lon=mapprops.eastedge;
@@ -204,7 +245,8 @@ function map_init(lat,lon,zoom)
     if(zoom<mapprops.zoomout) zoom=mapprops.zoomout;
     if(zoom>mapprops.zoomin)  zoom=mapprops.zoomin;
 
-    var lonlat = new OpenLayers.LonLat(lon,lat).transform(epsg4326,map.getProjectionObject());
+    var lonlat = new OpenLayers.LonLat(lon,lat);
+    lonlat.transform(epsg4326,epsg900913);
 
     map.moveTo(lonlat,zoom-map.minZoomLevel);
    }
@@ -212,36 +254,61 @@ function map_init(lat,lon,zoom)
 
 
 //
-// Map has moved
+// Format a number in printf("%.5f") format.
 //
 
-function mapMoved()
+function format5f(number)
 {
- var centre = map.getCenter().clone();
+ var newnumber=Math.floor(number*100000+0.5);
+ var delta=0;
 
- var lonlat = centre.transform(map.getProjectionObject(),epsg4326);
+ if(newnumber>=0 && newnumber<100000) delta= 100000;
+ if(newnumber<0 && newnumber>-100000) delta=-100000;
 
- var zoom = this.getZoom() + map.minZoomLevel;
+ var string=String(newnumber+delta);
 
- map_args="lat=" + lonlat.lat + ";lon=" + lonlat.lon + ";zoom=" + zoom;
+ var intpart =string.substring(0,string.length-5);
+ var fracpart=string.substring(string.length-5,string.length);
 
- updateCustomURL();
+ if(delta>0) intpart="0";
+ if(delta<0) intpart="-0";
+
+ return(intpart + "." + fracpart);
 }
 
 
 //
-// Update custom URL
+// Build a set of URL arguments for the map location
 //
 
-function updateCustomURL()
+function buildMapArguments()
 {
- var router_url=document.getElementById("router_url");
- var link_url  =document.getElementById("link_url");
- var edit_url  =document.getElementById("edit_url");
+ var lonlat = map.getCenter().clone();
+ lonlat.transform(epsg900913,epsg4326);
 
- router_url.href="customrouter.cgi?" + map_args;
- link_url.href="customvisualiser.cgi?" + map_args;
- edit_url.href="http://www.openstreetmap.org/edit?" + map_args;
+ var zoom = map.getZoom() + map.minZoomLevel;
+
+ return "lat=" + format5f(lonlat.lat) + ";lon=" + format5f(lonlat.lon) + ";zoom=" + zoom;
+}
+
+
+//
+// Update a URL
+//
+
+function updateURL(element)     // called from visualiser.html
+{
+ if(element.id == "permalink_url")
+    element.href=location.pathname + "?" + buildMapArguments();
+
+ if(element.id == "router_url")
+    element.href="router.html" + "?" + buildMapArguments();
+
+ if(element.id == "edit_url")
+    element.href="http://www.openstreetmap.org/edit" + "?" + buildMapArguments();
+
+ if(element.id.match(/^lang_([a-zA-Z-]+)_url$/))
+    element.href="visualiser.html" + "." + RegExp.$1 + "?" + buildMapArguments();
 }
 
 
@@ -255,9 +322,7 @@ function updateCustomURL()
 
 function displayStatus(type,subtype,content)
 {
- var div_status=document.getElementById("result_status");
-
- var child=div_status.firstChild;
+ var child=document.getElementById("result_status").firstChild;
 
  do
    {
@@ -289,7 +354,7 @@ function displayStatistics()
 {
  // Use AJAX to get the statistics
 
- OpenLayers.loadURL("statistics.cgi",null,null,runStatisticsSuccess);
+ OpenLayers.Request.GET({url: "statistics.cgi", success: runStatisticsSuccess});
 }
 
 
@@ -299,12 +364,8 @@ function displayStatistics()
 
 function runStatisticsSuccess(response)
 {
- var statistics_data=document.getElementById("statistics_data");
- var statistics_link=document.getElementById("statistics_link");
-
- statistics_data.innerHTML="<pre>" + response.responseText + "</pre>";
-
- statistics_link.style.display="none";
+ document.getElementById("statistics_data").innerHTML="<pre>" + response.responseText + "</pre>";
+ document.getElementById("statistics_link").style.display="none";
 }
 
 
@@ -312,7 +373,7 @@ function runStatisticsSuccess(response)
 // Get the requested data
 //
 
-function displayData(datatype)
+function displayData(datatype)  // called from visualiser.html
 {
  for(var data in data_types)
     hideshow_hide(data_types[data]);
@@ -355,23 +416,47 @@ function displayData(datatype)
  switch(datatype)
    {
    case 'junctions':
-    OpenLayers.loadURL(url,null,null,runJunctionsSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runJunctionsSuccess, failure: runFailure});
     break;
    case 'super':
-    OpenLayers.loadURL(url,null,null,runSuperSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runSuperSuccess, faliure: runFailure});
     break;
    case 'oneway':
-    OpenLayers.loadURL(url,null,null,runOnewaySuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runOnewaySuccess, failure: runFailure});
+    break;
+   case 'highway':
+    var highways=document.forms["highways"].elements["highway"];
+    for(var h in highways)
+       if(highways[h].checked)
+          highway=highways[h].value;
+    url+="-" + highway;
+    OpenLayers.Request.GET({url: url, success: runHighwaySuccess, falure: runFailure});
+    break;
+   case 'transport':
+    var transports=document.forms["transports"].elements["transport"];
+    for(var t in transports)
+       if(transports[t].checked)
+          transport=transports[t].value;
+    url+="-" + transport;
+    OpenLayers.Request.GET({url: url, success: runTransportSuccess, failure: runFailure});
+    break;
+   case 'barrier':
+    var transports=document.forms["barriers"].elements["barrier"];
+    for(var t in transports)
+       if(transports[t].checked)
+          transport=transports[t].value;
+    url+="-" + transport;
+    OpenLayers.Request.GET({url: url, success: runBarrierSuccess, failure: runFailure});
     break;
    case 'turns':
-    OpenLayers.loadURL(url,null,null,runTurnsSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runTurnsSuccess, failure: runFailure});
     break;
    case 'speed':
    case 'weight':
    case 'height':
    case 'width':
    case 'length':
-    OpenLayers.loadURL(url,null,null,runLimitSuccess,runFailure);
+    OpenLayers.Request.GET({url: url, success: runLimitSuccess, failure: runFailure});
     break;
    }
 }
@@ -398,7 +483,7 @@ function runJunctionsSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -447,7 +532,7 @@ function runSuperSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -505,7 +590,7 @@ function runOnewaySuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -551,6 +636,156 @@ function runOnewaySuccess(response)
 }
 
 
+//
+// Success in getting the highway data
+//
+
+function runHighwaySuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+
+       var segment = new OpenLayers.Geometry.LineString([point1,point2]);
+
+       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","highway",lines.length-2);
+}
+
+
+//
+// Success in getting the transport data
+//
+
+function runTransportSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var lonlat1= new OpenLayers.LonLat(lon1,lat1).transform(epsg4326,epsg900913);
+       var lonlat2= new OpenLayers.LonLat(lon2,lat2).transform(epsg4326,epsg900913);
+
+       var point1 = new OpenLayers.Geometry.Point(lonlat1.lon,lonlat1.lat);
+       var point2 = new OpenLayers.Geometry.Point(lonlat2.lon,lonlat2.lat);
+
+       var segment = new OpenLayers.Geometry.LineString([point1,point2]);
+
+       features.push(new OpenLayers.Feature.Vector(segment,{},super_segment_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","transport",lines.length-2);
+}
+
+
+//
+// Success in getting the barrier data
+//
+
+function runBarrierSuccess(response)
+{
+ var lines=response.responseText.split('\n');
+
+ var features=[];
+
+ for(var line=0;line<lines.length;line++)
+   {
+    var words=lines[line].split(' ');
+
+    if(line == 0)
+      {
+       var lat1=words[0];
+       var lon1=words[1];
+       var lat2=words[2];
+       var lon2=words[3];
+
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
+
+       box = new OpenLayers.Marker.Box(bounds);
+
+       layerBoxes.addMarker(box);
+      }
+    else if(words[0] != "")
+      {
+       var lat=words[0];
+       var lon=words[1];
+
+       var lonlat= new OpenLayers.LonLat(lon,lat).transform(epsg4326,epsg900913);
+
+       var point = new OpenLayers.Geometry.Point(lonlat.lon,lonlat.lat);
+
+       features.push(new OpenLayers.Feature.Vector(point,{},super_node_style));
+      }
+   }
+
+ layerVectors.addFeatures(features);
+
+ displayStatus("data","barrier",lines.length-2);
+}
+
+
 //
 // Success in getting the turn restrictions data
 //
@@ -572,7 +807,7 @@ function runTurnsSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
@@ -630,7 +865,7 @@ function runLimitSuccess(response)
        var lat2=words[2];
        var lon2=words[3];
 
-       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,map.getProjectionObject());
+       var bounds = new OpenLayers.Bounds(lon1,lat1,lon2,lat2).transform(epsg4326,epsg900913);
 
        box = new OpenLayers.Marker.Box(bounds);
 
